// -*- tab-width:4; c-basic-offset:2; -*-

#include <errno.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#define NOMINMAX
#include <windows.h>
#include <shlwapi.h>

// windows.h defines this to either LoadImageA or LoadImageW.
// We want the function in image.dll.
#undef LoadImage

#undef PACKED	// (unconditionally!) redefined within manalink.h
#include "manalink.h"

#include "gdiplus.h"

#define MAX(a, b) ((a) > (b) ? (a) : (b))
#define MIN(a, b) ((a) < (b) ? (a) : (b))
#define EXE_DWORD(addr) (*((int*)(addr)))

typedef enum
{
  // low byte: same as color_test_t
  COUNT_COLOR_BLACK		= 0x0002,
  COUNT_COLOR_BLUE		= 0x0004,
  COUNT_COLOR_GREEN		= 0x0008,
  COUNT_COLOR_RED		= 0x0010,
  COUNT_COLOR_WHITE		= 0x0020,

  COUNT_COLOR_COLOR_MASK= COUNT_COLOR_BLACK | COUNT_COLOR_BLUE | COUNT_COLOR_GREEN | COUNT_COLOR_RED | COUNT_COLOR_WHITE,

  // byte 2: presence of hybrid and/or mono-colored mana
  COUNT_COLOR_HYBRID	= 0x000100,	// set if there's any two-color hybrid in the mana cost
  COUNT_COLOR_MONO		= 0x000200,	// set if there's any single-color mana in the mana cost: this forces a gold frame
} CountColors;

#define MANA_SYMBOLS 58
signed char mana_tags[MANA_SYMBOLS * 3 + 1] =
{
#define MANA_TAG_HYBRID_MAX -33
  -33, 'B', 'U',	// 2 letter tags must be checked before single
  -34, 'B', 'G',
  -35, 'B', 'R',
  -36, 'B', 'W',
  -37, 'U', 'B',
  -38, 'U', 'G',
  -39, 'U', 'R',
  -40, 'U', 'W',
  -41, 'G', 'B',
  -42, 'G', 'U',
  -43, 'G', 'R',
  -44, 'G', 'W',
  -45, 'R', 'B',
  -46, 'R', 'U',
  -47, 'R', 'G',
  -48, 'R', 'W',
  -49, 'W', 'B',
  -50, 'W', 'U',
  -51, 'W', 'G',
  -52, 'W', 'R',
#define MANA_TAG_HYBRID_MIN -52

#define MANA_TAG_TWO_NUMBER_MAX -12
  -12, '1', '0',
  -13, '1', '1',
  -14, '1', '2',
  -15, '1', '3',
  -16, '1', '4',
  -17, '1', '5',
  -18, '1', '6',
#define MANA_TAG_TWO_NUMBER_MIN -18

#define MANA_TAG_MONO_HYBRID_MAX -53
  -53, '2', 'B',
  -54, '2', 'U',
  -55, '2', 'G',
  -56, '2', 'R',
  -57, '2', 'W',
#define MANA_TAG_MONO_HYBRID_MIN -57

#define MANA_TAG_PHYREXIAN_MAX -21
  -21, 'P', 'B',
  -22, 'P', 'U',
  -23, 'P', 'G',
  -24, 'P', 'R',
  -25, 'P', 'W',
#define MANA_TAG_PHYREXIAN_MIN -25

#define MANA_TAG_X -1
   -1, 'X', 0,

#define MANA_TAG_ONE_NUMBER_MAX -2
   -2, '0', 0,
   -3, '1', 0,
   -4, '2', 0,
   -5, '3', 0,
   -6, '4', 0,
   -7, '5', 0,
   -8, '6', 0,
   -9, '7', 0,
  -10, '8', 0,
  -11, '9', 0,
#define MANA_TAG_ONE_NUMBER_MIN -11

#define MANA_TAG_SNOW -19
  -19, 'I', 0,

#define MANA_TAG_CHAOS -20
  -20, 'C', 0,

#define MANA_TAG_COLOR_MAX -26
  -26, 'B', 0,
  -27, 'U', 0,
  -28, 'G', 0,
  -29, 'R', 0,
  -30, 'W', 0,
#define MANA_TAG_COLOR_MIN -30

#define MANA_TAG_TAP -31
  -31, 'T', 0,

#define MANA_TAG_UNTAP -32
  -32, 'Q', 0,

#define MANA_TAG_OR -58
  -58, '/', 0,
  0
};

CountColors mana_tag_to_count_colors[255] = { 0 };

typedef enum
{
  CP_COLOR_LESS = 0,
  CP_COLOR_BLACK,
  CP_COLOR_BLUE,
  CP_COLOR_ARTIFACT,
  CP_COLOR_MULTI,
  CP_COLOR_GREEN,
  CP_COLOR_LAND,
  CP_COLOR_RED,
  CP_COLOR_WHITE,
  CP_COLOR_SPECIAL = -1
} CardPtrClr;

typedef enum
{
  CP_TYPE_NONE = 0,
  CP_TYPE_ARTIFACT,
  CP_TYPE_ENCHANTMENT,
  CP_TYPE_INSTANT,
  CP_TYPE_INTERRUPT,
  CP_TYPE_LAND,
  CP_TYPE_SORCERY,
  CP_TYPE_CREATURE,
  CP_TYPE_TOKEN,
  CP_TYPE_PLANESWALKER,	// unused
  CP_TYPE_SCHEME,		// unused
  CP_TYPE_PLANE			// unused
} CardPtrType;

#define FONTSTXT		"Fonts"
#define EXPANSIONSTXT	"Expansions"
#define FONT1NAME		"\\TT0530m_.ttf"
#define FONT3NAME		"\\TT0127m_.ttf"
#define FONT4NAME		"\\TT0085m_.ttf"
#define FONT5NAME		"\\TT0298m_.ttf"
#define FONT6NAME		"\\TT0299m_.ttf"
#define FONT7NAME		"\\TT0300m_.ttf"

typedef enum
{
  CARDBACK = 0,
  MANASYMBOLS,
  EXPANSION_SYMBOLS,
  WATERMARKS,

#define FRAMEPARTS(frameset)																															\
  CARDBK_##frameset##_COLORLESS,	CARDBK_##frameset##_ARTIFACT,	CARDBK_##frameset##_LAND,	CARDBK_##frameset##_GOLD,	CARDBK_##frameset##_SPECIAL,\
  CARDBK_##frameset##_BLACK,		CARDBK_##frameset##_BLUE,		CARDBK_##frameset##_GREEN,	CARDBK_##frameset##_RED,	CARDBK_##frameset##_WHITE,	\
																																						\
  CARDOV_##frameset##_COLORLESS,		CARDOV_##frameset##_ARTIFACT,		CARDOV_##frameset##_COLORLESS_LAND,	CARDOV_##frameset##_LAND_BAR,			\
  CARDOV_##frameset##_GOLD,				CARDOV_##frameset##_GOLD_ARTIFACT,	CARDOV_##frameset##_GOLD_LAND,		CARDOV_##frameset##_GOLD_BAR,			\
  CARDOV_##frameset##_COLORLESS_NYX,	CARDOV_##frameset##_ARTIFACT_NYX,	CARDOV_##frameset##_LAND_NYX,		CARDOV_##frameset##_GOLD_NYX,			\
																																						\
  CARDOV_##frameset##_B,	CARDOV_##frameset##_BGOLD,	CARDOV_##frameset##_BLAND,	CARDOV_##frameset##_BARTIFACT,	CARDOV_##frameset##_BGOLD_ARTIFACT,	CARDOV_##frameset##_BNYX,	\
  CARDOV_##frameset##_U,	CARDOV_##frameset##_UGOLD,	CARDOV_##frameset##_ULAND,	CARDOV_##frameset##_UARTIFACT,	CARDOV_##frameset##_UGOLD_ARTIFACT, CARDOV_##frameset##_UNYX,	\
  CARDOV_##frameset##_G,	CARDOV_##frameset##_GGOLD,	CARDOV_##frameset##_GLAND,	CARDOV_##frameset##_GARTIFACT,	CARDOV_##frameset##_GGOLD_ARTIFACT, CARDOV_##frameset##_GNYX,	\
  CARDOV_##frameset##_R,	CARDOV_##frameset##_RGOLD,	CARDOV_##frameset##_RLAND,	CARDOV_##frameset##_RARTIFACT,	CARDOV_##frameset##_RGOLD_ARTIFACT, CARDOV_##frameset##_RNYX,	\
  CARDOV_##frameset##_W,	CARDOV_##frameset##_WGOLD,	CARDOV_##frameset##_WLAND,	CARDOV_##frameset##_WARTIFACT,	CARDOV_##frameset##_WGOLD_ARTIFACT, CARDOV_##frameset##_WNYX,	\
																																						\
  TOMBSTONE_##frameset,		POWERTOUGHNESS_##frameset,	TYPE_ICON_##frameset,																			\
  CARDBK_##frameset##_MIN = CARDBK_##frameset##_COLORLESS,							CARDBK_##frameset##_MAX = TYPE_ICON_##frameset

  FRAMEPARTS(MODERN),		CARDBK_MIN_FRAMEPART = CARDBK_MODERN_MIN, CARDBK_MAX_FRAMEPART_FRAMESET = CARDBK_MODERN_MAX,
#define NUM_FRAMEPARTS_PER_FRAMESET (CARDBK_MODERN_MAX - CARDBK_MODERN_MIN + 1)
  // No further entries in cfg_pic_names[] below - the frameparts for the other three framesets reuse the modern frameset's names, but in their own sections.
  FRAMEPARTS(CLASSIC),
  FRAMEPARTS(TIMESHIFT),
  FRAMEPARTS(FUTURESHIFT),
  FRAMEPARTS(PLANESWALKER),
  FRAMEPARTS(TOKEN), CARDBK_MAX_FRAMEPART_ALL = CARDBK_TOKEN_MAX,

#define BASE_FRAMEPART(framepart)	(framepart < CARDBK_MIN_FRAMEPART ? framepart : ((framepart - CARDBK_MIN_FRAMEPART) % NUM_FRAMEPARTS_PER_FRAMESET) + CARDBK_MIN_FRAMEPART)
#define FRAMESET_OF_FRAMEPART(framepart)		(framepart < CARDBK_MIN_FRAMEPART ? 0 : (framepart - CARDBK_MIN_FRAMEPART) / NUM_FRAMEPARTS_PER_FRAMESET)
#define FRAMEPART_FROM_BASE(framepart, config)	(framepart < CARDBK_MIN_FRAMEPART ? framepart : config->cardbk_base + framepart - CARDBK_MIN_FRAMEPART)

#undef FRAMEPARTS

  FRAME_MAX_LOADED = CARDBK_MAX_FRAMEPART_ALL,

  // The rest are generated, not loaded, so don't have entries in cfg_pic_names[] below.

  // Assumptions:
  // Ordering within each frameset - and thus within each colorset, and for each color among monocolor frames, and for each color pair among two-color frames - must always be the same.
  // Framesets must also be the same size.
  // Nyx framesets always have the same offset from their base frameset.

  // A set of five variants by color.
#define X_COLOR(x)		x##BLACK,x##BLUE,x##GREEN,x##RED,x##WHITE

  // A set of ten variants by color pairs.  Note that e.g. FRAME_MODERN_SPELL_GOLD is not within [FRAME_MODERN_SPELL_GOLD_MIN...FRAME_MODERN_SPELL_GOLD_MAX].
#define X_COLOR_2(x)	x##WHITE_BLUE,x##BLUE_BLACK,x##BLACK_RED,x##RED_GREEN,x##GREEN_WHITE,		\
						x##WHITE_BLACK,x##BLACK_GREEN,x##GREEN_BLUE,x##BLUE_RED,x##RED_WHITE,x##MIN = x##WHITE_BLUE, x##MAX = x##RED_WHITE

  // A set of 22 variants by all combinations of color (1*none, 5*1-color, 5*1-color-gold, 10*2-color, 1*3+-color).
#define FRAMES_PER_COLORSET	22
#define COLORSET(x)		x##COLORLESS, X_COLOR(x), X_COLOR(x##GOLD_), X_COLOR_2(x##GOLD_), x##GOLD, x##MIN = x##COLORLESS, x##MAX = x##GOLD

  // A set of 98 variants, by spell (including an extra 10 Ravnica-style 2-color hybrid, instead of the usual gold hybrid), artifact, land, and artifact land.
#define FRAMESET(x)		COLORSET(x##SPELL_), COLORSET(x##ARTIFACT_), COLORSET(x##LAND_), COLORSET(x##ARTIFACT_LAND_),	\
						X_COLOR_2(x##SPELL_HYBRID_), x##MIN = x##SPELL_COLORLESS, x##MAX = x##SPELL_HYBRID_MAX

  FRAMESET(FRAME_MODERN_),			// Basic (modern) frames
  FRAMESET(FRAME_MODERN_NYX_),		// Modern frames with Theros-style Nyx for multitype enchantments - must be done at the color level, annoyingly enough
  FRAMESET(FRAME_CLASSIC_),			// Old-style frames.
  FRAMESET(FRAME_CLASSIC_NYX_),		// Old-style frames with Theros-style Nyx for multitype enchantments
  FRAMESET(FRAME_TIMESHIFT_),		// Planar Chaos timeshifted frames.
  FRAMESET(FRAME_TIMESHIFT_NYX_),	// Planar Chaos timeshifted frames with Theros-style Nyx for multitype enchantments
  FRAMESET(FRAME_FUTURESHIFT_),		// Future Sight futureshifted frames.
  FRAMESET(FRAME_FUTURESHIFT_NYX_),	// Future Sight futureshifted frames with Theros-style Nyx for multitype enchantments
  FRAMESET(FRAME_PLANESWALKER_),	// Planeswalker frames.
  FRAMESET(FRAME_PLANESWALKER_NYX_),// Planeswalker frames with Theros-style Nyx for multitype enchantments
  FRAMESET(FRAME_TOKEN_),			// Token frames.
  FRAMESET(FRAME_TOKEN_NYX_),		// Token frames with Theros-style Nyx for multitype enchantments

  FRAME_MIN = FRAME_MODERN_MIN,
  FRAME_MAX = FRAME_TOKEN_NYX_MAX,
#define FRAMES_PER_FRAMESET	(FRAME_MODERN_NYX_MIN - FRAME_MODERN_MIN)

#define BASE_FRAME(frame)					(frame < FRAME_MIN ? frame : (((frame - FRAME_MIN) % FRAMES_PER_FRAMESET) + FRAME_MIN))
#define IS_NYX_FRAME(frame, base_frame)		((frame - base_frame) % (2 * FRAMES_PER_FRAMESET) >= FRAMES_PER_FRAMESET)

#undef X_COLOR
#undef X_COLOR_2
#undef COLORSET
#undef FRAMESET

  MAX_PICHANDLES_PLUS_1, PICHANDLE_INVALID = MAX_PICHANDLES_PLUS_1
} PicHandleNames;

const char* cfg_pic_names[CARDBK_MAX_FRAMEPART_FRAMESET + 1] =
{
  "CardBack",
  "ManaSymbols",
  "Expansion_Symbols",
  "Watermarks",

  "CardBk_Colorless",	"CardBk_Artifact",	"CardBk_Land",	"CardBk_Gold",	"CardBk_Special",
  "CardBk_Black",		"CardBk_Blue",		"CardBk_Green",	"CardBk_Red",	"CardBk_White",

  "CardOv_Colorless",		"CardOv_Artifact",		"CardOv_ColorlessLand",	"CardOv_LandBar",
  "CardOv_Gold",			"CardOv_GoldArtifact",	"CardOv_GoldLand",		"CardOv_GoldBar",
  "CardOv_ColorlessNyx",	"CardOv_ArtifactNyx",	"CardOv_LandNyx",		"CardOv_GoldNyx",

  "CardOv_B",	"CardOv_BGold",	"CardOv_BLand",	"CardOv_BArtifact",	"CardOv_BGoldArtifact",	"CardOv_BNyx",
  "CardOv_U",	"CardOv_UGold",	"CardOv_ULand",	"CardOv_UArtifact",	"CardOv_UGoldArtifact",	"CardOv_UNyx",
  "CardOv_G",	"CardOv_GGold",	"CardOv_GLand",	"CardOv_GArtifact",	"CardOv_GGoldArtifact",	"CardOv_GNyx",
  "CardOv_R",	"CardOv_RGold",	"CardOv_RLand",	"CardOv_RArtifact",	"CardOv_RGoldArtifact",	"CardOv_RNyx",
  "CardOv_W",	"CardOv_WGold",	"CardOv_WLand",	"CardOv_WArtifact",	"CardOv_WGoldArtifact",	"CardOv_WNyx",

  "Tombstone",	"PowerToughness",	"TypeIcon"
};

typedef struct
{
  WORD palVersion;
  WORD palNumEntries;
  PALETTEENTRY palPalEntry[256];
} Palette256;

Palette256 palette256 =
{
  0x300,
  256,
  {
	{  0,  0,  0,  0}, { 26, 24, 28,  0}, { 14, 18, 78,  0}, { 15, 18,109,  0},
	{ 22, 42, 79,  0}, { 20, 44,110,  0}, { 42, 23, 76,  0}, { 41, 22,110,  0},
	{ 45, 49, 76,  0}, { 44, 50,109,  0}, { 41, 84, 37,  0}, { 45, 82, 95,  0},
	{ 78, 15, 20,  0}, { 79, 18, 41,  0}, { 77, 46, 20,  0}, { 74, 50, 48,  0},
	{107, 15, 17,  0}, {108, 18, 46,  0}, {109, 47, 20,  0}, {110, 48, 44,  0},
	{ 86, 46, 86,  0}, { 92, 83, 42,  0}, { 79, 78, 78,  0}, { 76, 82,109,  0},
	{ 82,105, 83,  0}, { 81,107,112,  0}, {108, 82, 79,  0}, {109, 83,107,  0},
	{111,106, 82,  0}, {112,110,110,  0}, { 28, 36,155,  0}, { 29, 35,218,  0},
	{ 40, 88,158,  0}, { 34, 96,226,  0}, { 84, 47,154,  0}, { 85, 42,217,  0},
	{ 89,102,154,  0}, { 89,104,220,  0}, { 38,146, 40,  0}, { 39,154, 95,  0},
	{ 35,216, 42,  0}, { 34,214,100,  0}, { 95,149, 42,  0}, { 97,150,101,  0},
	{100,213, 37,  0}, { 98,209, 99,  0}, { 42,153,162,  0}, { 38,155,236,  0},
	{ 41,215,162,  0}, { 39,217,241,  0}, { 82,139,144,  0}, { 81,140,175,  0},
	{ 80,173,142,  0}, { 80,172,176,  0}, {112,140,144,  0}, {111,143,175,  0},
	{111,174,144,  0}, {114,170,177,  0}, { 80,142,207,  0}, { 78,144,243,  0},
	{ 80,172,208,  0}, { 79,175,246,  0}, {111,144,206,  0}, {110,146,241,  0},
	{113,173,207,  0}, {110,176,244,  0}, { 95,213,164,  0}, { 80,206,209,  0},
	{ 79,206,249,  0}, { 79,241,206,  0}, { 80,236,249,  0}, {113,204,208,  0},
	{112,205,248,  0}, {111,241,209,  0}, {111,237,249,  0}, {152, 40, 36,  0},
	{153, 39, 89,  0}, {144, 75, 19,  0}, {141, 79, 48,  0}, {142,111, 17,  0},
	{143,109, 49,  0}, {173, 76, 20,  0}, {173, 80, 45,  0}, {173,110, 19,  0},
	{174,111, 49,  0}, {141, 81, 78,  0}, {139, 82,108,  0}, {140,111, 82,  0},
	{139,115,110,  0}, {172, 81, 78,  0}, {173, 83,107,  0}, {173,113, 78,  0},
	{172,114,110,  0}, {217, 46, 34,  0}, {204, 49, 87,  0}, {202, 81, 18,  0},
	{203, 81, 48,  0}, {204,111, 20,  0}, {205,112, 50,  0}, {244, 78, 16,  0},
	{235, 79, 50,  0}, {234,111, 16,  0}, {234,113, 49,  0}, {212,103, 92,  0},
	{159, 46,152,  0}, {156, 49,217,  0}, {149,106,153,  0}, {149,105,225,  0},
	{208, 46,169,  0}, {215, 57,207,  0}, {213,102,159,  0}, {226,102,217,  0},
	{162,148, 34,  0}, {144,140, 82,  0}, {144,138,114,  0}, {141,167, 82,  0},
	{143,173,113,  0}, {177,140, 81,  0}, {173,143,113,  0}, {177,171, 79,  0},
	{176,171,115,  0}, {164,206, 48,  0}, {157,216, 94,  0}, {211,154, 43,  0},
	{205,145, 78,  0}, {202,147,110,  0}, {205,171, 79,  0}, {207,173,114,  0},
	{235,144, 81,  0}, {239,146,112,  0}, {237,173, 80,  0}, {242,176,113,  0},
	{219,205, 29,  0}, {227,209,104,  0}, {144,143,142,  0}, {141,147,172,  0},
	{145,171,146,  0}, {145,171,176,  0}, {171,147,141,  0}, {173,148,172,  0},
	{176,171,146,  0}, {175,176,174,  0}, {141,146,205,  0}, {141,145,241,  0},
	{143,176,205,  0}, {142,177,243,  0}, {172,146,205,  0}, {172,145,245,  0},
	{173,179,204,  0}, {173,178,242,  0}, {164,209,169,  0}, {145,202,209,  0},
	{144,207,245,  0}, {143,240,210,  0}, {144,239,249,  0}, {177,203,208,  0},
	{175,209,243,  0}, {177,237,208,  0}, {176,239,249,  0}, {202,148,141,  0},
	{205,148,171,  0}, {207,176,144,  0}, {204,180,174,  0}, {241,148,140,  0},
	{236,149,171,  0}, {241,177,143,  0}, {239,178,174,  0}, {203,145,207,  0},
	{204,147,241,  0}, {205,180,204,  0}, {204,179,242,  0}, {236,144,209,  0},
	{243,144,241,  0}, {237,182,203,  0}, {241,177,244,  0}, {209,202,146,  0},
	{208,203,179,  0}, {209,235,146,  0}, {208,235,178,  0}, {239,204,145,  0},
	{237,208,177,  0}, {243,236,145,  0}, {245,237,178,  0}, {209,209,207,  0},
	{206,211,240,  0}, {210,234,210,  0}, {208,240,247,  0}, {237,212,205,  0},
	{239,212,241,  0}, {242,234,210,  0}, {246,247,247,  0}, {254,254,254,  0},
	{222,222,222,  0}, {203,203,203,  0}, {184,184,184,  0}, {164,164,164,  0},
	{147,147,147,  0}, {125,125,125,  0}, {106,106,106,  0}, { 86, 86, 86,  0},
	{ 67, 67, 67,  0}, { 47, 47, 47,  0}, { 28, 28, 28,  0}, {152,167,205,  0},
	{121,145,206,  0}, {101,129,186,  0}, { 81,115,164,  0}, { 61, 99,144,  0},
	{ 40, 82,123,  0}, { 15, 66,104,  0}, { 69,230,255,  0}, { 37,211,252,  0},
	{ 12,186,241,  0}, { 15,149,202,  0}, { 16,117,167,  0}, { 16, 87,131,  0},
	{ 17,148,255,  0}, { 14,126,221,  0}, { 11,104,187,  0}, {  9, 83,154,  0},
	{  6, 61,120,  0}, {  3, 39, 86,  0}, {170,212,  7,  0}, {150,185,  8,  0},
	{130,158,  9,  0}, {111,132, 11,  0}, { 91,105, 12,  0}, {111,119,235,  0},
	{ 79, 85,188,  0}, { 63, 68,165,  0}, { 46, 50,142,  0}, {100,144,128,  0},
	{ 85,129,105,  0}, { 71,114, 83,  0}, { 56, 99, 60,  0}, { 36, 68, 33,  0},
	{  1,  1,  1,  0}, {  1,  1,  1,  0}, {  1,  1,  1,  0}, {  1,  1,  1,  0},
	{  1,  1,  1,  0}, {  1,  1,  1,  0}, {  1,  1,  1,  0}, {  1,  1,  1,  0},
	{  1,  1,  1,  0}, {  1,  1,  1,  0}, {  1,  1,  1,  0}, {  1,  1,  1,  0},
	{  1,  1,  1,  0}, {  1,  1,  1,  0}, {  1,  1,  1,  0}, {  1,  1,  1,  0},
	{  1,  1,  1,  0}, {  1,  1,  1,  0}, {  1,  1,  1,  0}, {255,255,255,  0}
  }
};
LOGPALETTE* logpalette = (LOGPALETTE*)(&palette256);
HPALETTE palette = NULL;

HANDLE pics[MAX_PICHANDLES_PLUS_1] = { 0 };
GpBitmap* gpics[MAX_PICHANDLES_PLUS_1] = { 0 };

typedef enum
{
  BIGCARDTITLE_FONT = 0,
  BIGCARDTITLE_FONT2,
  BIGCARDTITLE_FONT3,
  BIGCARDTITLE_FONT4,
  BIGCARDTITLE_FONT5,
  BIGCARDTITLE_FONT6,
  BIGCARDTITLE_FONT7,
  BIGCARDTITLE_FONT8,
  BIGCARDTITLE_FONT9,
  BIGCARDTYPE_FONT,
  BIGCARDTYPE_FONT2,
  BIGCARDTYPE_FONT3,
  BIGCARDTYPE_FONT4,
  BIGCARDTYPE_FONT5,
  BIGCARDTYPE_FONT6,
  BIGCARDTYPE_FONT7,
  BIGCARDTYPE_FONT8,
  BIGCARDTYPE_FONT9,
  BIGCARDPT_FONT,
  BIGCARDTEXT_FONT,
  SMALLCARDTITLE_FONT,
  FLAVOR_FONT,

  MAX_FONTHANDLES = FLAVOR_FONT
} FontHandleNames;

HFONT fonts[MAX_FONTHANDLES + 1] = { 0 };

enum
{
  PARENT_MANALINK,
  PARENT_SHANDALAR,
  PARENT_OTHER
} parent;

HDC screendc;
ULONG_PTR gdiplus_token = 0;
char base_dir[MAX_PATH + 1];
char duel_dat[2 * MAX_PATH + 1];
CRITICAL_SECTION critical_section;

typedef struct
{
  uint8_t default_expansion;
  uint8_t default_rarity;
  uint8_t watermark;
  uint8_t flags;
#define RARITYFLAG_TIMESHIFT	(1<<0)	// Has a Planar Chaos timeshift or Future Sight futureshift frame
#define RARITYFLAG_NYX			(1<<1)	// Is a multitype enchantment
  uint8_t rarity_in_expansion[0];
} Rarity;

Rarity** rarities;
int raw_expansions = 0;
int rarities_read = 0;
int total_cards = 0;

#define FAKE_EXPANSIONS	8	// The first eight "expansions" aren't expansions, but formats.

typedef enum
{
  WATERMARK_BLACK = 0,
  WATERMARK_BLUE,
  WATERMARK_GREEN,
  WATERMARK_RED,
  WATERMARK_WHITE,
  WATERMARK_AZORIUS,
  WATERMARK_DIMIR,
  WATERMARK_RAKDOS,
  WATERMARK_GRUUL,
  WATERMARK_SELESNYA,
  WATERMARK_ORZHOV,
  WATERMARK_GOLGARI,
  WATERMARK_SIMIC,
  WATERMARK_IZZET,
  WATERMARK_BOROS,
  WATERMARK_PHYREXIAN,
  WATERMARK_MIRRAN,

  WATERMARK_MAX = WATERMARK_MIRRAN,
  WATERMARK_NONE = 0xFF
} Watermark;

typedef enum
{
  EXPCAT_NONE = 0,	// always lowest priority
  EXPCAT_CORE,
  EXPCAT_EXPERT,
  EXPCAT_PORTAL,
  EXPCAT_REPRINT,
  EXPCAT_CASUAL,
  EXPCAT_PROMO,
  EXPCAT_BOX,
  EXPCAT_DECKS,

  MAX_EXPCAT = EXPCAT_DECKS
} ExpansionCategory;

ExpansionCategory expansion_categories[] =
{
  EXPCAT_NONE,		// unused
  EXPCAT_PROMO,		//EXPANSION_PROMO
  EXPCAT_CORE,		//EXPANSION_LIMITED_EDITION_ALPHA
  EXPCAT_CORE,		//EXPANSION_LIMITED_EDITION_BETA
  EXPCAT_CORE,		//EXPANSION_UNLIMITED_EDITION
  EXPCAT_EXPERT,	//EXPANSION_ARABIAN_NIGHTS
  EXPCAT_EXPERT,	//EXPANSION_ANTIQUITIES
  EXPCAT_CORE,		//EXPANSION_REVISED_EDITION
  EXPCAT_EXPERT,	//EXPANSION_LEGENDS
  EXPCAT_EXPERT,	//EXPANSION_THE_DARK
  EXPCAT_EXPERT,	//EXPANSION_FALLEN_EMPIRES
  EXPCAT_CORE,		//EXPANSION_FOURTH_EDITION
  EXPCAT_EXPERT,	//EXPANSION_ICE_AGE
  EXPCAT_REPRINT,	//EXPANSION_CHRONICLES
  EXPCAT_EXPERT,	//EXPANSION_HOMELANDS
  EXPCAT_EXPERT,	//EXPANSION_ALLIANCES
  EXPCAT_EXPERT,	//EXPANSION_MIRAGE
  EXPCAT_EXPERT,	//EXPANSION_VISIONS
  EXPCAT_CORE,		//EXPANSION_FIFTH_EDITION
  EXPCAT_PROMO,		//EXPANSION_ASTRAL
  EXPCAT_PORTAL,	//EXPANSION_PORTAL
  EXPCAT_EXPERT,	//EXPANSION_WEATHERLIGHT
  EXPCAT_EXPERT,	//EXPANSION_TEMPEST
  EXPCAT_EXPERT,	//EXPANSION_STRONGHOLD
  EXPCAT_EXPERT,	//EXPANSION_EXODUS
  EXPCAT_PORTAL,	//EXPANSION_PORTAL_SECOND_AGE
  EXPCAT_EXPERT,	//EXPANSION_URZAS_SAGA
  EXPCAT_EXPERT,	//EXPANSION_URZAS_LEGACY
  EXPCAT_CORE,		//EXPANSION_CLASSIC_SIXTH_EDITION
  EXPCAT_EXPERT,	//EXPANSION_URZAS_DESTINY
  EXPCAT_PORTAL,	//EXPANSION_PORTAL_THREE_KINGDOMS
  EXPCAT_PORTAL,	//EXPANSION_STARTER_1999
  EXPCAT_EXPERT,	//EXPANSION_MERCADIAN_MASQUES
  EXPCAT_BOX,		//EXPANSION_BATTLE_ROYALE_BOX_SET
  EXPCAT_EXPERT,	//EXPANSION_NEMESIS
  EXPCAT_PORTAL,	//EXPANSION_STARTER_2000
  EXPCAT_EXPERT,	//EXPANSION_PROPHECY
  EXPCAT_EXPERT,	//EXPANSION_INVASION
  EXPCAT_BOX,		//EXPANSION_BEATDOWN_BOX_SET
  EXPCAT_EXPERT,	//EXPANSION_PLANESHIFT
  EXPCAT_CORE,		//EXPANSION_SEVENTH_EDITION
  EXPCAT_EXPERT,	//EXPANSION_APOCALYPSE
  EXPCAT_EXPERT,	//EXPANSION_ODYSSEY
  EXPCAT_EXPERT,	//EXPANSION_TORMENT
  EXPCAT_EXPERT,	//EXPANSION_JUDGMENT
  EXPCAT_EXPERT,	//EXPANSION_ONSLAUGHT
  EXPCAT_EXPERT,	//EXPANSION_LEGIONS
  EXPCAT_EXPERT,	//EXPANSION_SCOURGE

  EXPCAT_CORE,		//EXPANSION_EIGHTH_EDITION
  EXPCAT_EXPERT,	//EXPANSION_MIRRODIN
  EXPCAT_EXPERT,	//EXPANSION_DARKSTEEL
  EXPCAT_EXPERT,	//EXPANSION_FIFTH_DAWN
  EXPCAT_EXPERT,	//EXPANSION_CHAMPIONS_OF_KAMIGAWA
  EXPCAT_EXPERT,	//EXPANSION_BETRAYERS_OF_KAMIGAWA
  EXPCAT_EXPERT,	//EXPANSION_SAVIORS_OF_KAMIGAWA
  EXPCAT_CORE,		//EXPANSION_NINTH_EDITION
  EXPCAT_EXPERT,	//EXPANSION_RAVNICA_CITY_OF_GUILDS
  EXPCAT_EXPERT,	//EXPANSION_GUILDPACT
  EXPCAT_EXPERT,	//EXPANSION_DISSENSION
  EXPCAT_EXPERT,	//EXPANSION_COLDSNAP
  EXPCAT_EXPERT,	//EXPANSION_TIME_SPIRAL
  EXPCAT_EXPERT,	//EXPANSION_PLANAR_CHAOS
  EXPCAT_EXPERT,	//EXPANSION_FUTURE_SIGHT
  EXPCAT_CORE,		//EXPANSION_TENTH_EDITION
  EXPCAT_REPRINT,	//EXPANSION_MASTERS_EDITION
  EXPCAT_EXPERT,	//EXPANSION_LORWYN
  EXPCAT_DECKS,		//EXPANSION_DUEL_DECKS_ELVES_VS_GOBLINS
  EXPCAT_EXPERT,	//EXPANSION_MORNINGTIDE
  EXPCAT_EXPERT,	//EXPANSION_SHADOWMOOR
  EXPCAT_EXPERT,	//EXPANSION_EVENTIDE
  EXPCAT_BOX,		//EXPANSION_FROM_THE_VAULT_DRAGONS
  EXPCAT_REPRINT,	//EXPANSION_MASTERS_EDITION_II
  EXPCAT_EXPERT,	//EXPANSION_SHARDS_OF_ALARA
  EXPCAT_DECKS,		//EXPANSION_DUEL_DECKS_JACE_VS_CHANDRA
  EXPCAT_EXPERT,	//EXPANSION_CONFLUX
  EXPCAT_DECKS,		//EXPANSION_DUEL_DECKS_DIVINE_VS_DEMONIC
  EXPCAT_EXPERT,	//EXPANSION_ALARA_REBORN
  EXPCAT_CORE,		//EXPANSION_MAGIC_2010
  EXPCAT_BOX,		//EXPANSION_FROM_THE_VAULT_EXILED
  EXPCAT_CASUAL,	//EXPANSION_PLANECHASE
  EXPCAT_REPRINT,	//EXPANSION_MASTERS_EDITION_III
  EXPCAT_EXPERT,	//EXPANSION_ZENDIKAR
  EXPCAT_DECKS,		//EXPANSION_DUEL_DECKS_GARRUK_VS_LILIANA
  EXPCAT_DECKS,		//EXPANSION_PREMIUM_DECK_SERIES_SLIVERS
  EXPCAT_EXPERT,	//EXPANSION_WORLDWAKE
  EXPCAT_DECKS,		//EXPANSION_DUEL_DECKS_PHYREXIA_VS_THE_COALITION
  EXPCAT_EXPERT,	//EXPANSION_RISE_OF_THE_ELDRAZI
  EXPCAT_CASUAL,	//EXPANSION_ARCHENEMY
  EXPCAT_CORE,		//EXPANSION_MAGIC_2011
  EXPCAT_BOX,		//EXPANSION_FROM_THE_VAULT_RELICS
  EXPCAT_DECKS,		//EXPANSION_DUEL_DECKS_ELSPETH_VS_TEZZERET
  EXPCAT_EXPERT,	//EXPANSION_SCARS_OF_MIRRODIN
  EXPCAT_DECKS,		//EXPANSION_PREMIUM_DECK_SERIES_FIRE_AND_LIGHTNING
  EXPCAT_REPRINT,	//EXPANSION_MASTERS_EDITION_IV
  EXPCAT_EXPERT,	//EXPANSION_MIRRODIN_BESIEGED
  EXPCAT_DECKS,		//EXPANSION_DUEL_DECKS_KNIGHTS_VS_DRAGONS
  EXPCAT_EXPERT,	//EXPANSION_NEW_PHYREXIA
  EXPCAT_CASUAL,	//EXPANSION_MAGIC_THE_GATHERINGCOMMANDER
  EXPCAT_CORE,		//EXPANSION_MAGIC_2012
  EXPCAT_BOX,		//EXPANSION_FROM_THE_VAULT_LEGENDS
  EXPCAT_DECKS,		//EXPANSION_DUEL_DECKS_AJANI_VS_NICOL_BOLAS
  EXPCAT_EXPERT,	//EXPANSION_INNISTRAD
  EXPCAT_DECKS,		//EXPANSION_PREMIUM_DECK_SERIES_GRAVEBORN
  EXPCAT_EXPERT,	//EXPANSION_DARK_ASCENSION
  EXPCAT_DECKS,		//EXPANSION_DUEL_DECKS_VENSER_VS_KOTH
  EXPCAT_EXPERT,	//EXPANSION_AVACYN_RESTORED
  EXPCAT_CASUAL,	//EXPANSION_PLANECHASE_2012
  EXPCAT_CORE,		//EXPANSION_MAGIC_2013
  EXPCAT_BOX,		//EXPANSION_FROM_THE_VAULT_REALMS
  EXPCAT_DECKS,		//Duel Decks: Izzet vs. Golgari
#define EXPANSION_DUEL_DECKS_IZZET_VS_GOLGARI	((EXPANSION_t)(EXPANSION_FROM_THE_VAULT_REALMS + 1))
  EXPCAT_EXPERT,	//Return to Ravnica
#define EXPANSION_RETURN_TO_RAVNICA	((EXPANSION_t)(EXPANSION_FROM_THE_VAULT_REALMS + 2))
  EXPCAT_CASUAL,	//Commander's Arsenal
  EXPCAT_EXPERT,	//Gatecrash
#define EXPANSION_GATECRASH	((EXPANSION_t)(EXPANSION_FROM_THE_VAULT_REALMS + 4))
  EXPCAT_DECKS,		//Duel Decks: Sorin vs. Tibalt
  EXPCAT_EXPERT,	//Dragon's Maze
#define EXPANSION_DRAGONS_MAZE	((EXPANSION_t)(EXPANSION_FROM_THE_VAULT_REALMS + 6))
  EXPCAT_REPRINT,	//Modern Masters
#define EXPANSION_MODERN_MASTERS	((EXPANSION_t)(EXPANSION_FROM_THE_VAULT_REALMS + 7))
  EXPCAT_CORE,		//Magic 2014
  EXPCAT_BOX,		//From the Vault: Twenty
  EXPCAT_DECKS,		//Duel Decks: Heroes vs. Monsters
  EXPCAT_EXPERT,	//Theros
#define EXPANSION_THEROS ((EXPANSION_t)(EXPANSION_FROM_THE_VAULT_REALMS + 11)) 
  EXPCAT_NONE,		//Future Expansion
  EXPCAT_NONE,		//Future Expansion
  EXPCAT_NONE,		//Future Expansion
  EXPCAT_NONE,		//Future Expansion
  EXPCAT_NONE,		//Future Expansion
  EXPCAT_NONE,		//Future Expansion
  EXPCAT_NONE,		//Future Expansion
  EXPCAT_NONE,		//Future Expansion
  EXPCAT_NONE,		//Future Expansion
  EXPCAT_NONE		//Future Expansion
};

/**************** Configuration ****************/
typedef struct
{
  int r, g, b;	// The raw values read from config
  COLORREF colorref;
} ColorDef;
#define COLORDEF_AND_VIS(x) ColorDef x##_color; int x##_visible
#define COLORDEF_AND_SHADOW_VIS_OFFSET(x) ColorDef x##_color, x##_shadow_color; int x##_shadow_visible; POINT x##_shadow_offset

typedef struct
{
  RECT smallcard_art, smallcard_expansion, smallcard_title;
  int smallcard_art_first;
  int smallcard_title_source_top, smallcard_title_source_height, smallcard_title_dest_height;
  int smallcard_frame_source_top, smallcard_frame_source_height, smallcard_frame_dest_top;
  int smallcard_expansion_visible, smallcard_expansion_no_early_core_sets, smallcard_expansion_no_early_rarity, smallcard_expansion_force_rarity;
  COLORDEF_AND_VIS(smallcard_outline);
  COLORDEF_AND_VIS(smallcard_artoutline);
  COLORDEF_AND_SHADOW_VIS_OFFSET(smallcard_title_owned);
  COLORDEF_AND_SHADOW_VIS_OFFSET(smallcard_title_owned_can_activate);
  COLORDEF_AND_SHADOW_VIS_OFFSET(smallcard_title_owned_must_activate);
  COLORDEF_AND_SHADOW_VIS_OFFSET(smallcard_title_unowned);
  COLORDEF_AND_SHADOW_VIS_OFFSET(smallcard_title_unowned_can_activate);
  COLORDEF_AND_SHADOW_VIS_OFFSET(smallcard_title_unowned_must_activate);
  COLORDEF_AND_VIS(smallcard_title_owned_background);
  COLORDEF_AND_VIS(smallcard_title_unowned_background);

  int fullcard_reread_config_on_redraw, fullcard_art_first, fullcard_watermark_outside_set;
  int fullcard_expand_top, fullcard_expand_height, fullcard_curve_mana_diameter;
  int fullcard_powertoughness_center, fullcard_powertoughness_box_visible, fullcard_extended_powertoughness;
  int fullcard_expansion_visible, fullcard_expansion_no_early_core_sets, fullcard_expansion_no_early_rarity, fullcard_expansion_force_rarity;
  int fullcard_tombstone_visible, fullcard_tombstone_title_offset, fullcard_type_icon_visible, fullcard_type_icon_suppressed_by_tombstone;
  RECT fullcard_frame, fullcard_title, fullcard_mana, fullcard_art, fullcard_type, fullcard_rulestext, fullcard_rulesbox, fullcard_watermark;
  RECT fullcard_powertoughness, fullcard_powertoughness_box, fullcard_expansion, fullcard_tombstone, fullcard_type_icon;
#define FULLCARD_CURVE_MANA_MAX	13
  POINT fullcard_curve_mana[FULLCARD_CURVE_MANA_MAX];
  POINT fullcard_rounding;
  ColorDef fullcard_border_color, fullcard_rulestext_color;
  COLORDEF_AND_VIS(fullcard_outline);
  COLORDEF_AND_VIS(fullcard_artoutline);
  COLORDEF_AND_SHADOW_VIS_OFFSET(fullcard_title);
  COLORDEF_AND_SHADOW_VIS_OFFSET(fullcard_powertoughness);
  COLORDEF_AND_SHADOW_VIS_OFFSET(fullcard_type);
  COLORDEF_AND_VIS(fullcard_title_on_light);
  COLORDEF_AND_VIS(fullcard_title_on_white_black);
  COLORDEF_AND_VIS(fullcard_rules_on_dark_land);
  COLORDEF_AND_VIS(fullcard_rules_on_white_black_land);

  int frames_percent_mixed_on_hybrid;
  int frames_no_early_hybrid_land;
  int frames_two_color_as_hybrid, frames_two_color_as_gold;
  int frames_translucent_colorless, frames_colorless_alpha;
  GpImageAttributes* colorless_alpha_xform;
  ColorMatrix matrix;	// Unclear if this has to outlive the call to GdipSetImageAttributesColorMatrix().  Doesn't hurt to keep it around, though.
  int frames_facedown_creature;
  int frames_artifact_land;

#undef COLORDEF_AND_VIS
#undef COLORDEF_AND_SHADOW_VIS_OFFSET

  // Constants
  const char* subsection_name;
  int cardbk_base;
  int frame_base;
} Config;

#define CFG_MODERN			0
#define CFG_CLASSIC			1
#define CFG_TIMESHIFT		2
#define CFG_FUTURESHIFT		3
#define CFG_PLANESWALKER	4
#define CFG_TOKEN			5
#define MAX_FRAMESET		CFG_TOKEN
#define CFG_BASE			MAX_FRAMESET + 1	// must be greater than MAX_FRAMESET
#define MAX_CFG				CFG_BASE

Config configs[MAX_CFG + 1];
Config* cfg = &configs[CFG_BASE];	// configuration for current frametype.  Only accurate after a call to select_frame().
int hack_power = 0, hack_toughness = 0;	// An ugly hack to make sure Theros enchantment-creatures show their power/toughness, even though the base cards are non-creature versions

typedef enum
{
  FRAMES_TRANSLUCENT_COLORLESS_ALSO_NORMAL_FULLCARD_ART = 1,
  FRAMES_TRANSLUCENT_COLORLESS_NO_SMALLCARD_FRAME = 2,
  FRAMES_TRANSLUCENT_COLORLESS_REDRAW_SMALLCARD_ART = 4
} FramesTranslucentColorless;

int force_frameset = -1, force_nyx = -1;
int expansions_latest_printing;
int expansions_priority[MAX_EXPCAT + 1];

int watermark_size_x, watermark_size_y;
char* frames_pic_names[FRAME_MAX_LOADED + 2] = { 0 };	// +2, so there's a NULL at the end
int frames_dup_of[FRAME_MAX_LOADED + 1] = { [0 ... FRAME_MAX_LOADED] = -1 };	// gcc extension: sets all frames_dup_of[0] through [FRAME_MAX_LOADED] to -1


#define DLLIMPORT __attribute__((dllimport))

/**************** In image.dll  ****************/
DLLIMPORT HANDLE LoadImage(const char*, HPALETTE, int, int);

/**************** In Cardartlib.dll ****************/
DLLIMPORT void DestroyAllBigArts(void);
DLLIMPORT void DestroyAllSmallArts(void);
DLLIMPORT int DrawBigArt(HDC hdc, const RECT* rect, int id, int version);
DLLIMPORT int DrawSmallArt(HDC hdc, const RECT* rect, int id, int version);
DLLIMPORT int IsBigArtRightSize(int id, int version, int width, int height);
DLLIMPORT int IsSmallArtIn(int id, int version);
DLLIMPORT int LoadBigArt(int id, int version, int width, int height);
DLLIMPORT int LoadSmallArt(int id, int version, int width, int height);
DLLIMPORT int ReloadSmallArtIfWrongSize(int id, int version, int width, int heigt);

/**************** In Magic.exe ****************/
void* get_displayed_card_instance_exe = (void*)(0x401ab0);
typedef int (*intFn_int)(int);
intFn_int get_internal_card_id_from_csv_id_exe = (intFn_int)(0x479df0);
card_data_t* cards_data_exe = (void*)(0x7e7010);
card_ptr_t** cards_ptr_exe = (void*)(0x73bb00);

/**************** Public API ****************/
BOOL WINAPI DllMain(HINSTANCE dll, DWORD reason, LPVOID reserved);
int CalcDrawManaText(HDC hdc, const RECT* rect, const char* str);
int DestroyAllCardBackgrounds(void);
int DrawCardBack(HDC hdc, const RECT* dest_rect);
int DrawFullCard(HDC hdc, const RECT* rect, const card_ptr_t* cp, int version, int big_art_style, int expand_text_box, const char* illus);
int DrawManaText(HDC hdc, const RECT* dest_rect, const char* str, int drawflag);
int DrawSmallCard(HDC hdc, const RECT* card_rect, const card_ptr_t* cp, int version);
int DrawSmallCardTitle(HDC hdc, const RECT* dest_rect, const char* card_title, int color, int controlled_by_owner);

/**************** Internal ****************/
static void popup(const char* title, const char* fmt, ...);
static const char* get_pichandlename(PicHandleNames handle_name, int idx);	// idx may be 0, 1, or 2, and should be different for simultaneous calls, since this may return an internal buffer
static int initialize(void);
static void init_palette(void);
static int read_cfg(void);
static void read_cfg_section(Config* config);
static int prepare_fonts_and_imgs(void);
static LOGFONT* cfg_font(const char* font_name, int italic);
static void get_cfg_int(Config* config, int* val, const char* section, const char* key, int deflt);
static int get_cfg_int_raw(const char* section, const char* key, int deflt);
static int get_cfg_int2_raw(const char* section, const char* key1, const char* key2, int deflt);
static void get_cfg_rect(Config* config, RECT* rect, const char* section, const char* key, int deflt_l, int deflt_t, int deflt_w, int deflt_h);
static void get_cfg_point(Config* config, POINT* pt, const char* section, const char* key, int deflt_x, int deflt_y);
static int get_cfg_colordef(Config* config, ColorDef* color, const char* section, const char* key, int deflt_r, int deflt_g, int deflt_b);
static void get_cfg_str_raw(char* dest, int sz_of_dest, const char* section, const char* key, const char* deflt_fmt, ...);
static void del_screendc(void);
static void del_resources(void);
static int is_token_by_instance(card_instance_t* instance);
static PicHandleNames select_frame(const card_ptr_t* cp, const Rarity* r);
static CountColors manacost_to_count_colors(const card_ptr_t* cp);
static PicHandleNames count_colors_to_hybrid_pic_handle_name(CountColors cc, PicHandleNames base, PicHandleNames deflt);
static int cost_values_to_cost_text(const card_ptr_t* cp, char* dest);
static int convert_initial_mana_tag(const char** src_string);
static void make_frame(PicHandleNames frame);
static void make_gpic_from_pic(PicHandleNames picnum);
static void make_frame_from_overlay(PicHandleNames target, PicHandleNames background, PicHandleNames overlay);	// background<overlay
static void make_frame_from_2_overlays(PicHandleNames target, PicHandleNames background, PicHandleNames overlay1, PicHandleNames overlay2);	// background<overlay1<overlay2
static void make_hybrid_frame(Config* config, PicHandleNames target, PicHandleNames left, PicHandleNames right);	// left|right
static void make_hybrid_frame_with_overlay(Config* config, PicHandleNames target, PicHandleNames left, PicHandleNames right, PicHandleNames overlay);	// (left|right)<overlay
static int blt_powertoughness_box(HDC hdc, const RECT* dest_rect, PicHandleNames frame, int translucent);
static void blt_type_icon(HDC hdc, const RECT* dest_rect, const card_ptr_t* cp, PicHandleNames frame);
static void gdip_blt_whole(HDC hdc, const RECT* dest_rect, PicHandleNames frame, int translucent);
static void gdip_blt(HDC hdc, const RECT* dest_rect, PicHandleNames frame, int src_x, int src_y, int width, int height, int translucent);
static void blt_expansion_symbol(HDC hdc, const RECT* dest_rect, EXPANSION_t expansion, rarity_t rarity, int smallcard);
static void draw_mana_symbol(HDC hdc, char c, int x, int y, int wid, int hgt);
static int calc_draw_mana_text_impl(HDC hdc, const RECT* rect, const char* str, int rules_text_flag, int no_addititional_space);
static int draw_mana_text_impl(HDC hdc, const RECT* dest_rect, const char* str, int drawflag, int rules_text_flag, int no_additional_space);
static void close_gdiplus(void);
static void init_gdiplus(void);
static void create_colorless_alpha_xform(Config* config);
static int draw_small_card_title_impl(HDC hdc, const RECT* dest_rect, const char* card_title, int color, int controlled_by_owner);
static const card_data_t* get_card_data_from_csvid(uint32_t csvid);
static int has_graveyard_ability(const card_ptr_t* cp);
static int draw_fullcard_art(HDC hdc, RECT* rect, uint32_t id, int version);
static void strip_hack_and_sleight_symbols(char* dest, const char* src);
static int manacost_width(HDC hdc, const RECT* dest_rect, const card_ptr_t* cp);
static void draw_manacost(HDC hdc, RECT* dest_rect, const card_ptr_t* cp);
static RECT* draw_smallcard_art(HDC hdc, const RECT* card_rect, int id, int version, int must_save_dc, int shrink_rect, int intersect_clip_with_shrunk_rect);
static void draw_smallcard_frame(HDC hdc, const RECT* dest_rect, PicHandleNames frame, int translucent);
static void del_obj(HGDIOBJ* obj);
static void rem_font(const char* fontname);
static const Rarity* get_rarity(int id);
static void read_rarities(void);
static void read_watermarks(void);
static void assign_default_rarities(void);

/**************** In get_displayed_card_instance.asm ****************/
card_instance_t* get_displayed_card_instance(int player, int card);

static void
popup(const char* title, const char* fmt, ...)
{
  char buf[8000];
  va_list args;
  va_start(args, fmt);
  vsnprintf(buf, 8000, fmt, args);
  va_end(args);

  MessageBox(0, buf, title, MB_ICONERROR|MB_TASKMODAL);
}

#if 0
void
lg(const char* fmt, ...)
{
  char buf[8000];
  va_list args;
  va_start(args, fmt);
  vsnprintf(buf, 8000, fmt, args);
  va_end(args);

  FILE* f = fopen("trace.txt", "a");
  fprintf(f, "%s\n", buf);
  fclose(f);
}
#endif

static const char*
get_pichandlename(PicHandleNames handle_name, int idx)
{
  if (idx < 0 || idx > 2)
	abort();

  static char buf[3][16];
  char* p = buf[idx];

  if (handle_name <= FRAME_MAX_LOADED)
	{
	  if (handle_name < CARDBK_MIN_FRAMEPART)
		return cfg_pic_names[handle_name];

	  sprintf(p, "[%s]%s", configs[FRAMESET_OF_FRAMEPART(handle_name)].subsection_name, cfg_pic_names[BASE_FRAMEPART(handle_name)]);
	}
  else
	sprintf(p, "#%d", (int)handle_name);

  return p;
}

BOOL WINAPI
DllMain(HINSTANCE dll, DWORD reason, LPVOID reserved)
{
  (void)dll;
  (void)reserved;

  switch (reason)
	{
	  case DLL_PROCESS_ATTACH:
		return initialize();

	  case DLL_PROCESS_DETACH:
		del_resources();
		del_screendc();
		return 1;

	  default:
		return 1;
	}
}

static int
initialize(void)
{
  HMODULE parent_mod = GetModuleHandle(0);
  if (GetProcAddress(parent_mod, "szDeckName"))
	parent = PARENT_MANALINK;
  else if (GetProcAddress(parent_mod, "_Opponface"))
	parent = PARENT_SHANDALAR;
  else
	parent = PARENT_OTHER;

  GetModuleFileName(NULL, base_dir, MAX_PATH + 1);
  *(strrchr(base_dir, '\\')) = 0;

  if (screendc)
	return 1;

  HDC hdc = GetDC(NULL);
  screendc = CreateCompatibleDC(hdc);
  ReleaseDC(NULL, hdc);

  init_palette();
  InitializeCriticalSection(&critical_section);

  if (!screendc)
	{
	  popup("drawcardlib.dll", "!screendc");
	  del_screendc();
	  return 0;
	}

  CountColors single_mana[256] = { 0 };
  single_mana[(uint8_t)'B'] = COUNT_COLOR_BLACK;
  single_mana[(uint8_t)'U'] = COUNT_COLOR_BLUE;
  single_mana[(uint8_t)'G'] = COUNT_COLOR_GREEN;
  single_mana[(uint8_t)'R'] = COUNT_COLOR_RED;
  single_mana[(uint8_t)'W'] = COUNT_COLOR_WHITE;

  int i;
  for (i = 0; mana_tags[i]; i += 3)
	switch (mana_tags[i])
	  {
		case MANA_TAG_COLOR_MIN ... MANA_TAG_COLOR_MAX:
		  mana_tag_to_count_colors[(uint8_t)mana_tags[i]] = single_mana[(uint8_t)mana_tags[i + 1]] | COUNT_COLOR_MONO;
		  break;

		case MANA_TAG_MONO_HYBRID_MIN ... MANA_TAG_MONO_HYBRID_MAX:
		case MANA_TAG_PHYREXIAN_MIN ... MANA_TAG_PHYREXIAN_MAX:
		  mana_tag_to_count_colors[(uint8_t)mana_tags[i]] = single_mana[(uint8_t)mana_tags[i + 2]] | COUNT_COLOR_MONO;
		  break;

		case MANA_TAG_HYBRID_MIN ... MANA_TAG_HYBRID_MAX:
		  mana_tag_to_count_colors[(uint8_t)mana_tags[i]] = single_mana[(uint8_t)mana_tags[i + 1]] | single_mana[(uint8_t)mana_tags[i + 2]] | COUNT_COLOR_HYBRID;
		  break;
	  }

  for (i = 0; i <= MAX_CFG; ++i)
	configs[i].colorless_alpha_xform = NULL;

  int result = read_cfg();

  return prepare_fonts_and_imgs() || result;
}

static void
init_palette(void)
{
  PALETTEENTRY* entry = &logpalette->palPalEntry[0];
  int i;
  for (i = 0; i < 256; ++i, ++entry)
	{
	  BYTE red = entry->peRed;
	  entry->peRed = entry->peBlue;
	  entry->peBlue = red;

	  entry->peFlags = PC_EXPLICIT;
	}

  palette = CreatePalette(logpalette);
}

char* base_frame_base_dir = NULL;
char* base_frame_pic_names[NUM_FRAMEPARTS_PER_FRAMESET] = {0};

static int
read_cfg(void)
{
  int rval = 0;

  strcpy(duel_dat, base_dir);
  strcat(duel_dat, "\\DuelArt\\Duel.dat");

  configs[CFG_BASE].subsection_name = "";
  configs[CFG_BASE].cardbk_base = 0;	// Handled specially.
  configs[CFG_BASE].frame_base = 0;		// Handled specially.
  configs[CFG_MODERN].subsection_name = "Modern";
  configs[CFG_MODERN].cardbk_base = CARDBK_MODERN_MIN;
  configs[CFG_MODERN].frame_base = FRAME_MODERN_MIN;
  configs[CFG_CLASSIC].subsection_name = "Classic";
  configs[CFG_CLASSIC].cardbk_base = CARDBK_CLASSIC_MIN;
  configs[CFG_CLASSIC].frame_base = FRAME_CLASSIC_MIN;
  configs[CFG_TIMESHIFT].subsection_name = "Timeshift";
  configs[CFG_TIMESHIFT].cardbk_base = CARDBK_TIMESHIFT_MIN;
  configs[CFG_TIMESHIFT].frame_base = FRAME_TIMESHIFT_MIN;
  configs[CFG_FUTURESHIFT].subsection_name = "Futureshift";
  configs[CFG_FUTURESHIFT].cardbk_base = CARDBK_FUTURESHIFT_MIN;
  configs[CFG_FUTURESHIFT].frame_base = FRAME_FUTURESHIFT_MIN;
  configs[CFG_PLANESWALKER].subsection_name = "Planeswalker";
  configs[CFG_PLANESWALKER].cardbk_base = CARDBK_PLANESWALKER_MIN;
  configs[CFG_PLANESWALKER].frame_base = FRAME_PLANESWALKER_MIN;
  configs[CFG_TOKEN].subsection_name = "Token";
  configs[CFG_TOKEN].cardbk_base = CARDBK_TOKEN_MIN;
  configs[CFG_TOKEN].frame_base = FRAME_TOKEN_MIN;

  expansions_latest_printing = get_cfg_int_raw(EXPANSIONSTXT, "LatestPrinting", 0);
  expansions_priority[EXPCAT_NONE] = INT_MIN;
  expansions_priority[EXPCAT_CORE] = get_cfg_int_raw(EXPANSIONSTXT, "PriorityCore", 10);
  expansions_priority[EXPCAT_EXPERT] = get_cfg_int_raw(EXPANSIONSTXT, "PriorityExpert", 10);
  expansions_priority[EXPCAT_PORTAL] = get_cfg_int_raw(EXPANSIONSTXT, "PriorityPortal", 7);
  expansions_priority[EXPCAT_REPRINT] = get_cfg_int_raw(EXPANSIONSTXT, "PriorityReprint", 6);
  expansions_priority[EXPCAT_CASUAL] = get_cfg_int_raw(EXPANSIONSTXT, "PriorityCasual", 4);
  expansions_priority[EXPCAT_PROMO] = get_cfg_int_raw(EXPANSIONSTXT, "PriorityPromo", 3);
  expansions_priority[EXPCAT_BOX] = get_cfg_int_raw(EXPANSIONSTXT, "PriorityBox", 2);
  expansions_priority[EXPCAT_DECKS] = get_cfg_int_raw(EXPANSIONSTXT, "PriorityDecks", 1);

  force_frameset = -1;
  force_nyx = -1;

  int i;
  char buf[MAX_PATH + 1];

  get_cfg_str_raw(buf, MAX_PATH, EXPANSIONSTXT, "ForceFrame", "");
  if (*buf)
	{
	  char* p = buf;
	  if (!strncasecmp(p, "nyx", 3))
		{
		  p += 3;
		  force_nyx = 1;
		}

	  if (*p)
		{
		  if (force_nyx == -1)
			force_nyx = 0;

		  for (i = 0; i <= MAX_FRAMESET; ++i)
			if (!strcasecmp(p, configs[i].subsection_name))
			  {
				force_frameset = i;
				break;
			  }
		  if (force_frameset == -1)
			{
			  char buf2[MAX_PATH + 1000];
			  p = buf2 + sprintf(buf2, "Unknown value for [Expansions]ForceFrame: \"%s\"\nValid values are:", buf);
			  for (i = 0; i <= MAX_FRAMESET; ++i)
				p += sprintf(p, "\n%s\nNyx%s", configs[i].subsection_name, configs[i].subsection_name);

			  popup("drawcardlib.dll", buf2);

			  rval = 1;
			}
		}
	}

  for (i = 0; i < CARDBK_MIN_FRAMEPART; ++i)
	if (!frames_pic_names[i])
	  {
		snprintf(buf, MAX_PATH, "%s.pic", cfg_pic_names[i]);
		frames_pic_names[i] = strdup(buf);
	  }

  read_cfg_section(&configs[CFG_BASE]);	// Must be read first
  read_cfg_section(&configs[CFG_MODERN]);
  read_cfg_section(&configs[CFG_CLASSIC]);
  read_cfg_section(&configs[CFG_TIMESHIFT]);
  read_cfg_section(&configs[CFG_FUTURESHIFT]);
  read_cfg_section(&configs[CFG_PLANESWALKER]);
  read_cfg_section(&configs[CFG_TOKEN]);

  free(base_frame_base_dir);
  base_frame_base_dir = NULL;

  for (i = 0; i < NUM_FRAMEPARTS_PER_FRAMESET; ++i)
	{
	  free(base_frame_pic_names[i]);
	  base_frame_pic_names[i] = NULL;
	}

  return rval;
}

static void
read_cfg_section(Config* config)
{
  char section[100];
  sprintf(section, "SmallCard%s", config->subsection_name);

  get_cfg_rect(config, &config->smallcard_art, section, "Art", 79, 161, 860, 786);

  get_cfg_int(config, &config->smallcard_art_first, section, "ArtFirst", 0);
  get_cfg_int(config, &config->smallcard_title_source_top, section, "TitleSourceTop", 4);
  get_cfg_int(config, &config->smallcard_title_source_height, section, "TitleSourceHeight", 15);
  get_cfg_int(config, &config->smallcard_title_dest_height, section, "TitleDestHeight", 200);
  get_cfg_int(config, &config->smallcard_frame_source_top, section, "FrameSourceTop", 14);
  get_cfg_int(config, &config->smallcard_frame_source_height, section, "FrameSourceHeight", 175);
  get_cfg_int(config, &config->smallcard_frame_dest_top, section, "FrameDestTop", 144);

  config->smallcard_outline_visible = get_cfg_colordef(config, &config->smallcard_outline_color, section, "OutlineColor", 147,147,147);
  config->smallcard_artoutline_visible = get_cfg_colordef(config, &config->smallcard_artoutline_color, section, "ArtOutlineColor", -1,-1,-1);

  get_cfg_rect(config, &config->smallcard_title, section, "Title", 8, -8, 800, 1120);

#define TXT_AND_SHADOW(v, txt, cr,cg,cb, sr,sg,sb, x,y)																		\
  get_cfg_colordef(config, &config->v##_color, section, txt "Color", cr,cg,cb);												\
  config->v##_shadow_visible = get_cfg_colordef(config, &config->v##_shadow_color, section, txt "ShadowColor", sr,sg,sb);	\
  get_cfg_point(config, &config->v##_shadow_offset, section, txt "ShadowOffset", x, y)

#define TXT_AND_SHADOW_ALL_DEFAULT(v, txt, deflt)																								\
  TXT_AND_SHADOW(v, txt, configs[CFG_BASE].deflt##_color.r, configs[CFG_BASE].deflt##_color.g, configs[CFG_BASE].deflt##_color.b,				\
				 configs[CFG_BASE].deflt##_shadow_color.r, configs[CFG_BASE].deflt##_shadow_color.g, configs[CFG_BASE].deflt##_shadow_color.b,	\
				 configs[CFG_BASE].deflt##_shadow_offset.x, configs[CFG_BASE].deflt##_shadow_offset.y)

#define TXT_AND_DEFAULT_SHADOW(v, txt, cr, cg, cb, deflt)																						\
  TXT_AND_SHADOW(v, txt, cr,cg,cb,																												\
				 configs[CFG_BASE].deflt##_shadow_color.r, configs[CFG_BASE].deflt##_shadow_color.g, configs[CFG_BASE].deflt##_shadow_color.b,	\
				 configs[CFG_BASE].deflt##_shadow_offset.x, configs[CFG_BASE].deflt##_shadow_offset.y)

  TXT_AND_SHADOW(smallcard_title_owned, "TitleOwned", 255,255,255, 47,47,47, 12,12);
  TXT_AND_DEFAULT_SHADOW(smallcard_title_owned_can_activate, "TitleOwnedCanActivate", 241,217,39, smallcard_title_owned);
  TXT_AND_DEFAULT_SHADOW(smallcard_title_owned_must_activate, "TitleOwnedMustActivate", 255,148,17, smallcard_title_owned);
  config->smallcard_title_owned_background_visible = get_cfg_colordef(config, &config->smallcard_title_owned_background_color, section,
																	  "TitleOwnedBackgroundColor", -1,-1,-1);

  TXT_AND_SHADOW_ALL_DEFAULT(smallcard_title_unowned, "TitleUnowned", smallcard_title_owned);
  TXT_AND_SHADOW_ALL_DEFAULT(smallcard_title_unowned_can_activate, "TitleUnownedCanActivate", smallcard_title_owned_can_activate);
  TXT_AND_SHADOW_ALL_DEFAULT(smallcard_title_unowned_must_activate, "TitleUnownedMustActivate", smallcard_title_owned_must_activate);
  config->smallcard_title_unowned_background_visible = get_cfg_colordef(config, &config->smallcard_title_unowned_background_color, section,
																		"TitleUnownedBackgroundColor", 147,147,147);

  get_cfg_rect(config, &config->smallcard_expansion, section, "Expansion", 587, 9, 208, 147);
  get_cfg_int(config, &config->smallcard_expansion_visible, section, "ExpansionVisible", 0);
  get_cfg_int(config, &config->smallcard_expansion_no_early_core_sets, section, "ExpansionNoEarlyCoreSets", 0);
  get_cfg_int(config, &config->smallcard_expansion_no_early_rarity, section, "ExpansionNoEarlyRarity", 0);
  get_cfg_int(config, &config->smallcard_expansion_force_rarity, section, "ExpansionForceRarity", 0);


  sprintf(section, "FullCard%s", config->subsection_name);

  get_cfg_int(config, &config->fullcard_reread_config_on_redraw, section, "RereadConfigOnRedraw", 0);
  get_cfg_int(config, &config->fullcard_art_first, section, "ArtFirst", configs[CFG_BASE].smallcard_art_first);
  get_cfg_int(config, &config->fullcard_expand_top, section, "ExpandTop", 6050);
  get_cfg_int(config, &config->fullcard_expand_height, section, "ExpandHeight", 3220);

  get_cfg_rect(config, &config->fullcard_frame, section, "Frame", 24, 24, 752, 1152);
  get_cfg_rect(config, &config->fullcard_title, section, "Title", 40, 20, 712, 64);
  get_cfg_rect(config, &config->fullcard_mana, section, "Mana", 48, 32, 712, 48);
  get_cfg_rect(config, &config->fullcard_art, section, "Art", 80, 96, 640, 560);
  get_cfg_rect(config, &config->fullcard_type, section, "Type", 48, 660, 590, 60);
  get_cfg_rect(config, &config->fullcard_rulestext, section, "Rulestext", 92, 736, 616, 336);
  get_cfg_rect(config, &config->fullcard_rulesbox, section, "Rulesbox", 64, 720, 672, 364);
  get_cfg_rect(config, &config->fullcard_watermark, section, "Watermark", 230, 736, 340, 340);
  get_cfg_rect(config, &config->fullcard_powertoughness, section, "Powertoughness", 44, 1084, 704, 72);
  get_cfg_rect(config, &config->fullcard_powertoughness_box, section, "PowertoughnessBox", 553, 1064, 201, 105);
  get_cfg_rect(config, &config->fullcard_expansion, section, "Expansion", 638, 666, 104, 53);

  get_cfg_int(config, &config->fullcard_curve_mana_diameter, section, "CurveManaDiameter", 0);
  int i;
  for (i = 0; i < FULLCARD_CURVE_MANA_MAX; ++i)
	{
	  char curve_mana[16];
	  sprintf(curve_mana, "CurveMana%d", i + 1);
	  get_cfg_point(config, &config->fullcard_curve_mana[i], section, curve_mana, 400, 100);
	}

  get_cfg_int(config, &config->fullcard_watermark_outside_set, section, "WatermarkOutsideSet", 0);
  get_cfg_int(config, &config->fullcard_powertoughness_center, section, "PowertoughnessCenter", 0);
  get_cfg_int(config, &config->fullcard_powertoughness_box_visible, section, "PowertoughnessBoxVisible", 0);
  get_cfg_int(config, &config->fullcard_extended_powertoughness, section, "ExtendedPowertoughness", 0);
  get_cfg_int(config, &config->fullcard_expansion_visible, section, "ExpansionVisible", 1);
  get_cfg_int(config, &config->fullcard_expansion_no_early_core_sets, section, "ExpansionNoEarlyCoreSets", configs[CFG_BASE].smallcard_expansion_no_early_core_sets);
  get_cfg_int(config, &config->fullcard_expansion_no_early_rarity, section, "ExpansionNoEarlyRarity", configs[CFG_BASE].smallcard_expansion_no_early_rarity);
  get_cfg_int(config, &config->fullcard_expansion_force_rarity, section, "ExpansionForceRarity", configs[CFG_BASE].smallcard_expansion_force_rarity);

  get_cfg_int(config, &config->fullcard_tombstone_visible, section, "TombstoneVisible", 0);
  get_cfg_int(config, &config->fullcard_tombstone_title_offset, section, "TombstoneTitleOffset", 28);
  get_cfg_rect(config, &config->fullcard_tombstone, section, "Tombstone", 32,32,32,48);

  get_cfg_int(config, &config->fullcard_type_icon_visible, section, "TypeIconVisible", 0);
  get_cfg_int(config, &config->fullcard_type_icon_suppressed_by_tombstone, section, "TypeIconSuppressedByTombstone", 0);
  get_cfg_rect(config, &config->fullcard_type_icon, section, "TypeIcon", 35,35,50,55);

  get_cfg_point(config, &config->fullcard_rounding, section, "Rounding", 12, 12);

  get_cfg_colordef(config, &config->fullcard_border_color, section, "BorderColor", 28,28,28);
  config->fullcard_outline_visible = get_cfg_colordef(config, &config->fullcard_outline_color, section, "OutlineColor", -1,-1,-1);
  config->fullcard_artoutline_visible = get_cfg_colordef(config, &config->fullcard_artoutline_color, section, "ArtOutlineColor", -1,-1,-1);

  TXT_AND_SHADOW(fullcard_title, "Title", 255,255,255, 47,47,47, 4,4);
  TXT_AND_SHADOW_ALL_DEFAULT(fullcard_powertoughness, "Powertoughness", fullcard_title);
  TXT_AND_SHADOW_ALL_DEFAULT(fullcard_type, "Type", fullcard_title);

  get_cfg_colordef(config, &config->fullcard_rulestext_color, section, "RulestextColor", 47,47,47);

  config->fullcard_title_on_light_visible = get_cfg_colordef(config, &config->fullcard_title_on_light_color, section, "TitleOnLightColor", -1,-1,-1);
  config->fullcard_title_on_white_black_visible = get_cfg_colordef(config, &config->fullcard_title_on_white_black_color, section, "TitleOnWhiteBlackColor", -1,-1,-1);
  config->fullcard_rules_on_dark_land_visible = get_cfg_colordef(config, &config->fullcard_rules_on_dark_land_color, section, "RulesOnDarkLandColor", -1,-1,-1);
  config->fullcard_rules_on_white_black_land_visible = get_cfg_colordef(config, &config->fullcard_rules_on_white_black_land_color, section, "RulesOnWhiteBlackLandColor", -1,-1,-1);

  sprintf(section, "Frames%s", config->subsection_name);

  get_cfg_int(config, &config->frames_percent_mixed_on_hybrid, section, "PercentMixedOnHybrid", 30);
  if (config->frames_percent_mixed_on_hybrid < 0)
	config->frames_percent_mixed_on_hybrid = -1;

  get_cfg_int(config, &config->frames_no_early_hybrid_land, section, "NoEarlyHybridLand", 0);
  get_cfg_int(config, &config->frames_two_color_as_hybrid, section, "TwoColorAsHybrid", 0);
  get_cfg_int(config, &config->frames_two_color_as_gold, section, "TwoColorAsGold", 0);
  get_cfg_int(config, &config->frames_translucent_colorless, section, "TranslucentColorless", -1);
  get_cfg_int(config, &config->frames_colorless_alpha, section, "ColorlessAlpha", 255);
  get_cfg_int(config, &config->frames_facedown_creature, section, "FaceDownCreature", -1);
  get_cfg_int(config, &config->frames_artifact_land, section, "ArtifactLand", 0);

  create_colorless_alpha_xform(config);


  char frame_base_dir[MAX_PATH + 1];
  get_cfg_str_raw(frame_base_dir, MAX_PATH, section, "FrameBaseDir", config == &configs[CFG_BASE] ? "." : base_frame_base_dir);
  if (config == &configs[CFG_BASE])
	base_frame_base_dir = strdup(frame_base_dir);

  char** frame_pic_name;
  if (config == &configs[CFG_BASE])
	frame_pic_name = &base_frame_pic_names[0];
  else
	frame_pic_name = &frames_pic_names[config->cardbk_base];

  for (i = 0; i < NUM_FRAMEPARTS_PER_FRAMESET; ++i)
	{
	  char filename[MAX_PATH + 1];
	  get_cfg_str_raw(filename, MAX_PATH, section, cfg_pic_names[i + CARDBK_MIN_FRAMEPART],
					  "%s%s",
					  config == &configs[CFG_BASE] ? cfg_pic_names[i + CARDBK_MIN_FRAMEPART] : base_frame_pic_names[i],
					  config == &configs[CFG_BASE] ? ".pic" : "");

	  if (frame_pic_name[i])
		free(frame_pic_name[i]);

	  if (config == &configs[CFG_BASE])
		frame_pic_name[i] = strdup(filename);
	  else
		{	// Add directory and canonicalize.
		  char path[2 * MAX_PATH + 2];
		  snprintf(path, 2 * MAX_PATH + 1, "%s\\%s", frame_base_dir, filename);

		  // replace [/\\]+ with \\ - it's obscene that PathCanonicalize() can't deal with multiple backslashes, or any forward slashes
		  char* p, *q;
		  for (p = q = path; *q; ++p, ++q)
			{
			  if (*q == '/' || *q == '\\')
				{
				  while (*(q + 1) == '/' || *(q + 1) == '\\')
					++q;
				  *p = '\\';
				}
			  else
				*p = *q;
			}

		  // simplify . and ..
		  char canonicalized[MAX_PATH + 1];
		  PathCanonicalize(canonicalized, path);

		  frame_pic_name[i] = strdup(canonicalized);
		}
	}

#undef TXT_AND_SHADOW
#undef TXT_AND_DEFAULT_SHADOW
#undef TXT_AND_SHADOW_ALL_DEFAULT
}

static int
prepare_fonts_and_imgs(void)
{
  char buf[3 * MAX_PATH + 1];

#define ADD_FONT(fontname)	do { strcpy(buf, base_dir); strcat(buf, fontname); AddFontResource(buf); } while (0)
  ADD_FONT(FONT1NAME);
  ADD_FONT(FONT3NAME);
  ADD_FONT(FONT4NAME);
  ADD_FONT(FONT5NAME);
  ADD_FONT(FONT6NAME);
  ADD_FONT(FONT7NAME);
#undef ADD_FONT

  int i, any_failures = 0;
  for (i = 0; frames_pic_names[i]; ++i)
	{
	  switch (BASE_FRAMEPART(i))
		{
		  case POWERTOUGHNESS_MODERN:
			if (!configs[FRAMESET_OF_FRAMEPART(i)].fullcard_powertoughness_box_visible)
			  continue;
			break;

		  case TOMBSTONE_MODERN:
			if (!configs[FRAMESET_OF_FRAMEPART(i)].fullcard_tombstone_visible)
			  continue;
			break;

		  case TYPE_ICON_MODERN:
			if (!configs[FRAMESET_OF_FRAMEPART(i)].fullcard_type_icon_visible)
			  continue;
			break;

		  case EXPANSION_SYMBOLS:
			{	// Don't read unless at least one set has visible expansions, either on smallcard or fullcard
			  int fs;
			  for (fs = 0; fs <= MAX_FRAMESET; ++fs)
				if (configs[fs].smallcard_expansion_visible || configs[fs].fullcard_expansion_visible)
				  goto found;
			  //otherwise
			  continue;
			found:
			  break;
			}
		}

	  int j;
	  for (j = 0; j < i; ++j)
		// If this has the same name as a previously loaded image, just copy the pointer.
		// Strcasecmp on the (previously canonicalized) filenames is sufficient for our purposes; opening the files and calling GetFileInformationByHandle() it the robust way, but it's much slower.
		if (pics[j] && !strcasecmp(frames_pic_names[i], frames_pic_names[j]))
		  {
			pics[i] = pics[j];
			frames_dup_of[i] = j;	// ...and mark it so we don't try to free it later
			goto outer_continue;
		  }

	  snprintf(buf, 3 * MAX_PATH + 1, "%s\\CARDART\\%s",
			   base_dir,
			   frames_pic_names[i]);
	  if (!(pics[i] = LoadImage(buf, palette, 0, 0)))
		{
		  popup("drawcardlib.dll", "Could not load %s=\n\"%s\"", get_pichandlename(i, 0), buf);
		  free(frames_pic_names[i]);
		  frames_pic_names[i] = strdup("\1");	// so anything else pointing to it gets a popup instead of being identified as a duplicate
		  any_failures = 1;
		}

	  if (i == WATERMARKS)
		{
		  BITMAP bmp;
		  GetObject(pics[i], sizeof(BITMAP), &bmp);
		  watermark_size_x = bmp.bmWidth / 5;
		  watermark_size_y = bmp.bmHeight / 4;
		}

	outer_continue:;
	}

  if (any_failures)
	{
	  del_resources();
	  return 0;
	}

  LOGFONT* logfont = cfg_font("BigCardTitle", 0);
  if ((unsigned int)logfont->lfWidth < 12)
	logfont->lfWidth = 24;
  for (i = BIGCARDTITLE_FONT; i <= BIGCARDTITLE_FONT9; ++i)
	{
	  fonts[i] = CreateFontIndirect(logfont);
	  logfont->lfWidth -= 1;
	}

  logfont = cfg_font("BigCardSubtitle", 0);
  if ((unsigned int)logfont->lfWidth < 12)
	logfont->lfWidth = 20;
  for (i = BIGCARDTYPE_FONT; i <= BIGCARDTYPE_FONT9; ++i)
	{
	  fonts[i] = CreateFontIndirect(logfont);
	  logfont->lfWidth -= 1;
	}

  logfont = cfg_font("BigCardPT", 0);
  fonts[BIGCARDPT_FONT] = CreateFontIndirect(logfont);

  logfont = cfg_font("BigCardText", 0);
  fonts[BIGCARDTEXT_FONT] = CreateFontIndirect(logfont);

  logfont = cfg_font("BigCardText2", 1);
  fonts[FLAVOR_FONT] = CreateFontIndirect(logfont);

  logfont = cfg_font("SmallCardTitle", 0);
  fonts[SMALLCARDTITLE_FONT] = CreateFontIndirect(logfont);

  if (!fonts[BIGCARDTITLE_FONT]
	  || !fonts[BIGCARDTITLE_FONT2]
	  || !fonts[BIGCARDTITLE_FONT3]
	  || !fonts[BIGCARDTITLE_FONT4]
	  || !fonts[BIGCARDTITLE_FONT5]
	  || !fonts[BIGCARDTITLE_FONT6]
	  || !fonts[BIGCARDTITLE_FONT7]
	  || !fonts[BIGCARDTITLE_FONT8]
	  || !fonts[BIGCARDTITLE_FONT9]
	  || !fonts[BIGCARDTYPE_FONT]
	  || !fonts[BIGCARDTYPE_FONT2]
	  || !fonts[BIGCARDTYPE_FONT3]
	  || !fonts[BIGCARDTYPE_FONT4]
	  || !fonts[BIGCARDTYPE_FONT5]
	  || !fonts[BIGCARDTYPE_FONT6]
	  || !fonts[BIGCARDTYPE_FONT7]
	  || !fonts[BIGCARDTYPE_FONT8]
	  || !fonts[BIGCARDTYPE_FONT9]
	  || !fonts[BIGCARDPT_FONT]
	  || !fonts[BIGCARDTEXT_FONT]
	  || !fonts[FLAVOR_FONT]
	  || !fonts[SMALLCARDTITLE_FONT])
	{
	  popup("drawcardlib.dll", "Could not load fonts");
	  del_resources();
	  return 0;
	}
  else
	return 1;
}

static LOGFONT*
cfg_font(const char* font_name, int italic)
{
  static LOGFONT fnt;
  memset(&fnt, 0, sizeof(LOGFONT));

  fnt.lfWidth = 4 * get_cfg_int2_raw(FONTSTXT, "width", font_name, 0);
  fnt.lfHeight = 4 * get_cfg_int2_raw(FONTSTXT, "size", font_name, 20);
  fnt.lfWeight = get_cfg_int2_raw(FONTSTXT, "bold", font_name, 0) ? 600 : 400;

  if (italic)
	fnt.lfItalic = 1;

  char buf[104];
  strcpy(buf, "font");
  strcat(buf, font_name);
  GetPrivateProfileString(FONTSTXT, buf, "MS Sans Serif", fnt.lfFaceName, LF_FACESIZE, duel_dat);
  return &fnt;
}

#define CFG_BASE_PTR(Typ, config, member) ((Typ*)((char*)&configs[CFG_BASE] + ((char*)member - (char*)config)))

static int
get_cfg_int_raw(const char* section, const char* key, int deflt)
{
  return GetPrivateProfileInt(section, key, deflt, duel_dat);
}

static void
get_cfg_int(Config* config, int* val, const char* section, const char* key, int deflt)
{
  if (config != &configs[CFG_BASE])
	deflt = *CFG_BASE_PTR(int, config, val);
  *val = GetPrivateProfileInt(section, key, deflt, duel_dat);
}

static int
get_cfg_int2_raw(const char* section, const char* key1, const char* key2, int deflt)
{
  char buf[104];
  strcpy(buf, key1);
  strcat(buf, key2);
  return GetPrivateProfileInt(section, buf, deflt, duel_dat);
}

// Sets rectangle parameters from *Left, *Top, *Width, *Height, with defaults given as left/right/width/height.
static void
get_cfg_rect(Config* config, RECT* rect, const char* section, const char* key, int deflt_l, int deflt_t, int deflt_w, int deflt_h)
{
  RECT* r;
  if (config != &configs[CFG_BASE])
	{
	  r = CFG_BASE_PTR(RECT, config, rect);
	  deflt_l = r->left;
	  deflt_t = r->top;
	}
  else
	r = NULL;

  rect->left = get_cfg_int2_raw(section, key, "Left", deflt_l);
  rect->top = get_cfg_int2_raw(section, key, "Top", deflt_t);

  if (r)
	{
	  deflt_w = r->right - r->left;
	  deflt_h = r->bottom - r->top;
	}

  rect->right = rect->left + get_cfg_int2_raw(section, key, "Width", deflt_w);
  rect->bottom = rect->top + get_cfg_int2_raw(section, key, "Height", deflt_h);
}

// Sets point parameters from *X and *Y.
static void
get_cfg_point(Config* config, POINT* pt, const char* section, const char* key, int deflt_x, int deflt_y)
{
  if (config != &configs[CFG_BASE])
	{
	  POINT* p = CFG_BASE_PTR(POINT, config, pt);
	  deflt_x = p->x;
	  deflt_y = p->y;
	}
  pt->x = get_cfg_int2_raw(section, key, "X", deflt_x);
  pt->y = get_cfg_int2_raw(section, key, "Y", deflt_y);
}

// Sets color from *R, *G, and *B and returns 1.  If any are out of range, sets to defaults and returns 0.
static int
get_cfg_colordef(Config* config, ColorDef* color, const char* section, const char* key, int deflt_r, int deflt_g, int deflt_b)
{
  if (config != &configs[CFG_BASE])
	{
	  ColorDef* deflt = CFG_BASE_PTR(ColorDef, config, color);
	  deflt_r = deflt->r;
	  deflt_g = deflt->g;
	  deflt_b = deflt->b;
	}

  color->r = get_cfg_int2_raw(section, key, "R", deflt_r);
  color->g = get_cfg_int2_raw(section, key, "G", deflt_g);
  color->b = get_cfg_int2_raw(section, key, "B", deflt_b);
  if (color->r >= 0 && color->r <= 255 && color->g >= 0 && color->g <= 255 && color->b >= 0 && color->b <= 255)
	{
	  color->colorref = RGB(color->r, color->g, color->b);
	  return 1;
	}
  else
	{
	  /* If base config's raw read r/g/b were out of range, this will end up being garbage; but that's ok, since they won't ever be drawn since visibility will
	   * be set to 0. */
	  color->colorref = RGB((uint8_t)deflt_r, (uint8_t)deflt_g, (uint8_t)deflt_b);
	  return 0;
	}
}

static void
get_cfg_str_raw(char* dest, int sz_of_dest, const char* section, const char* key, const char* deflt_fmt, ...)
{
  GetPrivateProfileString(section, key, "\1", dest, sz_of_dest, duel_dat);
  if (!strcmp(dest, "\1"))
	{
	  va_list args;
	  va_start(args, deflt_fmt);
	  vsnprintf(dest, sz_of_dest, deflt_fmt, args);
	  va_end(args);
	}
  dest[sz_of_dest - 1] = 0;
}

static void
del_screendc(void)
{
  if (screendc)
	{
	  DeleteDC(screendc);
	  screendc = 0;
	  DeleteCriticalSection(&critical_section);
	}
}

static void
del_resources(void)
{
  /*
   * Explicitly call these cleanup functions from CardArtLib.dll, since this dll gets unloaded first, and Cardartlib.dll will crash if it tries to destroy its
   * images after GdiplusShutdown has been called.  How Mok's build can depend on CardArtLib.dll but get unloaded after it is beyond me. -Korath
   */
  DestroyAllBigArts();
  DestroyAllSmallArts();

  int i;
  for (i = 0; i < MAX_PICHANDLES_PLUS_1; ++i)
	if (frames_dup_of[i] >= 0)
	  pics[i] = NULL;	// just set null if this image is a shallow copy of another
	else
	  del_obj(&pics[i]);

  for (i = 0; i < FRAME_MAX_LOADED + 2; ++i)
	if (frames_pic_names[i])
	  {
		free(frames_pic_names[i]);
		frames_pic_names[i] = NULL;
	  }

  for (i = 0; i <= MAX_FONTHANDLES; ++i)
	del_obj((HGDIOBJ*)(&fonts[i]));

  rem_font(FONT1NAME);
  rem_font(FONT3NAME);
  rem_font(FONT4NAME);
  rem_font(FONT5NAME);
  rem_font(FONT6NAME);
  rem_font(FONT7NAME);

  del_obj((HGDIOBJ*)(&palette));
  close_gdiplus();
}

static int
is_token_by_instance(card_instance_t* instance)
{
  return ((instance->token_status & STATUS_TOKEN)
		  && !(instance->targets[14].card > 0 && (instance->targets[14].card & SF_UNEARTH)));
}

static PicHandleNames
select_frame(const card_ptr_t* cp, const Rarity* r)
{
  int typ;
  int clr;
  card_instance_t* instance = NULL;

  if ((cp->color & 0xFFFE) == 0xC3D4)	// magic number indicating player and card are encoded into color
	{
	  if (parent != PARENT_MANALINK)
		{
		  popup("drawcardlib.dll", "Encoded instance found, but running from %s, not Manalink",
				parent == PARENT_SHANDALAR ? "Shandalar" : "non-Manalink, non-Shandalar (deckbuilder?)");
		  abort();
		}
	  instance = get_displayed_card_instance(cp->color & 0x1, HIWORD(cp->color));

	  /* Activation cards on the stack, other than the top one, for some reason get their colors overwritten by 1.  This can be observed with the original
	   * executable/drawcardlib by picking a permanent that has a repeatable activation ability (I used Figure of Destiny), changing its color (I used
	   * Lifelace), activating it, and activating it again in response.  Double-right-clicking the top activation card on the stack will show the permanent with
	   * its laced color (green in my test); double-right-clicking the earlier activation card on the stack will show the card's default color (gold here).  I
	   * can't find what's doing the overwriting, so can't comment on whether it's intentional.  Note that this won't affect cards that are actually supposed to
	   * be temporarily colorless; they end up with 0, not 1, for color. */

	  typ = cards_data_exe[instance->internal_card_id].type;

	  if (typ & TYPE_EFFECT)
		typ |= cards_data_exe[instance->original_internal_card_id].type;

	  if (instance->color == COLOR_TEST_COLORLESS)
		{
		  if (cp->id == CARD_ID_GHOSTFIRE)	// Actually *supposed* to be COLOR_TEST_COLORLESS, and manacost_to_count_colors() will be wrong
			clr = 0;
		  else
			clr = manacost_to_count_colors(cp);

		  if (clr == 0 && !(typ & (TYPE_ARTIFACT | TYPE_LAND)))	// no mana cost (like, say, Ancestral Vision, Evermind, Garruk Relentless, or a token)
			switch (cards_ptr_exe[cards_data_exe[((typ & TYPE_EFFECT) ? (int32_t)instance->original_internal_card_id : instance->internal_card_id)].id]->color)
			  {
				case CP_COLOR_BLACK:	clr = COLOR_TEST_BLACK;	break;
				case CP_COLOR_BLUE:		clr = COLOR_TEST_BLUE;	break;
				case CP_COLOR_MULTI:	clr = COLOR_TEST_ANY & ~COLOR_TEST_COLORLESS;	break;	// force gold frame
				case CP_COLOR_GREEN:	clr = COLOR_TEST_GREEN;	break;
				case CP_COLOR_RED:		clr = COLOR_TEST_RED;	break;
				case CP_COLOR_WHITE:	clr = COLOR_TEST_WHITE;	break;
				case CP_COLOR_SPECIAL:	clr = -2;				break;

				case CP_COLOR_LESS:
				case CP_COLOR_ARTIFACT:
				case CP_COLOR_LAND:
				default:
				  clr = 0;
				  break;
			  }
		  else
			clr &= (COLOR_TEST_ANY & ~COLOR_TEST_COLORLESS); // i.e., the five colors, but not colorless
		}
	  else
		{
		  clr = instance->color;
		  clr &= (COLOR_TEST_ANY & ~COLOR_TEST_COLORLESS); // i.e., the five colors, but not colorless
		}
	}
  else
	{
	  switch (cp->card_type)
		{
		  case CP_TYPE_ARTIFACT:
			typ = TYPE_ARTIFACT;
			break;

		  case CP_TYPE_LAND:
			if (cp->id == CARD_ID_ANCIENT_DEN
				|| cp->id == CARD_ID_DARKSTEEL_CITADEL
				|| cp->id == CARD_ID_GREAT_FURNACE
				|| cp->id == CARD_ID_SEAT_OF_THE_SYNOD
				|| cp->id == CARD_ID_TREE_OF_TALES
				|| cp->id == CARD_ID_VAULT_OF_WHISPERS)
			  typ = TYPE_ARTIFACT | TYPE_LAND;
			else
			  typ = TYPE_LAND;
			break;

		  default:
			typ = 0;
			break;
		}

	  switch (cp->color)
		{
		  case CP_COLOR_BLACK:		clr = COLOR_TEST_BLACK;	break;
		  case CP_COLOR_BLUE:		clr = COLOR_TEST_BLUE;	break;
		  case CP_COLOR_MULTI:		clr = -1;				break;
		  case CP_COLOR_GREEN:		clr = COLOR_TEST_GREEN;	break;
		  case CP_COLOR_RED:		clr = COLOR_TEST_RED;	break;
		  case CP_COLOR_WHITE:		clr = COLOR_TEST_WHITE;	break;
		  case CP_COLOR_SPECIAL:	clr = -2;				break;

		  case CP_COLOR_LESS:
		  case CP_COLOR_ARTIFACT:
		  case CP_COLOR_LAND:
		  default:
			clr = 0;
			break;
		}
	}

  if (force_frameset >= 0)
	cfg = &configs[force_frameset];
  else if (instance && is_token_by_instance(instance))
	cfg = &configs[CFG_TOKEN];
  else if (cp->types1 == SUBTYPE_PLANESWALKER)
	cfg = &configs[CFG_PLANESWALKER];
  else if ((r->flags & RARITYFLAG_TIMESHIFT)
		   && r->default_expansion == EXPANSION_PLANAR_CHAOS)
	cfg = &configs[CFG_TIMESHIFT];
  else if ((r->flags & RARITYFLAG_TIMESHIFT)
		   && r->default_expansion == EXPANSION_FUTURE_SIGHT)
	cfg = &configs[CFG_FUTURESHIFT];
  else if (r->default_expansion < EXPANSION_EIGHTH_EDITION
		   || r->default_expansion == EXPANSION_MASTERS_EDITION
		   || r->default_expansion == EXPANSION_MASTERS_EDITION_II
		   || r->default_expansion == EXPANSION_MASTERS_EDITION_III
		   || r->default_expansion == EXPANSION_MASTERS_EDITION_IV
		   || (r->default_expansion == EXPANSION_TIME_SPIRAL && r->default_rarity == SPECIAL))
	cfg = &configs[CFG_CLASSIC];
  else
	cfg = &configs[CFG_MODERN];

  int nyx;
  if (force_nyx >= 0)
	nyx = force_nyx;
  else if (cfg == &configs[CFG_PLANESWALKER])
	// This fails if the card has become part-enchantment by means other than Enchanted Evening.  The only way to do so that I'm aware of is for something
	// like Memnarch to change it to an artifact, and then a different player to Copy Artifact it.
	nyx = instance && instance->targets[14].card >= 0 && (instance->targets[14].card & SF_ENCHANTED_EVENING);
  else if (instance
		   && (instance->state & (STATE_OUBLIETTED | STATE_IN_PLAY | STATE_JUST_CAST | STATE_INVISIBLE)))	// More ugly hacks to get around Theros enchantment-creatures' base cards being non-creatures
	nyx = ((typ & TYPE_NONENCHANTMENT)
		   && ((typ & TYPE_ENCHANTMENT) || (instance->targets[14].card >= 0 && (instance->targets[14].card & SF_ENCHANTED_EVENING))));
  else
	{
#ifndef USE_TYPES_FOR_NYX
	  nyx = r->flags & RARITYFLAG_NYX;
#else
	  // Since the base cards are the pure-enchantment versions, this isn't currently workable for the gods or bestow creatures (which leaves only the artifacts)
	  int enchantment = 0, nonenchantment = 0;
	  const int16_t* p;
	  for (p = &cp->types1; p <= &cp->types7; ++p)
		switch (*p)
		  {
			case SUBT_ENCHANTMENT:
			  enchantment = 1;
			  if (nonenchantment)
				{
				  nyx = 1;
				  goto found;
				}
			  break;
			case SUBT_ARTIFACT:
			case SUBT_CREATURE:
			case SUBT_LAND:
			case SUBT_PLANESWALKER:
			  nonenchantment = 1;
			  if (enchantment)
				{
				  nyx = 1;
				  goto found;
				}
			  break;
		  }
	  // else
	  nyx = 0;
	found:;
#endif
	}

  // An ugly hack to make sure Theros enchantment-creatures show their power/toughness, even though the base cards are non-creature versions
  if ((r->flags & RARITYFLAG_NYX)
	  && cp->toughness <= 0
	  && parent == PARENT_MANALINK
	  && (!instance || !(instance->state & (STATE_OUBLIETTED | STATE_IN_PLAY | STATE_JUST_CAST | STATE_INVISIBLE)))
	  && (cp->id == CARD_ID_EREBOS_GOD_OF_THE_DEAD
		  || cp->id == CARD_ID_HELIOD_GOD_OF_THE_SUN
		  || cp->id == CARD_ID_NYLEA_GOD_OF_THE_HUNT
		  || cp->id == CARD_ID_PURPHOROS_GOD_OF_THE_FORGE
		  || cp->id == CARD_ID_THASSA_GOD_OF_THE_SEA))
	{
	  const card_ptr_t* base_cp = cards_ptr_exe[cp->id + 1];
	  hack_power = base_cp->power;
	  hack_toughness = base_cp->toughness;
	}
  else
	{
	  hack_power = cp->power;
	  hack_toughness = cp->toughness;
	}

  PicHandleNames frameset = cfg->frame_base;
  if (nyx)
	frameset += (FRAME_MODERN_NYX_MIN - FRAME_MODERN_MIN);

  PicHandleNames frame = frameset;

  if (clr == -2)
	frame = CARDBK_MODERN_SPECIAL;
  else if (typ & TYPE_LAND)
	{
	  int mana_clr;
	  if (instance)
		mana_clr = instance->mana_color;
	  else
		mana_clr = cp->mana_source_colors;

	  mana_clr &= (COLOR_TEST_ANY & ~COLOR_TEST_COLORLESS); // i.e., the five colors, but not colorless

	  if (typ & TYPE_ARTIFACT
		  && cfg->frames_artifact_land)
		switch (mana_clr)
		  {
			case COLOR_TEST_BLACK:	frame = FRAME_MODERN_ARTIFACT_LAND_BLACK;		break;
			case COLOR_TEST_BLUE:	frame = FRAME_MODERN_ARTIFACT_LAND_BLUE;		break;
			case COLOR_TEST_GREEN:	frame = FRAME_MODERN_ARTIFACT_LAND_GREEN;		break;
			case COLOR_TEST_RED:	frame = FRAME_MODERN_ARTIFACT_LAND_RED;			break;
			case COLOR_TEST_WHITE:	frame = FRAME_MODERN_ARTIFACT_LAND_WHITE;		break;
			case 0:					frame = FRAME_MODERN_ARTIFACT_LAND_COLORLESS;	break;
			default:
			  if (cfg->frames_percent_mixed_on_hybrid >= 0
				  && !(cfg->frames_no_early_hybrid_land
					   && (!r || r->default_expansion < EXPANSION_CLASSIC_SIXTH_EDITION)))
				frame = count_colors_to_hybrid_pic_handle_name(mana_clr, FRAME_MODERN_ARTIFACT_LAND_GOLD_MIN, FRAME_MODERN_ARTIFACT_LAND_GOLD);
			  else
				frame = FRAME_MODERN_ARTIFACT_LAND_GOLD;
			  break;
		  }
	  else
		switch (mana_clr)
		  {
			case COLOR_TEST_BLACK:	frame = FRAME_MODERN_LAND_BLACK;		break;
			case COLOR_TEST_BLUE:	frame = FRAME_MODERN_LAND_BLUE;			break;
			case COLOR_TEST_GREEN:	frame = FRAME_MODERN_LAND_GREEN;		break;
			case COLOR_TEST_RED:	frame = FRAME_MODERN_LAND_RED;			break;
			case COLOR_TEST_WHITE:	frame = FRAME_MODERN_LAND_WHITE;		break;
			case 0:					frame = FRAME_MODERN_LAND_COLORLESS;	break;
			default:
			  if (cfg->frames_percent_mixed_on_hybrid >= 0
				  && !(cfg->frames_no_early_hybrid_land
					   && (!r || r->default_expansion < EXPANSION_CLASSIC_SIXTH_EDITION)))
				frame = count_colors_to_hybrid_pic_handle_name(mana_clr, FRAME_MODERN_LAND_GOLD_MIN, FRAME_MODERN_LAND_GOLD);
			  else
				frame = FRAME_MODERN_LAND_GOLD;
			  break;
		  }
	}
  else if (typ & TYPE_ARTIFACT)
	switch (clr)
	  {
		case COLOR_TEST_BLACK:	frame = FRAME_MODERN_ARTIFACT_BLACK;		break;
		case COLOR_TEST_BLUE:	frame = FRAME_MODERN_ARTIFACT_BLUE;			break;
		case COLOR_TEST_GREEN:	frame = FRAME_MODERN_ARTIFACT_GREEN;		break;
		case COLOR_TEST_RED:	frame = FRAME_MODERN_ARTIFACT_RED;			break;
		case COLOR_TEST_WHITE:	frame = FRAME_MODERN_ARTIFACT_WHITE;		break;
		case 0:					frame = FRAME_MODERN_ARTIFACT_COLORLESS;	break;
		default:
		  if (cfg->frames_percent_mixed_on_hybrid >= 0 && !cfg->frames_two_color_as_gold)
			{
			  if (clr == -1)
				clr = manacost_to_count_colors(cp);
			  frame = count_colors_to_hybrid_pic_handle_name(clr, FRAME_MODERN_ARTIFACT_GOLD_MIN, FRAME_MODERN_ARTIFACT_GOLD);
			}
		  else
			frame = FRAME_MODERN_ARTIFACT_GOLD;
		  break;
	  }
  else
	switch (clr)
	  {
		case COLOR_TEST_BLACK:	frame = FRAME_MODERN_SPELL_BLACK;		break;
		case COLOR_TEST_BLUE:	frame = FRAME_MODERN_SPELL_BLUE;		break;
		case COLOR_TEST_GREEN:	frame = FRAME_MODERN_SPELL_GREEN;		break;
		case COLOR_TEST_RED:	frame = FRAME_MODERN_SPELL_RED;			break;
		case COLOR_TEST_WHITE:	frame = FRAME_MODERN_SPELL_WHITE;		break;
		case 0:					frame = FRAME_MODERN_SPELL_COLORLESS;	break;
		default:
		  if (cfg->frames_percent_mixed_on_hybrid >= 0)
			{
			  /* This is redundant to the previous call in the 0xC3D4 check only for fullcards of activation calls that are on the stack but not on the top,
			   * which I think is an unusual enough occurrence to not worry about. */
			  CountColors colors = manacost_to_count_colors(cp);
			  if (clr == -1)
				clr = colors & (COLOR_TEST_ANY & ~COLOR_TEST_COLORLESS);
			  /* Otherwise, we already have its current colors from the instance check above.  We still need to compute colors, however, solely to determine
			   * whether it's a hybrid card; in which case we draw the hybrid frame with its current colors, even if they're different. */

			  if (((colors & COUNT_COLOR_HYBRID)			// at least one hybrid mana symbol
				   && !(colors & COUNT_COLOR_MONO))			// no non-hybrid colored mana symbols
				  || cfg->frames_two_color_as_hybrid == 2	// And now the second call to manacost_to_count_colors() is redundant again.  Oh well.
				  || (cfg->frames_two_color_as_hybrid == 1
					  && (unsigned int)clr != (colors & (COLOR_TEST_ANY & ~COLOR_TEST_COLORLESS))))	// not original colors
				frame = count_colors_to_hybrid_pic_handle_name(clr, FRAME_MODERN_SPELL_HYBRID_MIN, FRAME_MODERN_SPELL_GOLD);
			  else if (cfg->frames_two_color_as_gold)
				frame = FRAME_MODERN_SPELL_GOLD;
			  else
				frame = count_colors_to_hybrid_pic_handle_name(clr, FRAME_MODERN_SPELL_GOLD_MIN, FRAME_MODERN_SPELL_GOLD);
			}
		  else
			frame = FRAME_MODERN_SPELL_GOLD;
		  break;
	  }

  if (frame == FRAME_MODERN_SPELL_COLORLESS && cp->id == CARD_ID_FACE_DOWN_CREATURE && cfg->frames_facedown_creature >= 0)
	frame = CARDBACK;
  else if (frame == CARDBK_MODERN_SPECIAL)	// on the frameparts scale, not the frame one
	frame = cfg->cardbk_base + (frame - CARDBK_MIN_FRAMEPART);
  else
	frame = frameset + (frame - FRAME_MODERN_MIN);

  if (!gpics[frame])
	make_frame(frame);

  return frame;
}

static RECT*
draw_smallcard_art(HDC hdc, const RECT* card_rect, int id, int version, int must_save_dc, int shrink_rect, int intersect_clip_with_shrunk_rect)
{
  int savedc = must_save_dc ? SaveDC(hdc) : 0;

  SetMapMode(hdc, MM_TEXT);

  static RECT small_art_rect;

  if (shrink_rect)
	{
	  int wid = card_rect->right - card_rect->left;
	  int hgt = card_rect->bottom - card_rect->top;

	  int art_left = (wid * cfg->smallcard_art.left) / 1000;
	  int art_top = (hgt * cfg->smallcard_art.top) / 1000;
	  int art_wid = (wid * (cfg->smallcard_art.right - cfg->smallcard_art.left)) / 1000;
	  int art_hgt = (hgt * (cfg->smallcard_art.bottom - cfg->smallcard_art.top)) / 1000;

	  SetRect(&small_art_rect, art_left, art_top, art_wid + art_left, art_hgt + art_top);

	  if (intersect_clip_with_shrunk_rect)
		IntersectClipRect(hdc, small_art_rect.left, small_art_rect.top, small_art_rect.right, small_art_rect.bottom);
	  else
		card_rect = &small_art_rect;
	}

  if (IsSmallArtIn(id, version))
	ReloadSmallArtIfWrongSize(id, version, card_rect->right - card_rect->left, card_rect->bottom - card_rect->top);
  else
	LoadSmallArt(id, version, card_rect->right - card_rect->left, card_rect->bottom - card_rect->top);

  DrawSmallArt(hdc, card_rect, id, version);

  if (must_save_dc)
	RestoreDC(hdc, savedc);

  return &small_art_rect;
}

int
DrawSmallCard(HDC hdc, const RECT* card_rect, const card_ptr_t* cp, int version)
{
  if (!hdc || !card_rect || !cp)
	return 0;

  EnterCriticalSection(&critical_section);
  int savedc = SaveDC(hdc);

  SetMapMode(hdc, MM_ANISOTROPIC);
  SetWindowExtEx(hdc, 800, 1120, NULL);
  SetViewportExtEx(hdc, card_rect->right - card_rect->left, card_rect->bottom - card_rect->top, NULL);
  SetWindowOrgEx(hdc, 0, 0, NULL);

  const Rarity* r = get_rarity(cp->id);
  PicHandleNames framenum = select_frame(cp, r);
  int colorless_option;
  if (framenum == CARDBACK)
	{
	  colorless_option = cfg->frames_facedown_creature;
	  framenum = FRAME_MODERN_SPELL_COLORLESS;
	}
  else
	colorless_option = cfg->frames_translucent_colorless;

#pragma message "suppress for nyx frames"
  int translucent = 0;
  if (BASE_FRAME(framenum) == FRAME_MODERN_SPELL_COLORLESS && colorless_option >= 0)
	{
	  draw_smallcard_art(hdc, card_rect, cp->id, version, 1, 0, 0);
	  framenum = FRAMEPART_FROM_BASE(CARDOV_MODERN_COLORLESS, cfg);
	  translucent = 1;
	}
  else if (cfg->smallcard_art_first)
	draw_smallcard_art(hdc, card_rect, cp->id, version, 1, 1, 0);

  if (!(translucent && (colorless_option & FRAMES_TRANSLUCENT_COLORLESS_NO_SMALLCARD_FRAME)))
	draw_smallcard_frame(hdc, card_rect, framenum, translucent);

  if (cfg->smallcard_expansion_visible && r)
	blt_expansion_symbol(hdc, &cfg->smallcard_expansion, r->default_expansion, r->default_rarity, 1);

  draw_small_card_title_impl(hdc, card_rect, cp->name, 0, 1);

  if (cfg->smallcard_outline_visible)
	{
	  SelectObject(hdc, GetStockObject(NULL_BRUSH));
	  HPEN pen = CreatePen(PS_SOLID, 0, cfg->smallcard_outline_color.colorref);
	  SelectObject(hdc, pen);
	  Rectangle(hdc, 0, 0, 800, 1120);
	  SelectObject(hdc, GetStockObject(NULL_PEN));
	  if (pen)
		DeleteObject(pen);
	}

  /************************************************************************
   * frame		art1st->artfirst	!artfirst							  *
   * none				no			yes									  *
   * ovcolorless		no			no									  *
   * ovcolorless+redraw	yes			yes									  *
   *																	  *
   *    So if smallcard_art_first is set, only draw art here if we both   *
   * have the overlay-only colorless frame and the redraw art option set. *
   *    If smallcard_art_first isn't set, also draw if we don't have the  *
   * overlay-only colorless frame.										  *
   ************************************************************************/

  if (cfg->smallcard_art_first ? (translucent && (colorless_option & FRAMES_TRANSLUCENT_COLORLESS_REDRAW_SMALLCARD_ART))
	  : (!translucent || (colorless_option & FRAMES_TRANSLUCENT_COLORLESS_REDRAW_SMALLCARD_ART)))
	{
	  RECT* small_art_rect = draw_smallcard_art(hdc, card_rect, cp->id, version, 0, 1, translucent);

	  if (cfg->smallcard_artoutline_visible)
		{
		  SelectObject(hdc, GetStockObject(NULL_BRUSH));
		  HPEN pen = CreatePen(PS_SOLID, 0, cfg->smallcard_artoutline_color.colorref);
		  SelectObject(hdc, pen);
		  Rectangle(hdc, small_art_rect->left, small_art_rect->top, small_art_rect->right, small_art_rect->bottom);
		  if (pen)
			DeleteObject(pen);
		}
	}

  int rval = RestoreDC(hdc, savedc);
  LeaveCriticalSection(&critical_section);
  return rval;
}

static CountColors
manacost_to_count_colors(const card_ptr_t* cp)
{
  if (!cp)
	return 0;

  CountColors rval = 0;

  if (*(const int32_t*)(&cp->unknown0x51[3]))
	{
	  const char* q = *(const char* const *)(&cp->unknown0x51[3]);
	  while (*q)
		if (*q != '|')
		  ++q;
		else
		  rval |= mana_tag_to_count_colors[convert_initial_mana_tag(&q)];
	}
  else
	{
	  // old style calculated from card_ptr_t values
	  if (cp->req_black > 0)
		rval |= COUNT_COLOR_MONO | COUNT_COLOR_BLACK;
	  if (cp->req_blue > 0)
		rval |= COUNT_COLOR_MONO | COUNT_COLOR_BLUE;
	  if (cp->req_green > 0)
		rval |= COUNT_COLOR_MONO | COUNT_COLOR_GREEN;
	  if (cp->req_red > 0)
		rval |= COUNT_COLOR_MONO | COUNT_COLOR_RED;
	  if (cp->req_white > 0)
		rval |= COUNT_COLOR_MONO | COUNT_COLOR_WHITE;
	}

  return rval;
}

static PicHandleNames
count_colors_to_hybrid_pic_handle_name(CountColors cc, PicHandleNames base, PicHandleNames deflt)
{
  switch (cc & COUNT_COLOR_COLOR_MASK)
	{
	  case COUNT_COLOR_WHITE | COUNT_COLOR_BLUE:	return base + (FRAME_MODERN_SPELL_HYBRID_WHITE_BLUE - FRAME_MODERN_SPELL_HYBRID_MIN);
	  case COUNT_COLOR_BLUE | COUNT_COLOR_BLACK:	return base + (FRAME_MODERN_SPELL_HYBRID_BLUE_BLACK - FRAME_MODERN_SPELL_HYBRID_MIN);
	  case COUNT_COLOR_BLACK | COUNT_COLOR_RED:		return base + (FRAME_MODERN_SPELL_HYBRID_BLACK_RED - FRAME_MODERN_SPELL_HYBRID_MIN);
	  case COUNT_COLOR_RED | COUNT_COLOR_GREEN:		return base + (FRAME_MODERN_SPELL_HYBRID_RED_GREEN - FRAME_MODERN_SPELL_HYBRID_MIN);
	  case COUNT_COLOR_GREEN | COUNT_COLOR_WHITE:	return base + (FRAME_MODERN_SPELL_HYBRID_GREEN_WHITE - FRAME_MODERN_SPELL_HYBRID_MIN);

	  case COUNT_COLOR_WHITE | COUNT_COLOR_BLACK:	return base + (FRAME_MODERN_SPELL_HYBRID_WHITE_BLACK - FRAME_MODERN_SPELL_HYBRID_MIN);
	  case COUNT_COLOR_BLACK | COUNT_COLOR_GREEN:	return base + (FRAME_MODERN_SPELL_HYBRID_BLACK_GREEN - FRAME_MODERN_SPELL_HYBRID_MIN);
	  case COUNT_COLOR_GREEN | COUNT_COLOR_BLUE:	return base + (FRAME_MODERN_SPELL_HYBRID_GREEN_BLUE - FRAME_MODERN_SPELL_HYBRID_MIN);
	  case COUNT_COLOR_BLUE | COUNT_COLOR_RED:		return base + (FRAME_MODERN_SPELL_HYBRID_BLUE_RED - FRAME_MODERN_SPELL_HYBRID_MIN);
	  case COUNT_COLOR_RED | COUNT_COLOR_WHITE:		return base + (FRAME_MODERN_SPELL_HYBRID_RED_WHITE - FRAME_MODERN_SPELL_HYBRID_MIN);

	  default:										return deflt;
	}
}

static int
cost_values_to_cost_text(const card_ptr_t* cp, char* dest)
{
  char buf[104];
  unsigned int cost_colorless;
  int cost_black, cost_blue, cost_green, cost_red, cost_white;

  if (!cp || !dest)
	return 0;

  cost_black = cp->req_black;
  cost_blue = cp->req_blue;
  cost_green = cp->req_green;
  cost_red = cp->req_red;
  cost_white = cp->req_white;
  cost_colorless = cp->req_colorless;

  char* p = buf;
  if (cost_colorless)
	{
	  if ((int)cost_colorless == -1
		  || cost_colorless == 72
		  || cost_colorless == 40)
		{
		  *p++ = '|';
		  *p++ = 'X';
		}
	  else if (cost_colorless <= 9)
		{
		  *p++ = '|';
		  *p++ = '0' + cost_colorless;
		}
	  else if (cost_colorless <= 16)
		{
		  *p++ = '|';
		  *p++ = '1';
		  *p++ = '0' + cost_colorless - 10;
		}
	  // otherwise just displays nothing!
	}

#define COLORED_MANA(var, chr)					\
  for (; var != 0; --var)						\
	{											\
	  *p++ = '|';								\
	  *p++ = chr;								\
	}

  COLORED_MANA(cost_white,	'W');
  COLORED_MANA(cost_green,	'G');
  COLORED_MANA(cost_red,	'R');
  COLORED_MANA(cost_black,	'B');
  COLORED_MANA(cost_blue,	'U');

#undef COLORED_MANA

#ifdef BUG_FOR_BUG_COMPATIBLE
  // This fail to append |0 for colorless cost >= 17 (where nothing is shown)
  // Plus, whenever it does append |0, it doesn't terminate the string.
  *p = 0;

  if ((uint8_t)cost_colorless == 0 && p == buf)
	{
	  *p++ = '|';
	  *p++ = '0';
	}
#else
  if (p == buf)
	{
	  *p++ = '|';
	  *p++ = '0';
	}

  *p = 0;
#endif

  strcpy(dest, buf);

  return p - buf;
}

/*
 * This takes a pointer to a string.  If it starts with a mana tag, it advances the pointer to the next char after the tag and returns the mana tag's code.
 * If it starts with a | but not a valid mana tag, it advances the pointer by one and returns 0.
 * If it doesn't start with a mana tag, it just returns 0.
 */
static int
convert_initial_mana_tag(const char** src_string)
{
  const char* p = *src_string;

  if (*p != '|')
	return 0;

  ++p;

  int16_t needle = *(const int16_t*)p;

  signed char mana_tag_tag;
  signed char* mana_tag_entry;
  for (mana_tag_entry = mana_tags; ((mana_tag_tag = *mana_tag_entry)); mana_tag_entry += 3)
	{
	  int16_t key = *(int16_t*)(mana_tag_entry + 1);
	  if (!(key & 0xff00))
		{
		  if ((int8_t)needle == (int8_t)key)
			break;
		}
	  else if (needle == key)
		{
		  p = p + 1;
		  break;
		}
	}

  if (mana_tag_tag)
	++p;

  *src_string = p;

  return (unsigned char)mana_tag_tag;
}

static void
make_frame(PicHandleNames frame)
{
  switch (frame)
	{
	  case CARDBACK:	case MANASYMBOLS:	case EXPANSION_SYMBOLS:	case WATERMARKS:
	  case CARDBK_MIN_FRAMEPART ... CARDBK_MAX_FRAMEPART_ALL:
		make_gpic_from_pic(frame);
		break;

	  case MAX_PICHANDLES_PLUS_1:
		popup("drawcardlib.dll", "Attempted to make_frame(%s)", get_pichandlename(frame, 0));
		abort();

#define CFG(frameset)	(configs[CFG_##frameset])

#define MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(background, overlay_normal, nyx, overlay_nyx)	\
		if (nyx)																		\
		  make_frame_from_2_overlays(frame, background, overlay_nyx, overlay_normal);	\
		else																			\
		  make_frame_from_overlay(frame, background, overlay_normal)

#define MAKE_MONOCOLORS_WITH_VARYING_BACKGROUND(frameset, framename, nyx)	\
		   framename##BLACK:	MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(CARDBK_##frameset##_BLACK, CARDOV_##frameset##_B, nyx, CARDOV_##frameset##_BNYX);	break;	\
	  case framename##BLUE:		MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(CARDBK_##frameset##_BLUE,  CARDOV_##frameset##_U, nyx, CARDOV_##frameset##_UNYX);	break;	\
	  case framename##GREEN:	MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(CARDBK_##frameset##_GREEN, CARDOV_##frameset##_G, nyx, CARDOV_##frameset##_GNYX);	break;	\
	  case framename##RED:		MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(CARDBK_##frameset##_RED,   CARDOV_##frameset##_R, nyx, CARDOV_##frameset##_RNYX);	break;	\
	  case framename##WHITE:	MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(CARDBK_##frameset##_WHITE, CARDOV_##frameset##_W, nyx, CARDOV_##frameset##_WNYX)

#define MAKE_MONOCOLORS(frameset, framename, base, suffix, nyx, nyxsuffix)	\
		   framename##BLACK:	MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(base, CARDOV_##frameset##_B##suffix, nyx, CARDOV_##frameset##_##nyxsuffix##_NYX);	break;	\
	  case framename##BLUE:		MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(base, CARDOV_##frameset##_U##suffix, nyx, CARDOV_##frameset##_##nyxsuffix##_NYX);	break;	\
	  case framename##GREEN:	MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(base, CARDOV_##frameset##_G##suffix, nyx, CARDOV_##frameset##_##nyxsuffix##_NYX);	break;	\
	  case framename##RED:		MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(base, CARDOV_##frameset##_R##suffix, nyx, CARDOV_##frameset##_##nyxsuffix##_NYX);	break;	\
	  case framename##WHITE:	MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(base, CARDOV_##frameset##_W##suffix, nyx, CARDOV_##frameset##_##nyxsuffix##_NYX)

#define MAKE_ONE_MONOCOLOR_ARTIFACT_LAND(frameset, framename, letter, color)	\
	  FRAME_##framename##_ARTIFACT_LAND_##color:								\
		if (CFG(frameset).frames_artifact_land == 3)							\
		  make_hybrid_frame(&CFG(frameset), frame, FRAME_##framename##_ARTIFACT_##color, FRAME_##framename##_LAND_##color);	\
		else																	\
		  make_frame_from_overlay(frame, FRAME_##framename##_ARTIFACT_LAND_COLORLESS, CARDOV_##frameset##_##letter##LAND)

#define MAKE_MONOCOLOR_ARTIFACT_LANDS(frameset, framename)							\
		   MAKE_ONE_MONOCOLOR_ARTIFACT_LAND(frameset, framename, B, BLACK);	break;	\
	  case MAKE_ONE_MONOCOLOR_ARTIFACT_LAND(frameset, framename, U, BLUE);	break;	\
	  case MAKE_ONE_MONOCOLOR_ARTIFACT_LAND(frameset, framename, G, GREEN);	break;	\
	  case MAKE_ONE_MONOCOLOR_ARTIFACT_LAND(frameset, framename, R, RED);	break;	\
	  case MAKE_ONE_MONOCOLOR_ARTIFACT_LAND(frameset, framename, W, WHITE)

#define MAKE_HYBRIDS(frameset, hybrid, half)																\
		   hybrid##WHITE_BLUE:	make_hybrid_frame(&CFG(frameset), frame, half##WHITE, half##BLUE);	break;	\
	  case hybrid##BLUE_BLACK:	make_hybrid_frame(&CFG(frameset), frame, half##BLUE,  half##BLACK);	break;	\
	  case hybrid##BLACK_RED:	make_hybrid_frame(&CFG(frameset), frame, half##BLACK, half##RED);	break;	\
	  case hybrid##RED_GREEN:	make_hybrid_frame(&CFG(frameset), frame, half##RED,   half##GREEN);	break;	\
	  case hybrid##GREEN_WHITE:	make_hybrid_frame(&CFG(frameset), frame, half##GREEN, half##WHITE);	break;	\
	  case hybrid##WHITE_BLACK:	make_hybrid_frame(&CFG(frameset), frame, half##WHITE, half##BLACK);	break;	\
	  case hybrid##BLACK_GREEN:	make_hybrid_frame(&CFG(frameset), frame, half##BLACK, half##GREEN);	break;	\
	  case hybrid##GREEN_BLUE:	make_hybrid_frame(&CFG(frameset), frame, half##GREEN, half##BLUE);	break;	\
	  case hybrid##BLUE_RED:	make_hybrid_frame(&CFG(frameset), frame, half##BLUE,  half##RED);	break;	\
	  case hybrid##RED_WHITE:	make_hybrid_frame(&CFG(frameset), frame, half##RED,   half##WHITE)

#define MAKE_HYBRIDS_WITH_BAR(frameset, hybrid, half, bar)																		\
		   hybrid##WHITE_BLUE:	make_hybrid_frame_with_overlay(&CFG(frameset), frame, half##WHITE, half##BLUE, bar);	break;	\
	  case hybrid##BLUE_BLACK:	make_hybrid_frame_with_overlay(&CFG(frameset), frame, half##BLUE,  half##BLACK, bar);	break;	\
	  case hybrid##BLACK_RED:	make_hybrid_frame_with_overlay(&CFG(frameset), frame, half##BLACK, half##RED, bar);		break;	\
	  case hybrid##RED_GREEN:	make_hybrid_frame_with_overlay(&CFG(frameset), frame, half##RED,   half##GREEN, bar);	break;	\
	  case hybrid##GREEN_WHITE:	make_hybrid_frame_with_overlay(&CFG(frameset), frame, half##GREEN, half##WHITE, bar);	break;	\
	  case hybrid##WHITE_BLACK:	make_hybrid_frame_with_overlay(&CFG(frameset), frame, half##WHITE, half##BLACK, bar);	break;	\
	  case hybrid##BLACK_GREEN:	make_hybrid_frame_with_overlay(&CFG(frameset), frame, half##BLACK, half##GREEN, bar);	break;	\
	  case hybrid##GREEN_BLUE:	make_hybrid_frame_with_overlay(&CFG(frameset), frame, half##GREEN, half##BLUE, bar);	break;	\
	  case hybrid##BLUE_RED:	make_hybrid_frame_with_overlay(&CFG(frameset), frame, half##BLUE,  half##RED, bar);		break;	\
	  case hybrid##RED_WHITE:	make_hybrid_frame_with_overlay(&CFG(frameset), frame, half##RED,   half##WHITE, bar)

#define MAKE_FRAMESET(frameset, framename, nyx)																												\
		   FRAME_##framename##_SPELL_COLORLESS:																												\
		MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(CARDBK_##frameset##_COLORLESS, CARDOV_##frameset##_COLORLESS, nyx, CARDOV_##frameset##_COLORLESS_NYX);	break;	\
	  case MAKE_MONOCOLORS_WITH_VARYING_BACKGROUND(frameset, FRAME_##framename##_SPELL_, nyx);														break;	\
	  case MAKE_MONOCOLORS(frameset, FRAME_##framename##_SPELL_GOLD_, CARDBK_##frameset##_GOLD, GOLD, nyx, GOLD);									break;	\
	  case MAKE_HYBRIDS_WITH_BAR(frameset, FRAME_##framename##_SPELL_GOLD_, FRAME_##framename##_SPELL_GOLD_, CARDOV_##frameset##_GOLD_BAR);			break;	\
	  case FRAME_##framename##_SPELL_GOLD:																													\
		MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(CARDBK_##frameset##_GOLD, CARDOV_##frameset##_GOLD, nyx, CARDOV_##frameset##_GOLD_NYX);					break;	\
																																							\
	  case FRAME_##framename##_ARTIFACT_COLORLESS:																											\
		MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(CARDBK_##frameset##_ARTIFACT, CARDOV_##frameset##_ARTIFACT, nyx, CARDOV_##frameset##_ARTIFACT_NYX);		break;	\
	  case MAKE_MONOCOLORS(frameset, FRAME_##framename##_ARTIFACT_, CARDBK_##frameset##_ARTIFACT, ARTIFACT, nyx, ARTIFACT);							break;	\
	  case MAKE_MONOCOLORS(frameset, FRAME_##framename##_ARTIFACT_GOLD_, CARDBK_##frameset##_ARTIFACT, GOLD_ARTIFACT, nyx, ARTIFACT);				break;	\
	  case MAKE_HYBRIDS_WITH_BAR(frameset, FRAME_##framename##_ARTIFACT_GOLD_, FRAME_##framename##_ARTIFACT_GOLD_, CARDOV_##frameset##_GOLD_BAR);	break;	\
	  case FRAME_##framename##_ARTIFACT_GOLD:																												\
		MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(CARDBK_##frameset##_ARTIFACT, CARDOV_##frameset##_GOLD_ARTIFACT, nyx, CARDOV_##frameset##_ARTIFACT_NYX);	break;	\
																																							\
	  case FRAME_##framename##_LAND_COLORLESS:																												\
		MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(CARDBK_##frameset##_LAND, CARDOV_##frameset##_COLORLESS_LAND, nyx, CARDOV_##frameset##_LAND_NYX);			break;	\
	  case MAKE_MONOCOLORS(frameset, FRAME_##framename##_LAND_, CARDBK_##frameset##_LAND, LAND, nyx, LAND);											break;	\
	  case MAKE_MONOCOLORS(frameset, FRAME_##framename##_LAND_GOLD_, CARDBK_##frameset##_LAND, LAND, nyx, LAND);									break;	\
	  case MAKE_HYBRIDS_WITH_BAR(frameset, FRAME_##framename##_LAND_GOLD_, FRAME_##framename##_LAND_GOLD_, CARDOV_##frameset##_LAND_BAR);			break;	\
	  case FRAME_##framename##_LAND_GOLD:																													\
		MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(CARDBK_##frameset##_LAND, CARDOV_##frameset##_GOLD_LAND, nyx, CARDOV_##frameset##_LAND_NYX);				break;	\
																																							\
	  case FRAME_##framename##_ARTIFACT_LAND_COLORLESS:																										\
		if (CFG(frameset).frames_artifact_land == 2 || CFG(frameset).frames_artifact_land == 3)																\
		  make_hybrid_frame(&CFG(frameset), frame, FRAME_##framename##_ARTIFACT_COLORLESS, FRAME_##framename##_LAND_COLORLESS);								\
		else																																				\
		  MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX(CARDBK_##frameset##_ARTIFACT, CARDOV_##frameset##_COLORLESS_LAND, nyx, CARDOV_##frameset##_ARTIFACT_NYX);		\
		break;																																				\
	  case MAKE_MONOCOLOR_ARTIFACT_LANDS(frameset, framename);																						break;	\
	  case MAKE_HYBRIDS_WITH_BAR(frameset, FRAME_##framename##_ARTIFACT_LAND_GOLD_, FRAME_##framename##_ARTIFACT_LAND_, CARDOV_##frameset##_LAND_BAR);	break;\
	  case FRAME_##framename##_ARTIFACT_LAND_GOLD_BLACK:	case FRAME_##framename##_ARTIFACT_LAND_GOLD_BLUE:	case FRAME_##framename##_ARTIFACT_LAND_GOLD_GREEN:\
	  case FRAME_##framename##_ARTIFACT_LAND_GOLD_RED:	case FRAME_##framename##_ARTIFACT_LAND_GOLD_WHITE:	abort();	/* unused, directly or indirectly */\
	  case MAKE_ONE_MONOCOLOR_ARTIFACT_LAND(frameset, framename, GOLD_, GOLD);																		break;	\
																																							\
	  case MAKE_HYBRIDS(frameset, FRAME_##framename##_SPELL_HYBRID_, FRAME_##framename##_SPELL_)


	  case MAKE_FRAMESET(MODERN, MODERN, 0);				break;
	  case MAKE_FRAMESET(MODERN, MODERN_NYX, 1);			break;
	  case MAKE_FRAMESET(CLASSIC, CLASSIC, 0);				break;
	  case MAKE_FRAMESET(CLASSIC, CLASSIC_NYX, 1);			break;
	  case MAKE_FRAMESET(TIMESHIFT, TIMESHIFT, 0);			break;
	  case MAKE_FRAMESET(TIMESHIFT, TIMESHIFT_NYX, 1);		break;
	  case MAKE_FRAMESET(FUTURESHIFT, FUTURESHIFT, 0);		break;
	  case MAKE_FRAMESET(FUTURESHIFT, FUTURESHIFT_NYX, 1);	break;
	  case MAKE_FRAMESET(PLANESWALKER, PLANESWALKER, 0);	break;
	  case MAKE_FRAMESET(PLANESWALKER, PLANESWALKER_NYX, 1);break;
	  case MAKE_FRAMESET(TOKEN, TOKEN, 0);					break;
	  case MAKE_FRAMESET(TOKEN, TOKEN_NYX, 1);				break;

#undef CFG
#undef MAKE_FRAME_FROM_OVERLAY_MAYBE_NYX
#undef MAKE_MONOCOLORS_WITH_VARYING_BACKGROUND
#undef MAKE_MONOCOLORS
#undef MAKE_ONE_MONOCOLOR_ARTIFACT_LAND
#undef MAKE_MONOCOLOR_ARTIFACT_LANDS
#undef MAKE_HYBRIDS
#undef MAKE_FRAMESET
	}
}

static void
make_gpic_from_pic(PicHandleNames picnum)
{
  if (!gpics[picnum])
	{
	  BITMAP bmp;
	  GetObject(pics[picnum], sizeof(BITMAP), &bmp);

	  init_gdiplus();

	  GdipCreateBitmapFromScan0(bmp.bmWidth, bmp.bmHeight, bmp.bmWidth * 4, picnum == CARDBACK ? PixelFormat32bppRGB : PixelFormat32bppARGB, bmp.bmBits, &gpics[picnum]);
	}
}

static void
make_frame_from_overlay(PicHandleNames target, PicHandleNames background, PicHandleNames overlay)
{
  make_frame_from_2_overlays(target, background, overlay, PICHANDLE_INVALID);
}

static void
make_frame_from_2_overlays(PicHandleNames target, PicHandleNames background, PicHandleNames overlay1, PicHandleNames overlay2)
{
  if (!gpics[background])
	make_frame(background);
  if (!gpics[overlay1])
	make_frame(overlay1);
  if (overlay2 != PICHANDLE_INVALID && !gpics[overlay2])
	make_frame(overlay2);

  UINT width, height;
  GdipGetImageWidth(gpics[background], &width);
  GdipGetImageHeight(gpics[background], &height);

  GpBitmap* gbmp_target = NULL;
  GdipCloneBitmapAreaI(0, 0, width, height, PixelFormat32bppARGB, gpics[background], &gbmp_target);

  GpGraphics* gfx_target = NULL;
  GdipGetImageGraphicsContext(gbmp_target, &gfx_target);

  GdipSetInterpolationMode(gfx_target, InterpolationModeHighQuality);
  GdipDrawImageI(gfx_target, gpics[overlay1], 0, 0);
  if (overlay2 != PICHANDLE_INVALID)
	GdipDrawImageI(gfx_target, gpics[overlay2], 0, 0);

  GdipDeleteGraphics(gfx_target);

  gpics[target] = gbmp_target;
}

static void
make_hybrid_frame(Config* config, PicHandleNames target, PicHandleNames left, PicHandleNames right)
{
  make_hybrid_frame_with_overlay(config, target, left, right, PICHANDLE_INVALID);
}

static void
make_hybrid_frame_with_overlay(Config* config, PicHandleNames target, PicHandleNames left, PicHandleNames right, PicHandleNames overlay)
{
  if (!gpics[left])
	make_frame(left);
  if (!gpics[right])
	make_frame(right);
  if (overlay != PICHANDLE_INVALID && !gpics[overlay])
	make_frame(overlay);

  UINT lwidth, lheight, rwidth, rheight;
  GdipGetImageWidth(gpics[left], &lwidth);
  GdipGetImageHeight(gpics[left], &lheight);
  GdipGetImageWidth(gpics[right], &rwidth);
  GdipGetImageHeight(gpics[right], &rheight);

  GpBitmap* gbmp_target = NULL;
  GdipCloneBitmapAreaI(0, 0, lwidth, lheight, PixelFormat32bppARGB, gpics[left], &gbmp_target);

  // Add alpha channel
  GpRect r;
  r.X = 0;
  r.Y = 0;
  r.Width = rwidth;
  r.Height = rheight;
  BitmapData right_data;
  GdipBitmapLockBits(gpics[right], &r, ImageLockModeRead, PixelFormat32bppARGB, &right_data);

  // We want the final alpha to be a weighted average of source and target alpha, not a sum, but gdi+ gives no control over the channel!  So have to do it manually. :( :(
  BitmapData target_data;
  GdipBitmapLockBits(gbmp_target, &r, ImageLockModeWrite, PixelFormat32bppARGB, &target_data);

  double percent_solid_on_hybrid = (100 - config->frames_percent_mixed_on_hybrid) / 200.0;
  double m = 255.0 / (rwidth * (1 - 2 * percent_solid_on_hybrid));
  double b = -255.0 * percent_solid_on_hybrid / (1 - 2 * percent_solid_on_hybrid);

  UINT x, y;
  UINT minheight = MIN(right_data.Height, target_data.Height);
  UINT minwidth = MIN(right_data.Width, target_data.Width);
  INT right_stride = abs(right_data.Stride);
  INT target_stride = abs(target_data.Stride);
  for (y = 0; y < minheight; ++y)
	{
	  uint8_t* rightbits = (uint8_t*)right_data.Scan0 + y * right_stride;
	  uint8_t* targetbits = (uint8_t*)target_data.Scan0 + y * target_stride;
	  for (x = 0; x < minwidth; ++x, rightbits += 4, targetbits += 4)
		{
		  double blend = m*x + b;
		  if (blend < 0.0)
			blend = 0;
		  else if (blend > 255.0)
			blend = 255;

#define CLAMP_INTO(to)				\
		  if (k < 0.0)				\
			targetbits[to] = 0;		\
		  else if (k > 255.0)		\
			targetbits[to] = 255;	\
		  else						\
			targetbits[to] = k + 0.5

		  double k;
#define BLT_RGB(col)																								\
		  k = (255 - blend)*targetbits[3]*targetbits[col]/(255*255) + blend*rightbits[3]*rightbits[col]/(255*255);	\
		  CLAMP_INTO(col)

		  BLT_RGB(0);
		  BLT_RGB(1);
		  BLT_RGB(2);
#undef BLT_RGB

		  k = (255 - blend)*targetbits[3]/255 + blend*rightbits[3]/255;
		  CLAMP_INTO(3);
#undef CLAMP
		}
	}

  GdipBitmapUnlockBits(gbmp_target, &target_data);
  GdipBitmapUnlockBits(gpics[right], &right_data);

  if (overlay != PICHANDLE_INVALID)
	{
	  GpGraphics* gfx_target = NULL;
	  GdipGetImageGraphicsContext(gbmp_target, &gfx_target);

	  GdipSetInterpolationMode(gfx_target, InterpolationModeHighQuality);
	  GdipDrawImageI(gfx_target, gpics[overlay], 0, 0);

	  GdipDeleteGraphics(gfx_target);
	}

  gpics[target] = gbmp_target;
}

// Returns 0 if text should use PowertoughnessColor, 1 if should use TitleOnLightColor
static int
blt_powertoughness_box(HDC hdc, const RECT* dest_rect, PicHandleNames frame, int translucent)
{
  PicHandleNames ptbox = FRAMEPART_FROM_BASE(POWERTOUGHNESS_MODERN, cfg);
  if (!hdc || !dest_rect || !pics[ptbox])
	return 0;

  typedef enum
  {
	PTY_WHITE = 0,
	PTY_BLUE,
	PTY_BLACK,
	PTY_RED,
	PTY_GREEN,
	PTY_COLORLESS,
	PTY_GOLD,
	PTY_ARTIFACT
  } PowerToughnessY;

  typedef enum
  {
	PTX_NORMAL = 0,
	PTX_GOLD,
	PTX_DUAL_LAND
  } PowerToughnessX;

  int src_x = PTX_NORMAL, src_y, light = 0;

  // Normalize to modern frame set
  frame = BASE_FRAME(frame);

  if (cfg->fullcard_extended_powertoughness)	// Futureshift logic
	switch (frame)
	  {
		case FRAME_MODERN_SPELL_COLORLESS:
		/* These are impossible, other than CARDOV_*_COLORLESS.  They're included to silence an enumeration value not handled in switch warning; but I don't
		 * want to use a default since I need the warning for FRAME_MODERN_MIN ... FRAME_MODERN_MAX. */
		case 0 ... FRAME_MAX_LOADED:
		case FRAME_MODERN_SPELL_GOLD_BLACK ... FRAME_MODERN_SPELL_GOLD_WHITE:
		case FRAME_MODERN_ARTIFACT_GOLD_BLACK ... FRAME_MODERN_ARTIFACT_GOLD_WHITE:
		case FRAME_MODERN_LAND_GOLD_BLACK ... FRAME_MODERN_LAND_GOLD_WHITE:
		case FRAME_MODERN_ARTIFACT_LAND_GOLD_BLACK ... FRAME_MODERN_ARTIFACT_LAND_GOLD_WHITE:
		case FRAME_MODERN_NYX_MIN ... FRAME_MAX:
		case PICHANDLE_INVALID:
		  src_x = PTX_NORMAL;		src_y = PTY_COLORLESS;	light = 1;	break;

		case FRAME_MODERN_ARTIFACT_COLORLESS:
		  src_x = PTX_NORMAL;		src_y = PTY_ARTIFACT;	light = 1;	break;

		case FRAME_MODERN_LAND_COLORLESS:			case FRAME_MODERN_ARTIFACT_LAND_COLORLESS:
		  src_x = PTX_DUAL_LAND;	src_y = PTY_COLORLESS;	light = 1;	break;

		case FRAME_MODERN_SPELL_BLACK:				case FRAME_MODERN_ARTIFACT_BLACK:
		case FRAME_MODERN_LAND_BLACK:				case FRAME_MODERN_ARTIFACT_LAND_BLACK:
		case FRAME_MODERN_SPELL_HYBRID_WHITE_BLACK:
		case FRAME_MODERN_SPELL_HYBRID_BLUE_BLACK:
		  src_x = PTX_NORMAL;		src_y = PTY_BLACK;		light = 0;	break;

		case FRAME_MODERN_SPELL_BLUE:				case FRAME_MODERN_ARTIFACT_BLUE:
		case FRAME_MODERN_LAND_BLUE:				case FRAME_MODERN_ARTIFACT_LAND_BLUE:
		case FRAME_MODERN_SPELL_HYBRID_GREEN_BLUE:
		case FRAME_MODERN_SPELL_HYBRID_WHITE_BLUE:
		  src_x = PTX_NORMAL;		src_y = PTY_BLUE;		light = 0;	break;

		case FRAME_MODERN_SPELL_GREEN:				case FRAME_MODERN_ARTIFACT_GREEN:
		case FRAME_MODERN_LAND_GREEN:				case FRAME_MODERN_ARTIFACT_LAND_GREEN:
		case FRAME_MODERN_SPELL_HYBRID_BLACK_GREEN:
		case FRAME_MODERN_SPELL_HYBRID_RED_GREEN:
		  src_x = PTX_NORMAL;		src_y = PTY_GREEN;		light = 0;	break;

		case FRAME_MODERN_SPELL_RED:				case FRAME_MODERN_ARTIFACT_RED:
		case FRAME_MODERN_LAND_RED:					case FRAME_MODERN_ARTIFACT_LAND_RED:
		case FRAME_MODERN_SPELL_HYBRID_BLUE_RED:
		case FRAME_MODERN_SPELL_HYBRID_BLACK_RED:
		  src_x = PTX_NORMAL;		src_y = PTY_RED;		light = 0;	break;

		case FRAME_MODERN_SPELL_WHITE:				case FRAME_MODERN_ARTIFACT_WHITE:
		case FRAME_MODERN_LAND_WHITE:				case FRAME_MODERN_ARTIFACT_LAND_WHITE:
		case FRAME_MODERN_SPELL_HYBRID_RED_WHITE:
		case FRAME_MODERN_SPELL_HYBRID_GREEN_WHITE:
		  src_x = PTX_NORMAL;		src_y = PTY_WHITE;		light = 1;	break;

		case FRAME_MODERN_SPELL_GOLD_WHITE_BLUE:	case FRAME_MODERN_ARTIFACT_GOLD_WHITE_BLUE:
		case FRAME_MODERN_SPELL_GOLD_GREEN_BLUE:	case FRAME_MODERN_ARTIFACT_GOLD_GREEN_BLUE:
		  src_x = PTX_GOLD;			src_y = PTY_BLUE;		light = 1;	break;

		case FRAME_MODERN_LAND_GOLD_WHITE_BLUE:		case FRAME_MODERN_ARTIFACT_LAND_GOLD_WHITE_BLUE:
		case FRAME_MODERN_LAND_GOLD_GREEN_BLUE:		case FRAME_MODERN_ARTIFACT_LAND_GOLD_GREEN_BLUE:
		  src_x = PTX_DUAL_LAND;	src_y = PTY_BLUE;		light = 1;	break;

		case FRAME_MODERN_SPELL_GOLD_BLUE_BLACK:	case FRAME_MODERN_ARTIFACT_GOLD_BLUE_BLACK:
		case FRAME_MODERN_SPELL_GOLD_WHITE_BLACK:	case FRAME_MODERN_ARTIFACT_GOLD_WHITE_BLACK:
		  src_x = PTX_GOLD;			src_y = PTY_BLACK;		light = 1;	break;

		case FRAME_MODERN_LAND_GOLD_BLUE_BLACK:		case FRAME_MODERN_ARTIFACT_LAND_GOLD_BLUE_BLACK:
		case FRAME_MODERN_LAND_GOLD_WHITE_BLACK:	case FRAME_MODERN_ARTIFACT_LAND_GOLD_WHITE_BLACK:
		  src_x = PTX_DUAL_LAND;	src_y = PTY_BLACK;		light = 1;	break;

		case FRAME_MODERN_SPELL_GOLD_BLACK_RED:		case FRAME_MODERN_ARTIFACT_GOLD_BLACK_RED:
		case FRAME_MODERN_SPELL_GOLD_BLUE_RED:		case FRAME_MODERN_ARTIFACT_GOLD_BLUE_RED:
		  src_x = PTX_GOLD;			src_y = PTY_RED;		light = 1;	break;

		case FRAME_MODERN_LAND_GOLD_BLACK_RED:		case FRAME_MODERN_ARTIFACT_LAND_GOLD_BLACK_RED:
		case FRAME_MODERN_LAND_GOLD_BLUE_RED:		case FRAME_MODERN_ARTIFACT_LAND_GOLD_BLUE_RED:
		  src_x = PTX_DUAL_LAND;	src_y = PTY_RED;		light = 1;	break;

		case FRAME_MODERN_SPELL_GOLD_RED_GREEN:		case FRAME_MODERN_ARTIFACT_GOLD_RED_GREEN:
		case FRAME_MODERN_SPELL_GOLD_BLACK_GREEN:	case FRAME_MODERN_ARTIFACT_GOLD_BLACK_GREEN:
		  src_x = PTX_GOLD;			src_y = PTY_GREEN;		light = 1;	break;

		case FRAME_MODERN_LAND_GOLD_RED_GREEN:		case FRAME_MODERN_ARTIFACT_LAND_GOLD_RED_GREEN:
		case FRAME_MODERN_LAND_GOLD_BLACK_GREEN:	case FRAME_MODERN_ARTIFACT_LAND_GOLD_BLACK_GREEN:
		  src_x = PTX_DUAL_LAND;	src_y = PTY_GREEN;		light = 1;	break;

		case FRAME_MODERN_SPELL_GOLD_GREEN_WHITE:	case FRAME_MODERN_ARTIFACT_GOLD_GREEN_WHITE:
		case FRAME_MODERN_SPELL_GOLD_RED_WHITE:		case FRAME_MODERN_ARTIFACT_GOLD_RED_WHITE:
		  src_x = PTX_GOLD;			src_y = PTY_WHITE;		light = 1;	break;

		case FRAME_MODERN_LAND_GOLD_GREEN_WHITE:	case FRAME_MODERN_ARTIFACT_LAND_GOLD_GREEN_WHITE:
		case FRAME_MODERN_LAND_GOLD_RED_WHITE:		case FRAME_MODERN_ARTIFACT_LAND_GOLD_RED_WHITE:
		  src_x = PTX_DUAL_LAND;	src_y = PTY_WHITE;		light = 1;	break;

		case FRAME_MODERN_SPELL_GOLD:				case FRAME_MODERN_ARTIFACT_GOLD:
		case FRAME_MODERN_LAND_GOLD:				case FRAME_MODERN_ARTIFACT_LAND_GOLD:
		  src_x = PTX_GOLD;			src_y = PTY_GOLD;		light = 1;	break;
	  }
  else
	switch (frame)
	  {
		case FRAME_MODERN_SPELL_COLORLESS:
		case FRAME_MODERN_LAND_COLORLESS:
		case FRAME_MODERN_ARTIFACT_LAND_COLORLESS:
		case FRAME_MODERN_LAND_GOLD_MIN ... FRAME_MODERN_LAND_GOLD_MAX:
		case FRAME_MODERN_ARTIFACT_LAND_GOLD_MIN ... FRAME_MODERN_ARTIFACT_LAND_GOLD_MAX:
		case FRAME_MODERN_SPELL_HYBRID_MIN ... FRAME_MODERN_SPELL_HYBRID_MAX:
		/* These are impossible, other than CARDOV_*_COLORLESS.  They're included to silence an enumeration value not handled in switch warning; but I don't
		 * want to use a default since I need the warning for FRAME_MODERN_MIN ... FRAME_MODERN_MAX. */
		case 0 ... FRAME_MAX_LOADED:
		case FRAME_MODERN_SPELL_GOLD_BLACK ... FRAME_MODERN_SPELL_GOLD_WHITE:
		case FRAME_MODERN_ARTIFACT_GOLD_BLACK ... FRAME_MODERN_ARTIFACT_GOLD_WHITE:
		case FRAME_MODERN_LAND_GOLD_BLACK ... FRAME_MODERN_LAND_GOLD_WHITE:
		case FRAME_MODERN_ARTIFACT_LAND_GOLD_BLACK ... FRAME_MODERN_ARTIFACT_LAND_GOLD_WHITE:
		case FRAME_MODERN_NYX_MIN ... FRAME_MAX:
		case PICHANDLE_INVALID:
		  src_y = PTY_COLORLESS;
		  break;

		case FRAME_MODERN_SPELL_WHITE:				case FRAME_MODERN_ARTIFACT_WHITE:
		case FRAME_MODERN_LAND_WHITE:				case FRAME_MODERN_ARTIFACT_LAND_WHITE:
		  src_y = PTY_WHITE;
		  break;
		case FRAME_MODERN_SPELL_BLUE:				case FRAME_MODERN_ARTIFACT_BLUE:
		case FRAME_MODERN_LAND_BLUE:				case FRAME_MODERN_ARTIFACT_LAND_BLUE:
		  src_y = PTY_BLUE;
		  break;
		case FRAME_MODERN_SPELL_BLACK:				case FRAME_MODERN_ARTIFACT_BLACK:
		case FRAME_MODERN_LAND_BLACK:				case FRAME_MODERN_ARTIFACT_LAND_BLACK:
		  src_y = PTY_BLACK;
		  break;
		case FRAME_MODERN_SPELL_RED:				case FRAME_MODERN_ARTIFACT_RED:
		case FRAME_MODERN_LAND_RED:					case FRAME_MODERN_ARTIFACT_LAND_RED:
		  src_y = PTY_RED;
		  break;
		case FRAME_MODERN_SPELL_GREEN:				case FRAME_MODERN_ARTIFACT_GREEN:
		case FRAME_MODERN_LAND_GREEN:				case FRAME_MODERN_ARTIFACT_LAND_GREEN:
		  src_y = PTY_GREEN;
		  break;
		case FRAME_MODERN_SPELL_GOLD:				case FRAME_MODERN_ARTIFACT_GOLD:
		case FRAME_MODERN_LAND_GOLD:				case FRAME_MODERN_ARTIFACT_LAND_GOLD:
		case FRAME_MODERN_SPELL_GOLD_MIN ... FRAME_MODERN_SPELL_GOLD_MAX:
		case FRAME_MODERN_ARTIFACT_GOLD_MIN ... FRAME_MODERN_ARTIFACT_GOLD_MAX:
		  src_y = PTY_GOLD;
		  break;
		case FRAME_MODERN_ARTIFACT_COLORLESS:
		  src_y = PTY_ARTIFACT;
		  break;
	  }

  GpGraphics* gfx;

  make_gpic_from_pic(ptbox);
  GdipCreateFromHDC(hdc, &gfx);
  GdipSetInterpolationMode(gfx, InterpolationModeHighQuality);

  UINT width, height;
  GdipGetImageWidth(gpics[ptbox], &width);
  GdipGetImageHeight(gpics[ptbox], &height);
  if (cfg->fullcard_extended_powertoughness)
	width /= 3;
  height /= 8;

  int dest_x = dest_rect->left;
  int dest_y = dest_rect->top;
  int dest_wid = (dest_rect->right >= dest_rect->left) ? (UINT)(dest_rect->right - dest_rect->left) : width;
  int dest_hgt = (dest_rect->bottom >= dest_rect->top) ? (UINT)(dest_rect->bottom - dest_rect->top) : height;

  GdipDrawImageRectRectI(gfx, gpics[ptbox],
						 dest_x, dest_y, dest_wid, dest_hgt,
						 src_x * width, src_y * height, width, height,
						 UnitPixel, translucent ? cfg->colorless_alpha_xform : NULL, NULL, NULL);

  GdipDeleteGraphics(gfx);

  return light;
}

static void
blt_type_icon(HDC hdc, const RECT* dest_rect, const card_ptr_t* cp, PicHandleNames frame)
{
  PicHandleNames iconimg = FRAMEPART_FROM_BASE(TYPE_ICON_MODERN, cfg);

  if (!hdc || !dest_rect || !iconimg)
	return;

  int src_y;

  // Normalize to modern frame set
  PicHandleNames base_frame = BASE_FRAME(frame);

  // Dark image for light backgrounds is 0.  Light image for dark backgrounds is 1.
  // Light backgrounds: artifact, colorless, white, gold.  Dark backgrounds: blue, black, green, red.
  switch (base_frame)
	{
	  case FRAME_MODERN_SPELL_BLACK:	case FRAME_MODERN_SPELL_BLUE:	case FRAME_MODERN_SPELL_GREEN:	case FRAME_MODERN_SPELL_RED:
	  case FRAME_MODERN_LAND_BLACK:		case FRAME_MODERN_LAND_BLUE:	case FRAME_MODERN_LAND_GREEN:	case FRAME_MODERN_LAND_RED:
	  case FRAME_MODERN_LAND_GOLD_BLUE_BLACK:			case FRAME_MODERN_LAND_GOLD_BLACK_RED:
	  case FRAME_MODERN_LAND_GOLD_RED_GREEN:			case FRAME_MODERN_LAND_GOLD_GREEN_WHITE:
	  case FRAME_MODERN_LAND_GOLD_BLACK_GREEN:			case FRAME_MODERN_LAND_GOLD_GREEN_BLUE:
	  case FRAME_MODERN_LAND_GOLD_BLUE_RED:				case FRAME_MODERN_LAND_GOLD_RED_WHITE:
	  case FRAME_MODERN_ARTIFACT_LAND_GOLD_BLUE_BLACK:	case FRAME_MODERN_ARTIFACT_LAND_GOLD_BLACK_RED:
	  case FRAME_MODERN_ARTIFACT_LAND_GOLD_RED_GREEN:	case FRAME_MODERN_ARTIFACT_LAND_GOLD_GREEN_WHITE:
	  case FRAME_MODERN_ARTIFACT_LAND_GOLD_BLACK_GREEN:	case FRAME_MODERN_ARTIFACT_LAND_GOLD_GREEN_BLUE:
	  case FRAME_MODERN_ARTIFACT_LAND_GOLD_BLUE_RED:	case FRAME_MODERN_ARTIFACT_LAND_GOLD_RED_WHITE:
	  case FRAME_MODERN_SPELL_HYBRID_BLUE_BLACK:		case FRAME_MODERN_SPELL_HYBRID_BLACK_RED:
	  case FRAME_MODERN_SPELL_HYBRID_RED_GREEN:			case FRAME_MODERN_SPELL_HYBRID_GREEN_WHITE:
	  case FRAME_MODERN_SPELL_HYBRID_BLACK_GREEN:		case FRAME_MODERN_SPELL_HYBRID_GREEN_BLUE:
	  case FRAME_MODERN_SPELL_HYBRID_BLUE_RED:			case FRAME_MODERN_SPELL_HYBRID_RED_WHITE:
		src_y = 1;
		break;

	  default:
#if 0
	  case FRAME_MODERN_SPELL_COLORLESS:				case FRAME_MODERN_SPELL_WHITE:
	  case FRAME_MODERN_SPELL_GOLD_MIN ... FRAME_MODERN_SPELL_GOLD_MAX:
	  case FRAME_MODERN_SPELL_GOLD:

	  case FRAME_MODERN_ARTIFACT_MIN ... FRAME_MODERN_ARTIFACT_MAX:

	  case FRAME_MODERN_LAND_COLORLESS:					case FRAME_MODERN_LAND_WHITE:
	  case FRAME_MODERN_LAND_GOLD_WHITE_BLUE:			case FRAME_MODERN_LAND_GOLD_WHITE_BLACK:
	  case FRAME_MODERN_LAND_GOLD:

	  case FRAME_MODERN_ARTIFACT_LAND_COLORLESS:		case FRAME_MODERN_ARTIFACT_LAND_WHITE:
	  case FRAME_MODERN_ARTIFACT_LAND_GOLD_WHITE_BLUE:	case FRAME_MODERN_ARTIFACT_LAND_GOLD_WHITE_BLACK:
	  case FRAME_MODERN_ARTIFACT_LAND_GOLD:

	  case FRAME_MODERN_SPELL_HYBRID_WHITE_BLUE:		case FRAME_MODERN_SPELL_HYBRID_WHITE_BLACK:
		// plus CARDOV_MODERN_COLORLESS and CARDOV_MODERN_SPECIAL
#endif
		src_y = 0;
		break;
	}

  int src_x = -1;

  // Find card type(s).  Surprisingly complex.
  typedef enum
  {
	ICON_TYPE_ARTIFACT = 0,
	ICON_TYPE_CREATURE,
	ICON_TYPE_ENCHANTMENT,
	ICON_TYPE_INSTANT,
	ICON_TYPE_LAND,
	ICON_TYPE_MULTIPLE,
	ICON_TYPE_PLANESWALKER,
	ICON_TYPE_SORCERY
  } IconType;

  const card_data_t* cd;

  if (IS_NYX_FRAME(frame, base_frame))	// If a Nyx frame, then we're drawing an enchantment+something else, so multi.
	src_x = ICON_TYPE_MULTIPLE;
  else if (cp->types1 == SUBT_TRIBAL)	// If a Tribal (and they're all in types1 currently), always something else, so multi.
	src_x = ICON_TYPE_MULTIPLE;
  else if ((cd = get_card_data_from_csvid(cp->id)))
	switch (cd->type & (TYPE_PERMANENT | TYPE_SPELL))
	  {
		case 0:	return;
		case TYPE_LAND:			src_x = ICON_TYPE_LAND;		break;
		case TYPE_CREATURE | TYPE_INSTANT:	// creature with flash
		case TYPE_CREATURE:		src_x = ICON_TYPE_CREATURE;	break;
		case TYPE_ENCHANTMENT:	src_x = cp->types1 == SUBT_PLANESWALKER ? ICON_TYPE_PLANESWALKER : ICON_TYPE_ENCHANTMENT;	break;
		case TYPE_SORCERY:		src_x = ICON_TYPE_SORCERY;	break;
		case TYPE_INSTANT | TYPE_INTERRUPT:
		case TYPE_INTERRUPT:
		case TYPE_INSTANT:		src_x = ICON_TYPE_INSTANT;	break;
		case TYPE_ARTIFACT:		src_x = ICON_TYPE_ARTIFACT;	break;
		default:	// two or more bits set
		  src_x = ICON_TYPE_MULTIPLE;
	  }
  else
	{
	  const int16_t* typ;
	  for (typ = &cp->types1; typ <= &cp->types6; ++typ)
		switch (*typ)
		  {
			case SUBT_ARTIFACT:		src_x = src_x != -1 ? ICON_TYPE_MULTIPLE : ICON_TYPE_ARTIFACT;	break;
			case SUBT_CREATURE:		src_x = src_x != -1 ? ICON_TYPE_MULTIPLE : ICON_TYPE_CREATURE;	break;
			case SUBT_ENCHANTMENT:	src_x = src_x != -1 ? ICON_TYPE_MULTIPLE : ICON_TYPE_ENCHANTMENT;	break;
			case SUBT_INSTANT:		src_x = src_x != -1 ? ICON_TYPE_MULTIPLE : ICON_TYPE_INSTANT;	break;
			case SUBT_LAND:			src_x = src_x != -1 ? ICON_TYPE_MULTIPLE : ICON_TYPE_LAND;	break;
			case SUBT_PLANE:
			case SUBT_PLANESWALKER:	src_x = src_x != -1 && src_x != ICON_TYPE_PLANESWALKER ? ICON_TYPE_MULTIPLE : ICON_TYPE_PLANESWALKER;	break;
			case SUBT_SORCERY:		src_x = src_x != -1 ? ICON_TYPE_MULTIPLE : ICON_TYPE_SORCERY;	break;
			case SUBT_TRIBAL:		src_x = ICON_TYPE_MULTIPLE;	break;
		  }
	}

  if (src_x == -1)
	return;

  GpGraphics* gfx;

  make_gpic_from_pic(iconimg);
  GdipCreateFromHDC(hdc, &gfx);
  GdipSetInterpolationMode(gfx, InterpolationModeHighQuality);

  UINT width, height;
  GdipGetImageWidth(gpics[iconimg], &width);
  GdipGetImageHeight(gpics[iconimg], &height);
  width /= 8;
  height /= 2;

  int dest_x = dest_rect->left;
  int dest_y = dest_rect->top;
  int dest_wid = (dest_rect->right >= dest_rect->left) ? (UINT)(dest_rect->right - dest_rect->left) : width;
  int dest_hgt = (dest_rect->bottom >= dest_rect->top) ? (UINT)(dest_rect->bottom - dest_rect->top) : height;

  GdipDrawImageRectRectI(gfx, gpics[iconimg],
						 dest_x, dest_y, dest_wid, dest_hgt,
						 src_x * width, src_y * height, width, height,
						 UnitPixel, NULL, NULL, NULL);

  GdipDeleteGraphics(gfx);
}

static void
gdip_blt_whole(HDC hdc, const RECT* dest_rect, PicHandleNames frame, int translucent)
{
  if (!gpics[frame])
	make_frame(frame);

  // When called through the DrawCardBack() api, cfg won't be set, but neither will translucent.
  if (translucent && cfg->colorless_alpha_xform)
	{
	  UINT width, height;
	  GdipGetImageWidth(gpics[frame], &width);
	  GdipGetImageHeight(gpics[frame], &height);
	  gdip_blt(hdc, dest_rect, frame, 0, 0, width, height, translucent);
	}
  else
	{
	  GpGraphics* gfx = NULL;

	  GdipCreateFromHDC(hdc, &gfx);
	  GdipSetInterpolationMode(gfx, InterpolationModeHighQualityBicubic);

	  GdipDrawImageRectI(gfx, gpics[frame], dest_rect->left, dest_rect->top, dest_rect->right - dest_rect->left, dest_rect->bottom - dest_rect->top);

	  GdipDeleteGraphics(gfx);
	}
}

static void
gdip_blt(HDC hdc, const RECT* dest_rect, PicHandleNames frame, int src_x, int src_y, int width, int height, int translucent)
{
  if (!gpics[frame])
	make_frame(frame);

  GpGraphics* gfx = NULL;

  GdipCreateFromHDC(hdc, &gfx);
  GdipSetInterpolationMode(gfx, InterpolationModeHighQualityBicubic);

  GdipDrawImageRectRectI(gfx, gpics[frame],
						 dest_rect->left, dest_rect->top, dest_rect->right - dest_rect->left, dest_rect->bottom - dest_rect->top,
						 src_x, src_y, width, height,
						 UnitPixel, translucent ? cfg->colorless_alpha_xform : NULL, NULL, NULL);

  GdipDeleteGraphics(gfx);
}

static void
blt_expansion_symbol(HDC hdc, const RECT* dest_rect, EXPANSION_t expansion, rarity_t rarity, int smallcard)
{
#define EXPANSIONS_WIDE	5	// Number of expansions wide (times four rarities)
#define EXPANSIONS_TALL	25	// Number of expansions tall

  if (!hdc || !dest_rect || !pics[EXPANSION_SYMBOLS])
	return;

  int src_x, src_y;
  switch (rarity)
	{
	  case NONE:
		return;

	  case COMMON:
	  case LAND:
		src_x = 0;
		break;

	  case UNCOMMON:
		src_x = 1;
		break;

	  case RARE:
		src_x = 2;
		break;

	  case MYTHIC:
	  case SPECIAL:
	  default:
		src_x = 3;
		break;
	}

  int no_early_core_sets, no_early_rarity, force_rarity;
  if (smallcard)
	{
	  no_early_core_sets = cfg->smallcard_expansion_no_early_core_sets;
	  no_early_rarity = cfg->smallcard_expansion_no_early_rarity;
	  force_rarity = cfg->smallcard_expansion_force_rarity;
	}
  else
	{
	  no_early_core_sets = cfg->fullcard_expansion_no_early_core_sets;
	  no_early_rarity = cfg->fullcard_expansion_no_early_rarity;
	  force_rarity = cfg->fullcard_expansion_force_rarity;
	}

  if (no_early_core_sets
	  && (expansion == EXPANSION_LIMITED_EDITION_ALPHA
		  || expansion == EXPANSION_LIMITED_EDITION_BETA
		  || expansion == EXPANSION_UNLIMITED_EDITION
		  || expansion == EXPANSION_REVISED_EDITION
		  || expansion == EXPANSION_FOURTH_EDITION
		  || expansion == EXPANSION_FIFTH_EDITION))
	return;
  else if (force_rarity >= 1 && force_rarity <= 4)
	src_x = force_rarity - 1;
  else if (no_early_rarity
		   && expansion <= EXPANSION_PORTAL_SECOND_AGE && expansion != EXPANSION_EXODUS)
	src_x = 0;

  src_y = expansion - 1;
  if (src_y < 0 || src_y >= (EXPANSIONS_WIDE * EXPANSIONS_TALL))
	return;

  src_x += 4 * (src_y / EXPANSIONS_TALL);
  src_y %= EXPANSIONS_TALL;

  GpGraphics* gfx;

  make_gpic_from_pic(EXPANSION_SYMBOLS);
  GdipCreateFromHDC(hdc, &gfx);
  GdipSetInterpolationMode(gfx, InterpolationModeHighQualityBicubic);

  UINT width, height;
  GdipGetImageWidth(gpics[EXPANSION_SYMBOLS], &width);
  GdipGetImageHeight(gpics[EXPANSION_SYMBOLS], &height);

  int sym_wid = width / (EXPANSIONS_WIDE * 4);
  int sym_hgt = height / EXPANSIONS_TALL;

  int dest_x = dest_rect->left;
  int dest_y = dest_rect->top;
  int dest_wid = (dest_rect->right >= dest_rect->left) ? dest_rect->right - dest_rect->left : sym_wid;
  int dest_hgt = (dest_rect->bottom >= dest_rect->top) ? dest_rect->bottom - dest_rect->top : sym_hgt;

  /* Gdi+ prefilters an image before scaling it, so even with a fully-transparent borders around each subimage, it'll still sometimes pick up edges from
   * adjacent subimages.  So what I do is first crop the image to a secondary bitmap (which does not interpolate), then blt with interpolation from *that*
   * to the final image. */

  GpBitmap* cropped = NULL;
  GdipCloneBitmapAreaI(src_x * sym_wid, src_y * sym_hgt, sym_wid, sym_hgt, PixelFormat32bppARGB, gpics[EXPANSION_SYMBOLS], &cropped);

  GdipDrawImageRectI(gfx, cropped, dest_x, dest_y, dest_wid, dest_hgt);

  GdipDisposeImage(cropped);
  GdipDeleteGraphics(gfx);
}

static void
draw_mana_symbol(HDC hdc, char c, int x, int y, int wid, int hgt)
{
  if (!hdc)
	return;

  if ((uint8_t)c < (uint8_t)(-MANA_SYMBOLS))
	return;

  make_gpic_from_pic(MANASYMBOLS);

  UINT src_hgt;
  GdipGetImageHeight(gpics[MANASYMBOLS], &src_hgt);

  int src_x = (-(int)c - 1) * src_hgt /*sic*/;
  int src_y = 0;

  if (wid < 0)
	wid = src_hgt /*sic*/;
  if (hgt < 0)
	hgt = src_hgt;

  GpGraphics* gfx;
  GdipCreateFromHDC(hdc, &gfx);
  GdipSetInterpolationMode(gfx, InterpolationModeHighQuality);

  GdipDrawImageRectRectI(gfx, gpics[MANASYMBOLS],
						 x, y, wid, hgt,
						 src_x, src_y, src_hgt /*sic*/ - 1, src_hgt - 1,	// 1 blank pixel for rounding error
						 UnitPixel, NULL, NULL, NULL);

  GdipDeleteGraphics(gfx);
}

int
CalcDrawManaText(HDC hdc, const RECT* rect, const char* str)
{
  EnterCriticalSection(&critical_section);
  int rval = calc_draw_mana_text_impl(hdc, rect, str, 0, 0);
  LeaveCriticalSection(&critical_section);
  return rval;
}

static int
calc_draw_mana_text_impl(HDC hdc, const RECT* rect, const char* str, int rules_text_flag, int no_additional_space)
{
  if (!hdc || !rect || !str)
	return 0;

  int savedc = SaveDC(hdc);

  IntersectClipRect(hdc, 0, 0, 1, 1);
  int rval = draw_mana_text_impl(hdc, rect, str, 0, rules_text_flag, no_additional_space);
  RestoreDC(hdc, savedc);

  return rval;
}

int
DrawManaText(HDC hdc, const RECT* dest_rect, const char* str, int drawflag)
{
  EnterCriticalSection(&critical_section);
  int rval = draw_mana_text_impl(hdc, dest_rect, str, drawflag, 0, 0);
  LeaveCriticalSection(&critical_section);
  return rval;
}

static int
draw_mana_text_impl(HDC hdc, const RECT* dest_rect, const char* str, int drawflag, int rules_text_flag, int no_additional_space)
{
  if (!hdc || !dest_rect || !str || !*str)
	return 0;

  int savedc = SaveDC(hdc);

  TEXTMETRIC metrics;
  GetTextMetrics(hdc, &metrics);

  int font_line_hgt = metrics.tmHeight;

  int drawing_on_fullcard = GetMapMode(hdc) == MM_ISOTROPIC;

  RECT rect;
  int line_hgt;
  rect.bottom = line_hgt = metrics.tmHeight - (drawing_on_fullcard ? 8 : 2);
  rect.top = rect.left = rect.right = 0;
  LPtoDP(hdc, (POINT*)(&rect), 2);

  rect.right = rect.bottom - rect.top;
  rect.bottom = rect.left = rect.top = 0;
  DPtoLP(hdc, (POINT*)(&rect), 2);

  // now a very crude hack to draw proper symbols on a fullcard
  // as GDI+ does not support GDI's map mode and I don't want
  // to mess with transformation matrixes atm -Mok

  int sym_ext_wid, sym_wid, sym_hgt;
  {
	int w = drawing_on_fullcard ? metrics.tmHeight : (rect.right - rect.left);

	sym_hgt = (int64_t)(metrics.tmHeight) * 75 / 100;
	sym_wid = (int64_t)w * 75 / 100;
	sym_ext_wid = (int64_t)w * 85 / 100;
  }

  IntersectClipRect(hdc, dest_rect->left, dest_rect->top, dest_rect->right + 1, dest_rect->bottom);

  GdiFlush();

  SelectObject(hdc, GetStockObject(BLACK_BRUSH));
  SelectObject(hdc, GetStockObject(NULL_PEN));

  char buf[3000];
  SIZE word_wid;
  int posx = dest_rect->left;
  int posy = dest_rect->top;
  int max_wid = 0;
  while (*str)
	{
	  if (*str == ' ')
		{
		  while (*++str == ' ')	{}

		  *buf = ' ';
		  *(buf + 1) = 0;

		  GetTextExtentPoint32(hdc, buf, 1, &word_wid);	// width of space char
		  posx += word_wid.cx;
		  continue;
		}

	  if (*str == '\n')
		{
		  ++str;
		  int h = line_hgt;
		  if (!no_additional_space)
			h += h >> 2;
		  posy += h;
		  posx = dest_rect->left;
		  continue;
		}

	  {
		int buf_len = 0;
		{
		  char* p = buf;
		  int tag;
		  while ((tag = convert_initial_mana_tag(&str)))
			{
			  *p++ = tag;
			  ++buf_len;
			}
		}
		if (buf_len)
		  {
			if (sym_wid * buf_len + posx > dest_rect->right)	// symbols fit into current line?
			  {
				// nope
				int w = posx - dest_rect->left;
				if (w > max_wid)
				  max_wid = w;
				posy += line_hgt;	// move to next line
				posx = dest_rect->left;
			  }

			char* p = buf;
			for (; buf_len; --buf_len)
			  {
				if (drawflag)	// otherwise, calculating size, so no symbols needed
				  draw_mana_symbol(hdc, *p++,
								   ((uint32_t)(sym_ext_wid - sym_wid) >> 1) + posx,
								   ((uint32_t)(font_line_hgt - sym_hgt) >> 1) + posy,
								   sym_wid, sym_hgt);
				posx += sym_ext_wid;
			  }

			if (*str == '.' || *str == ':' || *str == ',')
			  {
				*buf = *str;
				*(buf + 1) = 0;
				++str;

				GetTextExtentPoint32(hdc, buf, 1, &word_wid);
				TextOut(hdc, posx, posy, buf, 1);

				posx += word_wid.cx;
			  }

			continue;
		  }
	  }

	  {
		char* p = buf;
		int buf_len = 0;
		int switch_back = 0;
		while (*str && *str != ' ' && *str != '\n' && *str != '|')
		  {
			if (*str == '(' && rules_text_flag)
			  {
				if (buf_len)
				  break;

				SelectObject(hdc, fonts[FLAVOR_FONT]);
			  }

			char c = *str++;
			if (c == '\r')
			  continue;

			*p++ = c;
			++buf_len;

			if (c == ')' && rules_text_flag)
			  {
				switch_back = 1;	// switch back to normal font after draw
				break;
			  }
		  }
		*p = 0;

		GetTextExtentPoint32(hdc, buf, buf_len, &word_wid);

		if (word_wid.cx + posx > dest_rect->right)
		  {
			if (posx - dest_rect->left > max_wid)
			  max_wid = posx - dest_rect->left;

			posy += line_hgt;
			posx = dest_rect->left;
		  }
		TextOut(hdc, posx, posy, buf, buf_len);

		posx += word_wid.cx;

		if (switch_back)
		  {
			switch_back = 0;
			SelectObject(hdc, fonts[BIGCARDTEXT_FONT]);
		  }
	  }
	}

  if (posx - dest_rect->left > max_wid)
	max_wid = posx - dest_rect->left;

  int max_hgt = line_hgt + posy - dest_rect->top;

  RestoreDC(hdc, savedc);
  return (max_hgt << 16) | (max_wid & 0xffff);
}

static void
close_gdiplus(void)
{
  int i;
  for (i = 0; i < MAX_PICHANDLES_PLUS_1; ++i)
	if (gpics[i])
	  {
		GdipDisposeImage(gpics[i]);
		gpics[i] = NULL;
	  }

  GdiplusShutdown(gdiplus_token);
}

static void
init_gdiplus(void)
{
  if (!gdiplus_token)
	{
	  GdiplusStartupInput input;
	  input.GdiplusVersion = 1;
	  input.DebugEventCallback = NULL;
	  input.SuppressBackgroundThread = 0;
	  input.SuppressExternalCodecs = 0;

	  GdiplusStartup(&gdiplus_token, &input, NULL);

	  int i;
	  for (i = 0; i <= MAX_CFG; ++i)
		create_colorless_alpha_xform(&configs[i]);
	}
}

static void
create_colorless_alpha_xform(Config* config)
{
  if (gdiplus_token)	// otherwise, wait until the call from init_gdiplus()
	{
	  if (config->colorless_alpha_xform)
		GdipDisposeImageAttributes(config->colorless_alpha_xform);

	  if (config->frames_colorless_alpha >= 255)
		config->colorless_alpha_xform = NULL;
	  else
		{
		  GdipCreateImageAttributes(&config->colorless_alpha_xform);
		  int x, y;
		  for (x = 0; x < 5; ++x)
			for (y = 0; y < 5; ++y)
			  config->matrix.m[x][y] = (x == y) ? 1 : 0;
		  config->matrix.m[3][3] = MAX(config->frames_colorless_alpha, 0) / 255.0;
		  GdipSetImageAttributesColorMatrix(config->colorless_alpha_xform, ColorAdjustTypeBitmap, TRUE, &config->matrix, NULL, ColorMatrixFlagsDefault);
		}
	}
}

int
DrawSmallCardTitle(HDC hdc, const RECT* dest_rect, const char* card_title, int color, int controlled_by_owner)
{
  EnterCriticalSection(&critical_section);
#pragma message "This needs multiple injections so we can figure out which config to use."
  int rval = draw_small_card_title_impl(hdc, dest_rect, card_title, color, controlled_by_owner);
  LeaveCriticalSection(&critical_section);
  return rval;
}

static int
draw_small_card_title_impl(HDC hdc, const RECT* dest_rect, const char* card_title, int color, int controlled_by_owner)
{
  if (!hdc || !dest_rect || !card_title)
	return 0;

  int savedc = SaveDC(hdc);
  GdiFlush();

  SetMapMode(hdc,MM_ANISOTROPIC);
  SetWindowExtEx(hdc, 800, 1120, 0);
  SetViewportExtEx(hdc, dest_rect->right - dest_rect->left, dest_rect->bottom - dest_rect->top, NULL);
  SetWindowOrgEx(hdc, 0, 0, NULL);
  SetViewportOrgEx(hdc, dest_rect->left, dest_rect->top, NULL);

  int shadow_color, shadow_visible, background_color, background_visible;
  POINT shadow_offset;

#define SET_COLORS(base) do {						\
	color = base##_color.colorref;					\
	shadow_color = base##_shadow_color.colorref;	\
	shadow_visible = base##_shadow_visible;			\
	shadow_offset = base##_shadow_offset;			\
  } while (0)

  /* configs[CFG_BASE] instead of cfg since cfg won't be set correctly when called through the DrawSmallCardTitle() api -
   * there isn't enough information given to find which card is being drawn. */
  if (controlled_by_owner)
	{
	  background_visible = configs[CFG_BASE].smallcard_title_owned_background_visible;
	  background_color   = configs[CFG_BASE].smallcard_title_owned_background_color.colorref;
	  switch ((int8_t)color)
		{
		  case 0:	SET_COLORS(configs[CFG_BASE].smallcard_title_owned);	break;
		  case 2:	SET_COLORS(configs[CFG_BASE].smallcard_title_owned_must_activate);	break;
		  default:	SET_COLORS(configs[CFG_BASE].smallcard_title_owned_can_activate);	break;
		}
	}
  else
	{
	  background_visible = configs[CFG_BASE].smallcard_title_unowned_background_visible;
	  background_color   = configs[CFG_BASE].smallcard_title_unowned_background_color.colorref;
	  switch ((int8_t)color)
		{
		  case 0:	SET_COLORS(configs[CFG_BASE].smallcard_title_unowned);	break;
		  case 2:	SET_COLORS(configs[CFG_BASE].smallcard_title_unowned_must_activate);	break;
		  default:	SET_COLORS(configs[CFG_BASE].smallcard_title_unowned_can_activate);	break;
		}
	}
#undef SET_COLORS

  SelectObject(hdc, fonts[SMALLCARDTITLE_FONT]);
  SetTextAlign(hdc, TA_LEFT);

  if (background_visible)
	{
	  SetBkMode(hdc, OPAQUE);
	  SetBkColor(hdc, background_color);
	}
  else
	SetBkMode(hdc, TRANSPARENT);

  int len = strlen(card_title);

  if (shadow_visible)
	{
	  SetTextColor(hdc, shadow_color);
	  OffsetRect(&configs[CFG_BASE].smallcard_title, shadow_offset.x, shadow_offset.y);
	  ExtTextOut(hdc, configs[CFG_BASE].smallcard_title.left, configs[CFG_BASE].smallcard_title.top, ETO_CLIPPED, &configs[CFG_BASE].smallcard_title, card_title, len, NULL);
	  OffsetRect(&configs[CFG_BASE].smallcard_title, -shadow_offset.x, -shadow_offset.y);

	  SetBkMode(hdc, TRANSPARENT);
	}

  SetTextColor(hdc, color);
  ExtTextOut(hdc, configs[CFG_BASE].smallcard_title.left, configs[CFG_BASE].smallcard_title.top, ETO_CLIPPED, &configs[CFG_BASE].smallcard_title, card_title, len, NULL);

  return RestoreDC(hdc, savedc);
}

static const card_data_t*
get_card_data_from_csvid(uint32_t csvid)
{
  if (parent != PARENT_MANALINK)
	return NULL;

  static int32_t* csvid_to_iid = NULL;
  static uint32_t max_csvid = 8192;	// immediately doubled to 16384 in first call

  if (!csvid_to_iid || csvid > max_csvid)
	{
	  max_csvid *= 2;
	  csvid_to_iid = (int32_t*)malloc(sizeof(int32_t) * max_csvid);
	  memset(csvid_to_iid, -1, sizeof(int32_t) * max_csvid);

	  int i;
	  for (i = 0; cards_data_exe[i].id != (uint16_t)(-1); ++i)
		csvid_to_iid[cards_data_exe[i].id] = i;
	}

  if (csvid_to_iid[csvid] == -1)
	{
	  csvid_to_iid[csvid] = get_internal_card_id_from_csv_id_exe(csvid);	// Slow, and returns -1 unless a game has been started, but the only way to reliably find dynamically-created card types.
	  if (csvid_to_iid[csvid] == -1)
		csvid_to_iid[csvid] = -2;	// So we don't search again
	}

  if (csvid_to_iid[csvid] == -2)
	return NULL;
  else
	return &cards_data_exe[csvid_to_iid[csvid]];
}

static int
has_graveyard_ability(const card_ptr_t* cp)
{
  const card_data_t* cd = get_card_data_from_csvid(cp->id);
  if (!cd)
	return 0;

  switch (cd->cc[2])
	{
	  case 0:	// Probably a bug that it's cc[2]==0.
		return cp->id == CARD_ID_KROVIKAN_HORROR;
	  case 1:	// Has alternate casting cost
		return cp->id == CARD_ID_HAAKON_STROMGALD_SCOURGE;
	  case 2:	// Continuous responders to EVENT_GRAVEYARD_ABILITY.
		switch (cp->id)
		  {
			// These don't actually have graveyard abilities, but are implemented that way for their "whenever put into a graveyard" abilities
			case CARD_ID_BLIGHTSTEEL_COLOSSUS:
			case CARD_ID_DARKSTEEL_COLOSSUS:
			case CARD_ID_DREAD:
			case CARD_ID_EMRAKUL_THE_AEONS_TORN:
			case CARD_ID_GUILE:
			case CARD_ID_KOZILEK_BUTCHER_OF_TRUTH:
			case CARD_ID_LEGACY_WEAPON:
			case CARD_ID_PROGENITUS:
			case CARD_ID_PURITY:
			case CARD_ID_SERRA_AVATAR:
			case CARD_ID_ULAMOG_THE_INFINITE_GYRE:
			case CARD_ID_VIGOR:
			case CARD_ID_WORLDSPINE_WURM:
			  // And this has a cc[2] with bit 2 set so it gets EVENT_MODIFY_MANA_PROD
			case CARD_ID_FERTILE_GROUND:
			  return 0;
			default:
			  return 1;
		  }
	  case 4:	// Activates from hand, including cycling and bloodrush.
		switch (cp->id)
		  {
			case CARD_ID_ETERNAL_DRAGON:
			case CARD_ID_PYREWILD_SHAMAN:
			case CARD_ID_UNDEAD_GLADIATOR:
			case CARD_ID_VISCERA_DRAGGER:
			  return 1;
			default:
			  return 0;
		  }
	  case 5:	// Reanimates self
		return 1;
	  case 6:	// Dredge
		return 1;
	  case 9:	// Activated responders to EVENT_GRAVEYARD_ABILITY, mostly Flashback and Unearth.  Annoyingly, also includes Planeswalkers.
		return cp->types1 != SUBT_PLANESWALKER;
	  case 10:	// Retrace.  These two are probably bugs.
		return cp->id != CARD_ID_GREAT_SABLE_STAG && cp->id != CARD_ID_TERRA_STOMPER;
	  case 12:	// Storm count
		return cp->id == CARD_ID_VENGEVINE;
	  case 13:	// Trap conditions, including Prowl
		return cp->id == CARD_ID_AUNTIES_SNITCH;
	  case 17:	// Scavenge
		return 1;
	  case 18:	// Returns to play when a creature with cmc >= 6 is played
		return 1;
	  default:
		return 0;
	}
}

static int
draw_fullcard_art(HDC hdc, RECT* rect, uint32_t id, int version)
{
  RECT rect_big_art_pixels;
  CopyRect(&rect_big_art_pixels, &cfg->fullcard_mana);
  LPtoDP(hdc, (POINT*)(&rect_big_art_pixels), 2);

  LoadBigArt(id, version, rect_big_art_pixels.right - rect_big_art_pixels.left, rect_big_art_pixels.bottom - rect_big_art_pixels.top);
  DrawBigArt(hdc, rect, id, version);
  return IsBigArtRightSize(id, version, rect_big_art_pixels.right - rect_big_art_pixels.left, rect_big_art_pixels.bottom - rect_big_art_pixels.top);
}

int
DrawFullCard(HDC hdc, const RECT* dest_rect, const card_ptr_t* cp, int version, int big_art_style, int expand_text_box, const char* illus)
{
  /*
   * big_art_style is 0 or 2 in _BIGCARD_WNDPROC; 2 in
   * _TENTATIVE_DIALOG_MULLIGAN and sub_4842A0; 17 in sub_43DBE0() and
   * sub_43E620() and sub_470F80() and sub_4D1950() and sub_4D1BC0; 18 in
   * DialogFunc() and sub_46A530() and sub_46B950() and sub_4AA090; varies in
   * sub_4D1780 (second-last param, may be 17, 18, 0, or 2)
   */
  (void)big_art_style;
  (void)illus;

  char buf[2000];
  int8_t card_type = cp->card_type;	// deliberate narrowing conversion

  if (!hdc || !dest_rect || !cp)
	return 0;

  EnterCriticalSection(&critical_section);

  if (configs[CFG_BASE].fullcard_reread_config_on_redraw)
	{
	  read_cfg();
	  assign_default_rarities();
	}

  int savedc = SaveDC(hdc);
  SetMapMode(hdc, MM_ISOTROPIC);
  SetWindowExtEx(hdc, 800, 1200, NULL);

  SetViewportExtEx(hdc, dest_rect->right - dest_rect->left,
				   dest_rect->bottom - dest_rect->top, NULL);
  SetWindowOrgEx(hdc, 400, 600, NULL);
  SetViewportOrgEx(hdc, (dest_rect->right + dest_rect->left) / 2,
				   (dest_rect->bottom + dest_rect->top) / 2, NULL);

  const Rarity* r = get_rarity(cp->id);
  PicHandleNames framenum = select_frame(cp, r);

  int colorless_option, cardback, suppress_later_art = 0;
  if (framenum == CARDBACK)
	{
	  colorless_option = cfg->frames_facedown_creature;
	  cardback = 1;
	  framenum = FRAME_MODERN_SPELL_COLORLESS;
	}
  else
	{
	  colorless_option = cfg->frames_translucent_colorless;
	  cardback = 0;
	}

  int big_art_exists = 0, translucent = 0;
#pragma message "suppress for nyx frames"
  if (BASE_FRAME(framenum) == FRAME_MODERN_SPELL_COLORLESS && colorless_option >= 0)
	{
	  if (cardback)
		{
		  make_gpic_from_pic(CARDBACK);
		  gdip_blt_whole(hdc, &cfg->fullcard_frame, CARDBACK, translucent);
		  big_art_exists = 1;
		  suppress_later_art = 1;
		}
	  else
		{
		  big_art_exists = draw_fullcard_art(hdc, &cfg->fullcard_frame, cp->id, version);
		  suppress_later_art = !(colorless_option & FRAMES_TRANSLUCENT_COLORLESS_ALSO_NORMAL_FULLCARD_ART);
		}

	  framenum = FRAMEPART_FROM_BASE(CARDOV_MODERN_COLORLESS, cfg);
	  translucent = 1;
	  gdip_blt_whole(hdc, &cfg->fullcard_frame, framenum, translucent);
	}
  else
	{
	  if (cfg->fullcard_art_first)
		{
		  big_art_exists = draw_fullcard_art(hdc, &cfg->fullcard_art, cp->id, version);
		  suppress_later_art = 1;
		}

	  gdip_blt_whole(hdc, &cfg->fullcard_frame, framenum, translucent);
	}

  if (cfg->fullcard_outline_visible)
	{
	  SelectObject(hdc, GetStockObject(NULL_BRUSH));
	  HPEN pen = CreatePen(PS_SOLID, 0, cfg->fullcard_outline_color.colorref);
	  SelectObject(hdc, pen);
	  Rectangle(hdc, cfg->fullcard_frame.left, cfg->fullcard_frame.top, cfg->fullcard_frame.right, cfg->fullcard_frame.bottom);
	  SelectObject(hdc, GetStockObject(NULL_PEN));
	  if (pen)
		DeleteObject(pen);
	  SelectObject(hdc, GetStockObject(BLACK_BRUSH));
	}

  /*********************************
   * Tombstone and card full title *
   *********************************/
  RECT rect_title;
  CopyRect(&rect_title, &cfg->fullcard_title);

  int tombstone_drawn = 0;
  if (cfg->fullcard_tombstone_visible && has_graveyard_ability(cp))
	{
	  rect_title.left += cfg->fullcard_tombstone_title_offset;
	  gdip_blt_whole(hdc, &cfg->fullcard_tombstone, FRAMEPART_FROM_BASE(TOMBSTONE_MODERN, cfg), 0);
	  tombstone_drawn = 1;
	}

  if (cfg->fullcard_type_icon_visible && !(tombstone_drawn && cfg->fullcard_type_icon_suppressed_by_tombstone))
	blt_type_icon(hdc, &cfg->fullcard_type_icon, cp, framenum);

  int rt = 0;
  if (card_type != CP_TYPE_LAND
	  && card_type != CP_TYPE_TOKEN
	  && card_type != CP_TYPE_SCHEME
	  && card_type != CP_TYPE_PLANE
	  && card_type != CP_TYPE_NONE
	  && cfg->fullcard_curve_mana_diameter <= 0)
	rt = manacost_width(hdc, &cfg->fullcard_mana, cp);
  if (rt > 0)
	rt = cfg->fullcard_mana.right - rt - 6; // wanted space between name and mana cost
  else
	rt = cfg->fullcard_title.right;

  SIZE textsize;
  int len = strlen(cp->full_name);
  int fnt;
  for (fnt = BIGCARDTITLE_FONT; fnt < BIGCARDTITLE_FONT9; ++fnt)
	{
	  SelectObject(hdc, fonts[fnt]);
	  GetTextExtentPoint32(hdc, cp->full_name, len, &textsize);
	  if (rect_title.left + textsize.cx <= rt)
		goto found_bigcardtitle_font;
	}
  SelectObject(hdc, fonts[BIGCARDTITLE_FONT9]);
 found_bigcardtitle_font:

  SetBkMode(hdc, TRANSPARENT);

  // Draw title shadow
  if (cfg->fullcard_title_shadow_visible)
	{
	  SetTextColor(hdc, cfg->fullcard_title_shadow_color.colorref);
	  OffsetRect(&rect_title, cfg->fullcard_title_shadow_offset.x, cfg->fullcard_title_shadow_offset.y);
	  ExtTextOut(hdc, rect_title.left, rect_title.top, ETO_CLIPPED, &rect_title, cp->full_name, len, NULL);
	  OffsetRect(&rect_title, -cfg->fullcard_title_shadow_offset.x, -cfg->fullcard_title_shadow_offset.y);
	}

  // Draw title text
  COLORREF title_text_override_color = (COLORREF)(-1);
  if (cfg->fullcard_title_on_light_visible || cfg->fullcard_title_on_white_black_visible)
	switch (BASE_FRAME(framenum))
	  {
		case FRAME_MODERN_SPELL_HYBRID_WHITE_BLACK:
		  if (cfg->fullcard_title_on_white_black_visible)
			{
			  title_text_override_color = cfg->fullcard_title_on_white_black_color.colorref;
			  break;
			}
		  // otherwise, fall through
		case FRAME_MODERN_LAND_COLORLESS:
		case FRAME_MODERN_LAND_WHITE:
		case FRAME_MODERN_LAND_GOLD_MIN ... FRAME_MODERN_LAND_GOLD_MAX:
		case FRAME_MODERN_LAND_GOLD:

		case FRAME_MODERN_ARTIFACT_LAND_COLORLESS:
		case FRAME_MODERN_ARTIFACT_LAND_WHITE:
		case FRAME_MODERN_ARTIFACT_LAND_GOLD_MIN ... FRAME_MODERN_ARTIFACT_LAND_GOLD_MAX:
		case FRAME_MODERN_ARTIFACT_LAND_GOLD:

		case FRAME_MODERN_ARTIFACT_COLORLESS:
		case FRAME_MODERN_ARTIFACT_WHITE:
		case FRAME_MODERN_ARTIFACT_GOLD_MIN ... FRAME_MODERN_ARTIFACT_GOLD_MAX:
		case FRAME_MODERN_ARTIFACT_GOLD:

		case FRAME_MODERN_SPELL_COLORLESS:
		case FRAME_MODERN_SPELL_WHITE:
		case FRAME_MODERN_SPELL_GOLD_MIN ... FRAME_MODERN_SPELL_GOLD_MAX:
		case FRAME_MODERN_SPELL_GOLD:
		case FRAME_MODERN_SPELL_HYBRID_WHITE_BLUE:
		case FRAME_MODERN_SPELL_HYBRID_RED_WHITE:
		case FRAME_MODERN_SPELL_HYBRID_GREEN_WHITE:
		  if (cfg->fullcard_title_on_light_visible)
			title_text_override_color = cfg->fullcard_title_on_light_color.colorref;
		  break;

		default:
		  if (translucent && cfg->fullcard_title_on_light_visible)	// CARDOV_*_COLORLESS
			title_text_override_color = cfg->fullcard_title_on_light_color.colorref;
		  break;
	  }

  SetTextColor(hdc, title_text_override_color != (COLORREF)(-1) ? title_text_override_color : cfg->fullcard_title_color.colorref);
  ExtTextOut(hdc, rect_title.left, rect_title.top, ETO_CLIPPED, &rect_title, cp->full_name, len, NULL);

  /******************
   * Card type line *
   ******************/
  if (cp->card_type != (uint32_t)(-1) && cp->card_type != CP_TYPE_NONE)	// deliberately non-narrowed
	{
	  strip_hack_and_sleight_symbols(buf, cp->type_text);

	  len = strlen(buf);

	  for (fnt = BIGCARDTYPE_FONT; fnt < BIGCARDTYPE_FONT9; ++fnt)
		{
		  SelectObject(hdc, fonts[fnt]);
		  GetTextExtentPoint32(hdc, buf, len, &textsize);
		  if (cfg->fullcard_type.left + textsize.cx <= cfg->fullcard_type.right)
			goto found_bigcardtype_font;
		}
	  SelectObject(hdc, fonts[BIGCARDTYPE_FONT9]);
	found_bigcardtype_font:

	  // Draw type shadow
	  if (cfg->fullcard_type_shadow_visible)
		{
		  SetTextColor(hdc, cfg->fullcard_type_shadow_color.colorref);
		  OffsetRect(&cfg->fullcard_type, cfg->fullcard_type_shadow_offset.x, cfg->fullcard_type_shadow_offset.y);
		  ExtTextOut(hdc, cfg->fullcard_type.left, cfg->fullcard_type.top, ETO_CLIPPED, &cfg->fullcard_type, buf, len, NULL);
		  OffsetRect(&cfg->fullcard_type, -cfg->fullcard_type_shadow_offset.x, -cfg->fullcard_type_shadow_offset.y);
		}

	  // Draw type text
	  SetTextColor(hdc, title_text_override_color != (COLORREF)(-1) ? title_text_override_color : cfg->fullcard_type_color.colorref);
	  ExtTextOut(hdc, cfg->fullcard_type.left, cfg->fullcard_type.top, ETO_CLIPPED, &cfg->fullcard_type, buf, len, NULL);
	}

  /********************
   * Expansion symbol *
   ********************/
  if (cfg->fullcard_expansion_visible && r)
	blt_expansion_symbol(hdc, &cfg->fullcard_expansion, r->default_expansion, r->default_rarity, 0);

  /*********************
   * Casting cost line *
   *********************/
  if (card_type != CP_TYPE_LAND
	  && card_type != CP_TYPE_TOKEN
	  && card_type != CP_TYPE_SCHEME
	  && card_type != CP_TYPE_PLANE
	  && card_type != CP_TYPE_NONE)
	draw_manacost(hdc, &cfg->fullcard_mana, cp);

  /***************
   * Big picture *
   ***************/
  if (!suppress_later_art)
	big_art_exists = draw_fullcard_art(hdc, &cfg->fullcard_art, cp->id, version);

  if (cfg->fullcard_artoutline_visible)
	{
	  SelectObject(hdc, GetStockObject(NULL_BRUSH));
	  HPEN pen = CreatePen(PS_SOLID, 0, cfg->fullcard_artoutline_color.colorref);
	  SelectObject(hdc, pen);
	  Rectangle(hdc, cfg->fullcard_art.left, cfg->fullcard_art.top, cfg->fullcard_art.right, cfg->fullcard_art.bottom);
	  if (pen)
		DeleteObject(pen);
	}

  /************************************
   * Rules and flavor text; Watermark *
   ************************************/
  strip_hack_and_sleight_symbols(buf, cp->rules_text);

  RECT rect_rulestext, rect_rulesbox;
  CopyRect(&rect_rulestext, &cfg->fullcard_rulestext);
  CopyRect(&rect_rulesbox, &cfg->fullcard_rulesbox);

  COLORREF rulestext_color = cfg->fullcard_rulestext_color.colorref;
  if (cfg->fullcard_rules_on_dark_land_visible || cfg->fullcard_rules_on_white_black_land_visible)
	switch (BASE_FRAME(framenum))
	  {
		case FRAME_MODERN_LAND_GOLD_WHITE_BLACK:	case FRAME_MODERN_ARTIFACT_LAND_GOLD_WHITE_BLACK:
		  if (cfg->fullcard_rules_on_white_black_land_visible)
			{
			  rulestext_color = cfg->fullcard_rules_on_white_black_land_color.colorref;
			  break;
			}
		  // otherwise, fall through
		case FRAME_MODERN_LAND_BLACK:				case FRAME_MODERN_ARTIFACT_LAND_BLACK:
		case FRAME_MODERN_LAND_BLUE:				case FRAME_MODERN_ARTIFACT_LAND_BLUE:
		case FRAME_MODERN_LAND_GREEN:				case FRAME_MODERN_ARTIFACT_LAND_GREEN:
		case FRAME_MODERN_LAND_RED:					case FRAME_MODERN_ARTIFACT_LAND_RED:
		case FRAME_MODERN_LAND_GOLD_BLUE_BLACK:		case FRAME_MODERN_ARTIFACT_LAND_GOLD_BLUE_BLACK:
		case FRAME_MODERN_LAND_GOLD_BLACK_RED:		case FRAME_MODERN_ARTIFACT_LAND_GOLD_BLACK_RED:
		case FRAME_MODERN_LAND_GOLD_RED_GREEN:		case FRAME_MODERN_ARTIFACT_LAND_GOLD_RED_GREEN:
		case FRAME_MODERN_LAND_GOLD_BLACK_GREEN:	case FRAME_MODERN_ARTIFACT_LAND_GOLD_BLACK_GREEN:
		case FRAME_MODERN_LAND_GOLD_GREEN_BLUE:		case FRAME_MODERN_ARTIFACT_LAND_GOLD_GREEN_BLUE:
		case FRAME_MODERN_LAND_GOLD_BLUE_RED:		case FRAME_MODERN_ARTIFACT_LAND_GOLD_BLUE_RED:
		  if (cfg->fullcard_rules_on_dark_land_visible)
			rulestext_color = cfg->fullcard_rules_on_dark_land_color.colorref;
		  break;
	  }
  SetTextColor(hdc, rulestext_color);

  if (expand_text_box)
	{
	  int txt_hgt = 0;

	  if (*buf)
		{
		  SelectObject(hdc, fonts[BIGCARDTEXT_FONT]);

		  TEXTMETRIC metrics;
		  GetTextMetrics(hdc, &metrics);

		  txt_hgt += HIWORD(calc_draw_mana_text_impl(hdc, &rect_rulestext, buf, 1, 0)) + metrics.tmHeight / 2;
		}

	  if (*cp->flavor_text)
		{
		  SelectObject(hdc, fonts[FLAVOR_FONT]);

		  txt_hgt += HIWORD(calc_draw_mana_text_impl(hdc, &rect_rulestext, cp->flavor_text, 0, 1));
		}

	  if (rect_rulestext.bottom - rect_rulestext.top < txt_hgt)
		{
		  int diff = rect_rulestext.top - rect_rulesbox.top;
		  rect_rulestext.top = rect_rulestext.bottom - txt_hgt;
		  rect_rulesbox.top = rect_rulestext.top - diff;

		  UINT width, height;
		  GdipGetImageWidth(gpics[framenum], &width);
		  GdipGetImageHeight(gpics[framenum], &height);

		  // Blit from full width to full width with clipping instead of just to rect_rulesbox to avoid artifacts from the horrid nearest-neighbor filtering.
		  int savedc2 = SaveDC(hdc);
		  IntersectClipRect(hdc, rect_rulesbox.left, rect_rulesbox.top, rect_rulesbox.right, rect_rulesbox.bottom);
		  rect_rulesbox.left = cfg->fullcard_frame.left;
		  rect_rulesbox.right = cfg->fullcard_frame.right;
		  gdip_blt(hdc, &rect_rulesbox, framenum,
				   0, (height * cfg->fullcard_expand_top / 10000),
				   width, (height * (cfg->fullcard_expand_height) / 10000), translucent);
		  RestoreDC(hdc, savedc2);
		}
	  // else fits, no need to expand
	}

  RECT rect_flavor_text;
  CopyRect(&rect_flavor_text, &rect_rulestext);

  // Watermark - draw after calculating text height and maybe expanding textbox, but before actually drawing text
  if (r && r->watermark <= WATERMARK_MAX
	  && (cfg->fullcard_watermark_outside_set
		  || r->watermark <= WATERMARK_WHITE
		  || r->default_expansion == EXPANSION_RAVNICA_CITY_OF_GUILDS
		  || r->default_expansion == EXPANSION_GUILDPACT
		  || r->default_expansion == EXPANSION_DISSENSION
		  || r->default_expansion == EXPANSION_SCARS_OF_MIRRODIN
		  || r->default_expansion == EXPANSION_MIRRODIN_BESIEGED
		  || r->default_expansion == EXPANSION_NEW_PHYREXIA
		  || r->default_expansion == EXPANSION_DUEL_DECKS_IZZET_VS_GOLGARI
		  || r->default_expansion == EXPANSION_RETURN_TO_RAVNICA
		  || r->default_expansion == EXPANSION_GATECRASH
		  || r->default_expansion == EXPANSION_DRAGONS_MAZE
		  || r->default_expansion == EXPANSION_MODERN_MASTERS))
	{
	  RECT rect_watermark;
	  CopyRect(&rect_watermark, &cfg->fullcard_watermark);
	  if (rect_rulestext.top < cfg->fullcard_rulestext.top)	// i.e., it was expanded
		{
		  int delta = (cfg->fullcard_rulestext.top - rect_rulestext.top) / 2;
		  rect_watermark.top -= delta;
		  rect_watermark.bottom -= delta;
		}

	  int x, y;
	  x = (r->watermark % 5) * watermark_size_x;
	  y = (r->watermark / 5) * watermark_size_y;
	  gdip_blt(hdc, &rect_watermark, WATERMARKS, x, y, watermark_size_x, watermark_size_y, 0);
	}

  if (*buf)
	{
	  SelectObject(hdc, fonts[BIGCARDTEXT_FONT]);

	  TEXTMETRIC metrics;
	  GetTextMetrics(hdc, &metrics);

	  rect_flavor_text.top += HIWORD(draw_mana_text_impl(hdc, &rect_rulestext, buf, 1, 1, 0)) + metrics.tmHeight / 3;
	}

  SelectObject(hdc, fonts[FLAVOR_FONT]);

  if (*cp->flavor_text)
	draw_mana_text_impl(hdc, &rect_flavor_text, cp->flavor_text, 1, 0, 1);

  /*******************
   * Power/Toughness *
   *******************/
  SelectObject(hdc, fonts[BIGCARDPT_FONT]);
  if (hack_power || hack_toughness
	  || card_type == CP_TYPE_CREATURE || card_type == CP_TYPE_TOKEN
	  || (card_type == CP_TYPE_ARTIFACT
		  && (cp->types1 == SUBT_CREATURE
			  || cp->types2 == SUBT_CREATURE
			  || cp->types3 == SUBT_CREATURE
			  || cp->types4 == SUBT_CREATURE
			  || cp->types5 == SUBT_CREATURE
			  || cp->types6 == SUBT_CREATURE
			  || cp->types7 == SUBT_CREATURE)))
	{
	  COLORREF pt_color = cfg->fullcard_powertoughness_color.colorref;
	  if (cfg->fullcard_powertoughness_box_visible
		  && blt_powertoughness_box(hdc, &cfg->fullcard_powertoughness_box, framenum, translucent)
		  && cfg->fullcard_title_on_light_visible)
		pt_color = cfg->fullcard_title_on_light_color.colorref;

	  *buf = 0;

	  if (hack_power < 100)
		sprintf(buf, "%d/", hack_power);
	  else if (hack_power == 100)
		strcpy(buf, "*/");
	  else if (hack_power < 200)
		sprintf(buf, "%d+*/", hack_power - 100);
	  else // hack_power >= 200
		sprintf(buf, "%d-*/", hack_power - 200);

	  char* p = buf + strlen(buf);

	  // Toughness
	  if (hack_toughness < 100)
		sprintf(p, "%d", hack_toughness);
	  else if (hack_toughness == 100)
		strcpy(p, "*");
	  else if (hack_toughness < 200)
		sprintf(p, "%d+*", hack_toughness - 100);
	  else // hack_toughness >= 200
		sprintf(p, "%d-*", hack_toughness - 200);

	  // display p/t line

	  int pos = cfg->fullcard_powertoughness_center ? (cfg->fullcard_powertoughness.right + cfg->fullcard_powertoughness.left) / 2 : cfg->fullcard_powertoughness.right;
	  SetTextAlign(hdc, cfg->fullcard_powertoughness_center ? TA_CENTER : TA_RIGHT);
	  len = strlen(buf);
	  // Draw powertoughness shadow
	  if (cfg->fullcard_powertoughness_shadow_visible)
		{
		  SetTextColor(hdc, cfg->fullcard_powertoughness_shadow_color.colorref);
		  OffsetRect(&cfg->fullcard_powertoughness, cfg->fullcard_powertoughness_shadow_offset.x, cfg->fullcard_powertoughness_shadow_offset.y);
		  ExtTextOut(hdc, pos + cfg->fullcard_powertoughness_shadow_offset.x, cfg->fullcard_powertoughness.top, ETO_CLIPPED, &cfg->fullcard_powertoughness, buf, len, NULL);
		  OffsetRect(&cfg->fullcard_powertoughness, -cfg->fullcard_powertoughness_shadow_offset.x, -cfg->fullcard_powertoughness_shadow_offset.y);
		}

	  // Draw powertoughness text
	  SetTextColor(hdc, pt_color);
	  ExtTextOut(hdc, pos, cfg->fullcard_powertoughness.top, ETO_CLIPPED, &cfg->fullcard_powertoughness, buf, len, NULL);

	  SetTextAlign(hdc, TA_LEFT);
	}

  // Make very, very sure that that the edges are even, even if there are multiple visible frame elements abutting one (as in the futureshifted frames)
  ExcludeClipRect(hdc, cfg->fullcard_frame.left+2, cfg->fullcard_frame.top+2, cfg->fullcard_frame.right-2, cfg->fullcard_frame.bottom-2);
  {
	HBRUSH solid_brush = CreateSolidBrush(cfg->fullcard_border_color.colorref);
	SelectObject(hdc, solid_brush);
	SelectObject(hdc, GetStockObject(NULL_PEN));

	RoundRect(hdc, 0, 0, 800, 1200, cfg->fullcard_rounding.x, cfg->fullcard_rounding.y);

	SelectObject(hdc, GetStockObject(BLACK_BRUSH));

	if (solid_brush)
	  DeleteObject(solid_brush);
  }

  RestoreDC(hdc, savedc);
  LeaveCriticalSection(&critical_section);
  return big_art_exists;
}

static void
strip_hack_and_sleight_symbols(char* dest, const char* src)
{
  while ((*dest = *src))
	{
	  if (*src == '|'
		  && (*(src + 1) == 'H' || *(src + 1) == 'S'))
		src += (*(src + 2) >= '1' && *(src + 2) <= '4') ? 3 : 2;
	  else
		{
		  ++dest;
		  ++src;
		}
	}
}

static int
manacost_width(HDC hdc, const RECT* dest_rect, const card_ptr_t* cp)
{
  char cost_text[104];

  if (!hdc || !dest_rect || !cp)
	return 0;

  int len;
  if (*(const int32_t*)(&cp->unknown0x51[3]))
	{
	  len = 0;
	  const char* q = *(const char* const *)(&cp->unknown0x51[3]);
	  while (*q)
		if (*q++ == '|')
		  ++len;
	}
  else
	{
	  // old style calculated from card_ptr_t values
	  cost_values_to_cost_text(cp, cost_text);
	  len = strlen(cost_text) >> 1;
	}
  //continue:
  len *= (dest_rect->bottom - dest_rect->top);
  return len;
}

static void
draw_manacost(HDC hdc, RECT* dest_rect, const card_ptr_t* cp)
{
  char buf[104];
  const char* p;

  if (!hdc || !dest_rect || !cp)
	return;

  int len;
  if (*(const int32_t*)(&cp->unknown0x51[3]))
	{
	  len = 0;
	  const char* q = p = *(const char* const *)(&cp->unknown0x51[3]);
	  while (*q)
		if (*q++ == '|')
		  ++len;
	}
  else
	{
	  // old style calculated from card_ptr_t values
	  cost_values_to_cost_text(cp, buf);
	  len = strlen(buf) >> 1;
	  p = buf;
	}

  if (cfg->fullcard_curve_mana_diameter > 0)
	{
	  // Ignore dest_rect, use cfg->fullcard_curve_mana[] instead
	  int diam = cfg->fullcard_curve_mana_diameter;
	  int rad = diam / 2;
	  int pos = 0;
	  while (*p && pos < FULLCARD_CURVE_MANA_MAX)
		{
		  int8_t tag = convert_initial_mana_tag(&p);
		  if (!tag)
			++p;
		  else
			{
			  draw_mana_symbol(hdc, tag, cfg->fullcard_curve_mana[pos].x - rad, cfg->fullcard_curve_mana[pos].y - rad, diam, diam);
			  ++pos;
			}
		}
	}
  else
	{
	  int width_height = dest_rect->bottom - dest_rect->top;	// mana symbols have same width/height
	  int posx = dest_rect->right - (len * width_height);
	  int posy = dest_rect->top;

	  while (*p)
		{
		  int8_t tag = convert_initial_mana_tag(&p);
		  if (!tag)
			++p;
		  else
			{
			  draw_mana_symbol(hdc, tag, posx, posy, width_height, width_height);
			  posx += width_height;
			}
		}
	}
}

int
DrawCardBack(HDC hdc, const RECT* dest_rect)
{
  EnterCriticalSection(&critical_section);
  RECT pic_rect;
  int borderx, bordery;

  if (!hdc || !dest_rect)
	return 0;

  borderx = (dest_rect->right - dest_rect->left) * 3 / 100;
  if (borderx < 1)
	borderx = 1;

  bordery = (dest_rect->bottom - dest_rect->top) / 50;
  if (bordery < 1)
	bordery = 1;

  FillRect(hdc, dest_rect, GetStockObject(BLACK_BRUSH));

  SetRect(&pic_rect,
		  dest_rect->left + borderx, dest_rect->top + bordery,
		  dest_rect->right - borderx, dest_rect->bottom - bordery);

  gdip_blt_whole(hdc, &pic_rect, CARDBACK, 0);
  LeaveCriticalSection(&critical_section);
  return 1;
}

static void
draw_smallcard_frame(HDC hdc, const RECT* dest_rect, PicHandleNames framenum, int translucent)
{
  RECT card_rect;

  if (!hdc || !dest_rect)
	return;

  UINT width;
  GdipGetImageWidth(gpics[framenum], &width);

  SetRect(&card_rect, 0, 0, 800, cfg->smallcard_title_dest_height);

  gdip_blt(hdc, &card_rect, framenum,
		   0, cfg->smallcard_title_source_top,
		   width, cfg->smallcard_title_source_height, translucent);

  SetRect(&card_rect, 0, cfg->smallcard_frame_dest_top, 800, 1120);

  gdip_blt(hdc, &card_rect, framenum,
		   0, cfg->smallcard_frame_source_top,
		   width, cfg->smallcard_frame_source_height, translucent);
}

static void
del_obj(HGDIOBJ* obj)
{
  if (obj)
	{
	  DeleteObject(*obj);
	  *obj = NULL;
	}
}

static void
rem_font(const char* fontname)
{
  char buf[2 * MAX_PATH + 1];

  strcpy(buf, base_dir);
  strcat(buf, fontname);
  RemoveFontResource(buf);
}

int
DestroyAllCardBackgrounds(void)
{
  // Obsolete.  Card backgrounds are freed on dll unload.
  return 1;
}

static const Rarity*
get_rarity(int id)
{
  if (!rarities_read)
	read_rarities();

  if (!rarities || id < 0 || id >= total_cards)
	return NULL;

  if (rarities[id]->default_expansion == 0
	  && id > 0
	  && id < CARD_ID_RULES_ENGINE	// so not a token, vanguard avatar, etc.
	  && id != CARD_ID_FACE_DOWN_CREATURE)
	{
	  int candidate = id == CARD_ID_ASSEMBLY_WORKER ? CARD_ID_MISHRAS_FACTORY : id - 1;
	  if (rarities[candidate]->default_expansion != 0)
		id = candidate;
	}

  return rarities[id];
}

static void
read_rarities(void)
{
#define BAIL(...) do { popup("drawcardlib.dll", __VA_ARGS__); return; } while (0)
  rarities_read = 1;

  FILE* rarity_dat;
  if (!(rarity_dat = fopen("rarity.dat", "rb")))
	BAIL("Could not open rarity.dat: %s", strerror(errno));

  if (fread(&total_cards, 4, 1, rarity_dat) != 1)
	BAIL("Could not read number of cards: %s", strerror(errno));

  if (parent == PARENT_MANALINK
	  && total_cards != EXE_DWORD(0x7375ac)/*available_slots*/)
	BAIL("Mismatch between rarity.dat (%d) and cards.dat (%d)", total_cards, EXE_DWORD(0x7375ac));

  if (fread(&raw_expansions, 4, 1, rarity_dat) != 1)
	BAIL("Could not read number of expansions: %s", strerror(errno));

  int expansion_size = (raw_expansions * 3 + 7) / 8;

  uint8_t* raw_rarities;

  if (!(raw_rarities = (uint8_t*)calloc(expansion_size, total_cards)))
	BAIL("Could not calloc uint8_t* %d, %d", expansion_size, total_cards);

  int count;
  if ((count = fread(raw_rarities, expansion_size, total_cards, rarity_dat)) != total_cards)
	BAIL("Could only read %d entries, expected %d: %s", count, total_cards, strerror(errno));

  if (fclose(rarity_dat))
	BAIL("Could not close rarity.dat: %s", strerror(errno));

  if (!(rarities = (Rarity**)malloc(total_cards * sizeof(Rarity**))))
	BAIL("Could not malloc Rarity** %d", total_cards * sizeof(Rarity**));

  int card;
  for (card = 0; card < total_cards; ++card)
	{
	  Rarity* r;
	  if (!(r = (Rarity*)malloc(sizeof(Rarity) + raw_expansions + 2)))	// This is wider than it needs to be because of FAKE_EXPANSIONS, but that's ok.
		{
		  rarities = NULL;
		  BAIL("Could not malloc Rarity* %d", sizeof(Rarity) + raw_expansions + 2);
		}

	  rarities[card] = r;

	  int raw_exp;
	  for (raw_exp = FAKE_EXPANSIONS; raw_exp <= raw_expansions; ++raw_exp)
		{
		  int bit_pos = raw_exp * 3;
		  int bit_mask = 0x07 << (bit_pos & 0x07);
		  int offset = card * expansion_size + bit_pos / 8;

		  int exp = raw_exp - FAKE_EXPANSIONS + 1;

		  r->rarity_in_expansion[exp] = ((((uint16_t)(raw_rarities[offset + 1]) << 8) | raw_rarities[offset]) & bit_mask) >> (bit_pos & 0x07);
		}

	  r->flags = 0;
	  r->watermark = 0xFF;
	}
  free(raw_rarities);

  assign_default_rarities();

  read_watermarks();

#undef BAIL
}

static void
assign_default_rarities(void)
{
  int card;
  for (card = 0; card < total_cards; ++card)
	{
	  Rarity* r = rarities[card];

	  r->default_expansion = 0;
	  r->default_rarity = NONE;

	  int raw_exp, this_priority, best_priority = expansions_priority[EXPCAT_NONE];

	  for (raw_exp = FAKE_EXPANSIONS; raw_exp <= raw_expansions; ++raw_exp)
		{
		  int exp = raw_exp - FAKE_EXPANSIONS + 1;

		  if (r->rarity_in_expansion[exp] != NONE
			  && (this_priority = expansions_priority[expansion_categories[exp]]) >= best_priority
			  && (this_priority > best_priority		// always use if we've found a new best priority
				  || expansions_latest_printing))	// also use if we've only tied our previous best priority, but using the latest printing option
			{
			  best_priority = this_priority;

			  r->default_rarity = r->rarity_in_expansion[exp];
			  r->default_expansion = exp;
			}
		}
	}
}

static void
read_watermarks(void)
{
#define BAIL(...) do { popup("drawcardlib.dll", __VA_ARGS__); return; } while (0)
  FILE* watermarks_csv;
  if (!(watermarks_csv = fopen("watermarks.csv", "r")))
	BAIL("Could not open watermarks.csv: %s", strerror(errno));

  char line[1024], wmark[1024];
  int id = 0, linenum = 0;
  while (fgets(line, 1024, watermarks_csv))
	{
	  ++linenum;
	  if (linenum == 1 && !strncasecmp(line, "id;", 3))
		continue;

	  char* semi = strchr(line, ';');
	  if (!semi)
		BAIL("Could not find first ; in line #%d of watermarks.csv [%s]", linenum, line);

	  id = atoi(line);
	  if (id == 0 && !strncmp(line, "0;", 2))
		BAIL("Id not a number in line #%d of watermarks.csv [%s]", linenum, line);
	  if (id < 0)
		break;

	  if (!(rarities && id >= 0 && id < total_cards))
		BAIL("Unknown id %d in line #%d of watermarks.csv [%s]", id, linenum, line);

	  Rarity* r = rarities[id];

	  ++semi;
	  strcpy(wmark, semi);
	  semi = strchr(wmark, ';');
	  if (semi)
		*semi = '\0';

	  if (!strcasecmp(wmark, "mirran"))			r->watermark = WATERMARK_MIRRAN;
	  else if (!strcasecmp(wmark, "phyrexian"))	r->watermark = WATERMARK_PHYREXIAN;
	  else if (!strcasecmp(wmark, "dimir"))		r->watermark = WATERMARK_DIMIR;
	  else if (!strcasecmp(wmark, "simic"))		r->watermark = WATERMARK_SIMIC;
	  else if (!strcasecmp(wmark, "selesnya"))	r->watermark = WATERMARK_SELESNYA;
	  else if (!strcasecmp(wmark, "rakdos"))	r->watermark = WATERMARK_RAKDOS;
	  else if (!strcasecmp(wmark, "golgari"))	r->watermark = WATERMARK_GOLGARI;
	  else if (!strcasecmp(wmark, "boros"))		r->watermark = WATERMARK_BOROS;
	  else if (!strcasecmp(wmark, "gruul"))		r->watermark = WATERMARK_GRUUL;
	  else if (!strcasecmp(wmark, "izzet"))		r->watermark = WATERMARK_IZZET;
	  else if (!strcasecmp(wmark, "orzhov"))	r->watermark = WATERMARK_ORZHOV;
	  else if (!strcasecmp(wmark, "azorius"))	r->watermark = WATERMARK_AZORIUS;
	  else if (!strcasecmp(wmark, "timeshift"))	r->flags |= RARITYFLAG_TIMESHIFT;
	  else if (!strcasecmp(wmark, "nyx"))		r->flags |= RARITYFLAG_NYX;
	  else if (!strcasecmp(wmark, "black"))		r->watermark = WATERMARK_BLACK;
	  else if (!strcasecmp(wmark, "blue"))		r->watermark = WATERMARK_BLUE;
	  else if (!strcasecmp(wmark, "green"))		r->watermark = WATERMARK_GREEN;
	  else if (!strcasecmp(wmark, "red"))		r->watermark = WATERMARK_RED;
	  else if (!strcasecmp(wmark, "white"))		r->watermark = WATERMARK_WHITE;
	  else
		BAIL("Unknown watermark \"%s\" in line #%d of watermarks.csv [%s]", wmark, linenum, line);
	}
  if (id >= 0 && !feof(watermarks_csv))
	BAIL("Could not read from watermarks.csv: %s", strerror(errno));

  if (fclose(watermarks_csv))
	BAIL("Could not close watermarks.csv: %s", strerror(errno));

#undef BAIL
}
